Class CCRA.Process.ReferralDataExtraction Extends Ens.BusinessProcess [ ClassType = persistent ]
{

Method OnRequest(pRequest As Ens.Request, Output pResponse As Ens.Response) As %Status
{

	#dim tSC As %Status = $$$OK
	#dim e As %Exception.AbstractException
	
	try {
	
	if pRequest.%ClassName(0)="ReferralListRequest" {
		
		$$$ThrowOnError(..GetReferralList(pRequest,.pResponse))
		
	} elseif pRequest.%ClassName(0)="GetReferralByIdRequest" {
		
		$$$ThrowOnError(..GetReferral(pRequest,.pResponse))
	}
	
	} catch e {
	
		set tSC = e.AsStatus()
			
	}
	
	quit tSC
}

Method GetReferralList(pRequest As CCRA.Message.ReferralListRequest, pResponse As CCRA.Message.ReferralListResponse) As %Status
{
	#dim tSC As %Status = $$$OK
	#dim e As %Exception.AbstractException
	#dim tResponse As CCRA.Message.ReferralListResponse = ##class(CCRA.Message.ReferralListResponse).%New()
	#dim tCCRAReferralList As CCRA.ReferralList
	#dim tCCRAReferralListItem As CCRA.ReferralListItem
	
	try {
	
		//Get List of unprocessed Referrals for the date range specified in the request message
		//For each unprocessed Referral in the CCRS Referral Index change process date $zts
		set tStartDate = pRequest.RequestStartDate
		set tEndDate = pRequest.RequestEndDate
		
		set tProcessDate = $zdt($h,3)
		$$$LOGINFO("Received Referral List Request - "_tProcessDate)		
		&sql(UPDATE Region_USXX_HSRM.CCRS 
				SET Processed = 0
				WHERE Processed IS NULL)
		
		//Get Resultset containing list of Referral Id's to process
		set tReferralsToProcess = ##class(%ResultSet).%New("%DynamicQuery:SQL")
		set tReferralsToProcess.RuntimeMode = 0
		
		set tSQL = "SELECT r.WLNO, wl.WL_RowId As WLRowId, wl.WL_ReferralUniqueID As WLReferralUniqueId,r.DateAdded"
		set tSQL = tSQL _ " FROM Region_USXX_HSRM.CCRS AS r"
		set tSQL = tSQL _ " JOIN SQLUser.PA_WaitingList AS wl ON r.WLNO = wl.WL_NO"
		set tSQL = tSQL _ " WHERE Processed = 0"
		
		$$$ThrowOnError(tReferralsToProcess.Prepare(tSQL))
		$$$ThrowOnError(tReferralsToProcess.Execute())

		//Instantiate Referral List
		set tCCRAReferralList = ##class(CCRA.ReferralList).%New()
		
		//For each Referral in the resultset
		// 	1. Instantiate CCRA Referral
		//	2. Add to CCRA Referral List
		while tReferralsToProcess.Next() {
			
			set tCCRAReferralListItem = ##class(CCRA.ReferralListItem).%New()
			if '(tCCRAReferralListItem=$$$NULLOREF) { 
				set tCCRAReferralListItem.WLNO = tReferralsToProcess.Get("WLNO")
				set tCCRAReferralListItem.WLRowId = tReferralsToProcess.Get("WLRowId")
				set tCCRAReferralListItem.WLReferralUniqueId = tReferralsToProcess.Get("WLReferralUniqueId")
				set tCCRAReferralListItem.DateAdded = tReferralsToProcess.Get("DateAdded")
				$$$ThrowOnError(tCCRAReferralList.Referrals.Insert(tCCRAReferralListItem))
			}
			
		}

		$$$LOGINFO("Completed Referral List Request. " _ tCCRAReferralList.Referrals.Count() _ " Referrals sent.")
	
		set pResponse = ##class(CCRA.Message.ReferralListResponse).%New()
		set pResponse.ReferralList = tCCRAReferralList
		
		&sql(UPDATE Region_USXX_HSRM.CCRS 
				SET Processed = 1 
				WHERE (Processed = 0))
		
	} catch e {
		
		set tSC = e.AsStatus()
		
	}
	
	quit tSC
}

Method GetReferral(pRequest As CCRA.Message.GetReferralByIdRequest, pResponse As CCRA.Message.GetReferralByIdResponse) As %Status
{
	#dim tSC As %Status = $$$OK
	#dim e As %Exception.AbstractException
	#dim tHSRMReferral As User.PAWaitingList
	#dim tCCRAReferral As CCRA.Referral = ""
	
	try {
		
		set pResponse = ##class(CCRA.Message.GetReferralByIdResponse).%New()
		
		if pRequest.Mode = 0 { ;use row Id
		
			set tHSRMReferral = ##class(User.PAWaitingList).%OpenId(pRequest.ReferralId)
			
		} elseif pRequest.Mode=1 { ;use HSRM Referral Id
			
			set tWLNO = pRequest.ReferralId
			set tWLRowId = ""
			
			&sql(SELECT WL_RowId INTO :tWLRowId
					FROM SQLUser.PA_WaitingList
					WHERE WL_NO = :tWLNO)
			
			
			if '(tWLRowId="") {
				set tHSRMReferral = ##class(User.PAWaitingList).%OpenId(tWLRowId)
			}
			
		} elseif pRequest.Mode=2 { ;use Referral Unique Id (Consult Id)
			
			set tWLReferralUniqueId = pRequest.ReferralId
			set tWLRowId = ""
			
			&sql(SELECT WL_RowId INTO :tWLRowId
					FROM SQLUser.PA_WaitingList
					WHERE WL_ReferralUniqueId = :tWLReferralUniqueId)
					
			If '(tWLRowId="") {
				set tHSRMReferral = ##class(User.PAWaitingList).%OpenId(tWLRowId)
			}
			
		}
		
		if '(tHSRMReferral=$$$NULLOREF) {
			$$$ThrowOnError(##class(CCRA.Transform.HSRMReferralToCCRAReferral).Transform(tHSRMReferral,.tCCRAReferral))
		
			if '(tCCRAReferral=$$$NULLOREF) {
				set pResponse.Referral = tCCRAReferral
			}
		}
		
	} catch e {
		
		set tSC = e.AsStatus()
		
	}
	
	quit tSC
}

Storage Default
{
<Type>%Library.CacheStorage</Type>
}

}
