Class CCRA.Referral Extends (%Persistent, %Populate, %XML.Adaptor)
{

Property Patient As CCRA.Patient;

Property Diagnosis As %String(MAXLEN = "", POPSPEC = "ValueList(""|Type 1 diabetes mellitus|Pressure ulcer of right elbow|Disp fx of 1st metatarsal bone, l ft, 7thG"")");

// 06-05-2018 MKC

// Rational Jazz Task #750981 - added Diagnosis code

Property DiagnosisCode As %String(MAXLEN = "");

Property MedicalNecessity As %String(MAXLEN = "");

Property EnrollmentStatus As %String(POPSPEC = "ValueList("",Enrolled,Non-Enrolled"")");

Property Payor As %String(POPSPEC = "Company()");

// 06-06-2018 MKC

// Rational Jazz Task #752492 - added Payor Status 

Property PayorStatus As %String(MAXLEN = "");

Property Plan As %String(POPSPEC = "ValueList("",Plan A,Plan B,Plan C"")");

// 06-07-2018 MKC

// Rational Jazz Story #742981 - Add Primary Care Provider NPI to data extraction

Property PrimaryCareProviderNPI As %String(MAXLEN = "");

// MM 10/04/2018 Add Primary Care Provider Name

Property PrimaryCareProviderName As %String(MAXLEN = "");

Property TreatingProvider As %String(MAXLEN = "", POPSPEC = "ValueList("",ABBEVILLE AREA MEDICAL CENTER,AU MEDICAL CENTER,MADISON HOSPITAL,TUFTS MEDICAL CENTER"")");

Property TreatingProviderNPI As %String(MAXLEN = "");

Property ReferralCategory As %String(MAXLEN = "", POPSPEC = "ValueList("",Inpatient,Outpatient"")");

Property ReferralDate As %String(MAXLEN = "", POPORDER = 2, POPSPEC = "CDate(3,$h-10,$h)");

Property ReferralExpirationDate As %String(MAXLEN = "", POPORDER = 3, POPSPEC = ".ExpirationDate()");

Property ReferralLastUpdateDate As %String(MAXLEN = "", POPSPEC = "CDate(3,$h-2,$h)");

Property ReferralNumber As %String(MAXLEN = "", POPSPEC = "##class(CCRS.Populate.Utils).ReferralNumber()");

Property ReferralConsultId As %String(MAXLEN = "");

Property ReferralPeriod As %String(MAXLEN = "", POPORDER = 1, POPSPEC = "ValueList("",7,14,30,60,90,120,180,365"")");

Property ReferringFacility As %String(MAXLEN = "", POPSPEC = "ValueList("",VA Central Western Massachusetts Healthcare System,ADVANCED SURGICAL HOSPITAL,VA Butler Healthcare"")");

Property ReferringProvider As %String(MAXLEN = "", POPSPEC = "Name()");

Property ReferralReturnReason As %String(MAXLEN = "", POPSPEC = "ValueList("",Already Appointed,Duplicate,Care Scheduled - Veteran cancels,Missing VA Date"")");

Property ReferringSpecialty As %String(MAXLEN = "", POPSPEC = "ValueList("",Agencies Home Health,Behavioral Health & Social Service Providers Counselor School,Chiropractic Providers Chiropractor"")");

Property SourceOfReferral As %String(MAXLEN = "", POPSPEC = "##class(CCRS.Populate.Utils).SourceOfReferral()");

Property Status As %String(MAXLEN = "", POPSPEC = "ValueList("",Accepted,Admitted,Approved,Bill Paid,Cancelled,Initial"")");

Property ServiceRequested As %String(MAXLEN = "", POPSPEC = "ValueList(""|VENOGRAPHY, EPIDURAL, RADIOLOGICAL S&I|ANGIOGRAPHY, ADRENAL, UNILAT, SELECTIVE, RADIOLOGICAL S&I|CARDIAC MRI, MORPHOLOGY AND FUNCTION W/O &W CONTRAST MATL & FURTHER SEQ"")");

Property SecondaryPayor As %String(POPSPEC = "Company()");

Property SecondaryPlan As %String(POPSPEC = "ValueList("",Plan A,Plan B,Plan C"")");

Property TreatingFacility As %String(MAXLEN = "", POPSPEC = "ValueList("",MADISON HOSPITAL,VA Ann Arbor Healthcare System,VA Butler Healthcare"")");

Property TreatingSpeciality As %String(MAXLEN = "", POPSPEC = "ValueList("",Agencies Home Health,Behavioral Health & Social Service Providers,Allopathic & Osteopathic Physicians Psychiatry & Neurology Vascular Neurology"")");

Property CoPaymentStatus As CCRA.ES.CoPaymentStatus;

Property Network As %String;

Property NetworkCode As %String;

// JLM 10-11-18 Adding Network Code

Property CLINCode As %String(MAXLEN = "", POPSPEC = "ValueList("",CLINCode1,CLINCode2,CLINCode3"")");

Property CLINDesc As %String(MAXLEN = "", POPSPEC = "ValueList("",CLINCodeDescription"")");

Property PaymentAuthorityCode As %String(MAXLEN = "", POPSPEC = "ValueList("",PaymentAuthorityCode1,PaymentAuthorityCode2,PaymentAuthorityCode3"")");

Property PaymentAuthorityDesc As %String(MAXLEN = "", POPSPEC = "ValueList("",PaymentAuthorityCodeDescription"")");

Relationship PreAuthorizations As CCRA.PreAuthorization [ Cardinality = children, Inverse = Referral ];

Property MilitarySexualTraumaInfo As CCRA.ES.MilitarySexualTraumaInfo;

Property NoseThroatRadiumInfo As CCRA.ES.NoseThroatRadiumInfo;

Relationship TransactionHistory As CCRA.TransactionHistory [ Cardinality = children, Inverse = Referral ];

Property TreatmentStartDate As %Date;

Property PreferredProvider As %String;

// OHI Info

// MKC 06-06-2018

// Jazz Task# 752166 - Adding OHI data elements to HSRM data extraction

Property OHI As list Of CCRA.OHI;

Property OrderingOfficerName As %String;

// BEGIN: eCAMS fields

// JLM 07-11-18

// Jazz Story #772894 - eCAMS add additional required fields to CCRA data transformation

Property CategoryOfCare As %String(MAXLEN = 220);

Property StationID As %String;

Property Sta6 As %String;

// 8/13/18  JLM added per eCAMS team request

Property EstimatedCostOfCare As %String;

Property SeocID As %String(MAXLEN = 220);

Property Comments As %String(MAXLEN = "");

Property ReferringProviderNPI As %String;

Property TreatingFacilityTaxID As %String;

Property TreatingProviderTaxID As %String;

Property NoteOnAppointmentScreen As %String;

Property CommentOnReferralScreen As %String;

// END: eCAMS fields

// 10/22/18 JLM New TriWest Fields

Property PriorityCode As %String;

Property PriorityDescription As %String;

// Property AppointmentTime as %String;

Property AppointmentDate As %String;

Property Appointments As list Of CCRA.Appointment;

Property ClinicallyIndicatedDate As %String;

Property TreatingSpecialityCode As %String;

Property AffilitationDesc As %String;

Property AffilitationCode As %String;

Property TreatingFacilityAddress As CCRA.Address;

Property TreatingFacilityPhone As %String;

Property TreatingFacilityFax As %String;

// END: TriWest fields

Method ExpirationDate() As %String
{
	set tDays = $p(i%ReferralPeriod," ",1)
	set tDate = $zdh(i%ReferralDate,3)+tDays
	quit ##class(%Library.PopulateUtils).CDate(3,tDate,tDate)
}

Storage Default
{
<Data name="ReferralDefaultData">
<Value name="1">
<Value>%%CLASSNAME</Value>
</Value>
<Value name="2">
<Value>Patient</Value>
</Value>
<Value name="3">
<Value>Diagnosis</Value>
</Value>
<Value name="4">
<Value>DiagnosisCode</Value>
</Value>
<Value name="5">
<Value>MedicalNecessity</Value>
</Value>
<Value name="6">
<Value>EnrollmentStatus</Value>
</Value>
<Value name="7">
<Value>Payor</Value>
</Value>
<Value name="8">
<Value>PayorStatus</Value>
</Value>
<Value name="9">
<Value>Plan</Value>
</Value>
<Value name="10">
<Value>PrimaryCareProviderNPI</Value>
</Value>
<Value name="11">
<Value>TreatingProvider</Value>
</Value>
<Value name="12">
<Value>TreatingProviderNPI</Value>
</Value>
<Value name="13">
<Value>ReferralCategory</Value>
</Value>
<Value name="14">
<Value>ReferralDate</Value>
</Value>
<Value name="15">
<Value>ReferralExpirationDate</Value>
</Value>
<Value name="16">
<Value>ReferralLastUpdateDate</Value>
</Value>
<Value name="17">
<Value>ReferralNumber</Value>
</Value>
<Value name="18">
<Value>ReferralConsultId</Value>
</Value>
<Value name="19">
<Value>ReferralPeriod</Value>
</Value>
<Value name="20">
<Value>ReferringFacility</Value>
</Value>
<Value name="21">
<Value>ReferringProvider</Value>
</Value>
<Value name="22">
<Value>ReferralReturnReason</Value>
</Value>
<Value name="23">
<Value>ReferringSpecialty</Value>
</Value>
<Value name="24">
<Value>SourceOfReferral</Value>
</Value>
<Value name="25">
<Value>Status</Value>
</Value>
<Value name="26">
<Value>ServiceRequested</Value>
</Value>
<Value name="27">
<Value>SecondaryPayor</Value>
</Value>
<Value name="28">
<Value>SecondaryPlan</Value>
</Value>
<Value name="29">
<Value>TreatingFacility</Value>
</Value>
<Value name="30">
<Value>TreatingSpeciality</Value>
</Value>
<Value name="31">
<Value>CoPaymentStatus</Value>
</Value>
<Value name="32">
<Value>Network</Value>
</Value>
<Value name="33">
<Value>CLINCode</Value>
</Value>
<Value name="34">
<Value>CLINDesc</Value>
</Value>
<Value name="35">
<Value>PaymentAuthorityCode</Value>
</Value>
<Value name="36">
<Value>PaymentAuthorityDesc</Value>
</Value>
<Value name="37">
<Value>MilitarySexualTraumaInfo</Value>
</Value>
<Value name="38">
<Value>NoseThroatRadiumInfo</Value>
</Value>
<Value name="39">
<Value>AppointmentDate</Value>
</Value>
<Value name="40">
<Value>TreatmentStartDate</Value>
</Value>
<Value name="41">
<Value>PreferredProvider</Value>
</Value>
<Value name="42">
<Value>OHI</Value>
</Value>
<Value name="43">
<Value>OrderingOfficerName</Value>
</Value>
<Value name="44">
<Value>CategoryOfCare</Value>
</Value>
<Value name="45">
<Value>StationID</Value>
</Value>
<Value name="46">
<Value>EstimatedCostOfCare</Value>
</Value>
<Value name="47">
<Value>SeocID</Value>
</Value>
<Value name="48">
<Value>Comments</Value>
</Value>
<Value name="49">
<Value>ReferringProviderNPI</Value>
</Value>
<Value name="50">
<Value>TreatingFacilityTaxID</Value>
</Value>
<Value name="51">
<Value>TreatingProviderTaxID</Value>
</Value>
<Value name="52">
<Value>NoteOnAppointmentScreen</Value>
</Value>
<Value name="53">
<Value>CommentOnReferralScreen</Value>
</Value>
<Value name="54">
<Value>Sta6</Value>
</Value>
<Value name="55">
<Value>PrimaryCareProviderName</Value>
</Value>
<Value name="56">
<Value>NetworkCode</Value>
</Value>
<Value name="57">
<Value>Priority</Value>
</Value>
<Value name="58">
<Value>PriorityCode</Value>
</Value>
<Value name="59">
<Value>PriorityDescription</Value>
</Value>
<Value name="60">
<Value>AppointmentTime</Value>
</Value>
<Value name="61">
<Value>Appointments</Value>
</Value>
<Value name="62">
<Value>ClinicallyIndicatedDate</Value>
</Value>
<Value name="63">
<Value>SpecialtyCode</Value>
</Value>
<Value name="64">
<Value>TreatingSpecialityCode</Value>
</Value>
<Value name="65">
<Value>Affilitation</Value>
</Value>
<Value name="66">
<Value>AffilitationDesc</Value>
</Value>
<Value name="67">
<Value>AffilitationCode</Value>
</Value>
<Value name="68">
<Value>TreatingFacilityAddress</Value>
</Value>
<Value name="69">
<Value>TreatingFacilityPhone</Value>
</Value>
<Value name="70">
<Value>TreatingFacilityFax</Value>
</Value>
</Data>
<DataLocation>^CCRA.ReferralD</DataLocation>
<DefaultData>ReferralDefaultData</DefaultData>
<IdLocation>^CCRA.ReferralD</IdLocation>
<IndexLocation>^CCRA.ReferralI</IndexLocation>
<StreamLocation>^CCRA.ReferralS</StreamLocation>
<Type>%Library.CacheStorage</Type>
}

}
