Class CCRA.Referral.DataExtraction.Process Extends (Ens.BusinessProcess, HS.Util.Trace.Helper) [ ClassType = persistent ]
{

Property HSRMTarget As Ens.DataType.ConfigName [ InitialExpression = "HSRM.ReferralDataExtraction.WS.Client" ];

Property DASTarget As Ens.DataType.ConfigName [ InitialExpression = "DAS.ReferralList.Process" ];

Property CCRSTarget As Ens.DataType.ConfigName [ InitialExpression = "CCRS.ReferralList.Process" ];

Property ReferralIdMode As %Integer [ InitialExpression = 0 ];

Parameter SETTINGS = "HSRMTarget,DASTarget,CCRSTarget";

Method OnRequest(pRequest As CCRA.Message.ReferralDataExtractionRequest, Output pResponse As Ens.Response) As %Status
{
 
 
	//Baseline Check-In 03/19/2018
	#dim tSC As %Status = $$$OK
	#dim e As %Exception.AbstractException
	#dim tHSRMReferralListRequest As CCRA.Message.RetrieveReferralListFromCCRSIndexRequest
	#dim tHSRMReferralListResponse As CCRA.Message.RetrieveReferralListFromCCRSIndexResponse = ""
	#dim tHSRMReferralList As CCRA.WSClient.s0.ReferralList
	#dim tRetrieveReferralResponse AS CCRA.Message.RetrieveReferralResponse
	#dim tXMLReader As %XML.Reader
	#dim tReferralList As %ListOfObjects
	#dim tNetworkReferralList As DAS.CCNReferralList
	#dim tNetworkReferralLists As %ArrayOfObjects
	
	try {
		
		//Get Referrals to process from HSRM
		set tHSRMReferralListRequest = ##class(CCRA.Message.RetrieveReferralListFromCCRSIndexRequest).%New()
		set tHSRMReferralListRequest.StartDate = pRequest.RetrievalDate
		set tHSRMReferralListRequest.EndDate = pRequest.RetrievalDate+1
		
		//DEBUG ONLY
		//set tHSRMReferralListResponse = ##class(CCRA.Message.RetrieveReferralListFromCCRSIndexResponse).%OpenId(63)
		$$$ThrowOnError(..SendRequestSync(..HSRMTarget,tHSRMReferralListRequest,.tHSRMReferralListResponse))
		
		if '(tHSRMReferralListResponse.RetrieveRefrrlLstFrmCCRSndxRslt=$$$NULLOREF) {
		
			set tHSRMReferralList = tHSRMReferralListResponse.RetrieveRefrrlLstFrmCCRSndxRslt
			
			//Walk-thru results and request Referral Data for each referral
			//Add the referral to the Referral List
			if tHSRMReferralList.Referrals.Count() > 0 {
				
				set tReferralsToProcess = ##class(%ListOfObjects).%New()				
				for i=1:1:tHSRMReferralList.Referrals.Count() {					
					set tRetrieveReferralRequest = ##class(CCRA.Message.RetrieveReferralRequest).%New()
					set tRetrieveReferralRequest.Id = tHSRMReferralList.Referrals.GetAt(i).WLRowId
					set tRetrieveReferralRequest.Mode = ..ReferralIdMode
					
					$$$ThrowOnError(..SendRequestSync(..HSRMTarget,tRetrieveReferralRequest,.tRetrieveReferralResponse))
					
					//Transform referral received from HSRM to local referral
					if '(tRetrieveReferralResponse=$$$NULLOREF)	{
						$$$ThrowOnError(##class(CCRA.Referral.Transform.CCRAWSResponseToCCRAReferral).Transform(tRetrieveReferralResponse.RetrieveReferralResult,.tCCRAReferral))
						$$$ThrowOnError(tReferralsToProcess.Insert(tCCRAReferral))
					}
				}
				
				//At this point, a %ListOfObjects (CCRA.Referrals)  
				//Send the list of referrals to both the CCRS and DAS processes
				set tProcessReferralListRequest = ##class(CCRA.Message.ProcessReferralListRequest).%New()
				set tProcessReferralListRequest.Referrals = ##class(%ListOfObjects).%New()
				for i=1:1:tReferralsToProcess.Count() {
					set tReferral = tReferralsToProcess.GetAt(i)
					d tProcessReferralListRequest.Referrals.Insert(tReferral)
				}
				
				$$$ThrowOnError(..SendRequestAsync(..CCRSTarget,tProcessReferralListRequest))
				//$$$ThrowOnError(..SendRequestAsync(..DASTarget,tProcessReferralListRequest)) switching to real time process for DAS			
			}		
		} 
		else {
			$$$HSTRACE("After HSRM Soap Call","No Data to Process.")
		}
		 
		
		
	} catch e {
		
		set tSC = e.AsStatus()
		
	}
	
	Quit tSC
}

Method OutputReferralList(ByRef pReferralList As DAS.CCNReferralList) As %Status
{
	#dim tSC As %Status
	#dim e As %Exception.AbstractException
	#dim tReferralList As %Stream.GlobalCharacter
	
	try {
		
		set tReferralList = ##class(%Stream.GlobalCharacter).%New()
		$$$ThrowOnError(pReferralList.XMLExportToStream(.tReferralList))
		
		//TODO: Send to DAS Transport Operation
		
	} catch e {
		
		set tSC = e.AsStatus()
	
	}
	
	quit tSC
}

/// Handle a 'Response'
Method OnResponse(request As %Library.Persistent, ByRef response As %Library.Persistent, callrequest As %Library.Persistent, callresponse As %Library.Persistent, pCompletionKey As %String) As %Status
{
	Quit $$$OK
}

Storage Default
{
<Data name="ProcessDefaultData">
<Subscript>"Process"</Subscript>
<Value name="1">
<Value>HSRMTarget</Value>
</Value>
<Value name="2">
<Value>DASOperation</Value>
</Value>
<Value name="3">
<Value>CCNList</Value>
</Value>
<Value name="4">
<Value>ReferralIdMode</Value>
</Value>
<Value name="5">
<Value>TraceOperations</Value>
</Value>
<Value name="6">
<Value>DASTarget</Value>
</Value>
<Value name="7">
<Value>CCRSTarget</Value>
</Value>
</Data>
<DefaultData>ProcessDefaultData</DefaultData>
<Type>%Library.CacheStorage</Type>
}

}
