/// Transform response from the CCRA WS into a local CCRA Referral.
Class CCRA.Referral.Transform.CCRAWSResponseToCCRAReferral Extends Ens.DataTransform
{

ClassMethod Transform(pSource As CCRA.WS.Client.s0.Referral, pTarget As CCRA.Referral) As %Status
{
	#dim tSC As %Status = $$$OK
	#dim e As %Exception.AbstractException
	#dim tTransHistory As CCRA.TransactionHistory
	#dim tTransaction As CCRA.TransactionHistory
	#dim tSourceTransaction As CCRA.WS.Client.s0.TransactionHistory
	#dim tSourcePreAuthorization As CCRA.WS.Client.s0.PreAuthorization
	#dim tSourcePreAuthorizationDetail As CCRA.WS.Client.s0.PreAuthorizationDetail
	#dim tSourcePreAuthorizationOrderSet As CCRA.WS.Client.s0.PreAuthorizationOrderSet
	#dim tSourcePreAuthorizationOrderSetDate As CCRA.WS.Client.s0.PreAuthorizationOrderSetDate
	#dim tSourcePreAuthorizationOrderSetDateItem As CCRA.WS.Client.s0.PreAuthorizationOrderSetDateItem
	#dim tPreAuthorization As CCRA.PreAuthorization
	#dim tPreAuthorizationDetail As CCRA.PreAuthorizationDetail
	#dim tPreAuthorizationOrderSet As CCRA.PreAuthorizationOrderSet
	#dim tPreAuthorizationOrderSetDate As CCRA.PreAuthorizationOrderSetDate
	#dim tPreAuthorizationOrderSetDateItem As CCRA.PreAuthorizationOrderSetDateItem
	
	try {
		
		set pTarget = ##class(CCRA.Referral).%New()
		
		//Patient Demographics
		set pTarget.Patient = ##class(CCRA.Patient).%New()
		set pTarget.Patient.ICN = pSource.Patient.ICN
		set pTarget.Patient.LastName = pSource.Patient.LastName
		set pTarget.Patient.FirstName = pSource.Patient.FirstName
		set pTarget.Patient.MName = pSource.Patient.MName
		set pTarget.Patient.Gender = pSource.Patient.Gender
		set pTarget.Patient.DOB = pSource.Patient.DOB
		set pTarget.Patient.HomeAddress.Address1 = pSource.Patient.HomeAddress.Address1
		set pTarget.Patient.HomeAddress.Address2 = pSource.Patient.HomeAddress.Address2
		set pTarget.Patient.HomeAddress.City = pSource.Patient.HomeAddress.City
		set pTarget.Patient.HomeAddress.State = pSource.Patient.HomeAddress.State
		set pTarget.Patient.HomeAddress.ZipCode = pSource.Patient.HomeAddress.ZipCode
		set pTarget.Patient.HomeAddress.Country = pSource.Patient.HomeAddress.Country
		set pTarget.Patient.SSN = pSource.Patient.SSN
		
		//Referral Info
		set pTarget.Diagnosis = pSource.Diagnosis
		// 06-05-2018 MKC
		// Rational Jazz Task #750981 - added Diagnosis code
		set pTarget.DiagnosisCode = pSource.DiagnosisCode
		set pTarget.ReferralNumber = pSource.ReferralNumber
		set pTarget.ReferralConsultId = pSource.ReferralConsultId
		set pTarget.ReferralDate = pSource.ReferralDate
		set pTarget.ReferralExpirationDate = pSource.ReferralExpirationDate
		set pTarget.ReferralPeriod = pSource.ReferralPeriod
		set pTarget.ReferralLastUpdateDate = pSource.ReferralLastUpdateDate
		set pTarget.ReferralCategory = pSource.ReferralCategory
		set pTarget.MedicalNecessity = pSource.MedicalNecessity
		set pTarget.SourceOfReferral = pSource.SourceOfReferral
		set pTarget.Status = pSource.Status
		set pTarget.ReferralReturnReason = pSource.ReferralReturnReason
		set pTarget.ServiceRequested = pSource.ServiceRequested
		set pTarget.ReferringFacility = pSource.ReferringFacility
		set pTarget.ReferringProvider = pSource.ReferringProvider
		set pTarget.ReferringSpecialty = pSource.ReferringSpecialty
		set pTarget.TreatingFacility = pSource.TreatingFacility
		set pTarget.TreatingProvider = pSource.TreatingProvider
		set pTarget.TreatingProviderNPI = pSource.TreatingProviderNPI
		set pTarget.TreatingSpeciality = pSource.TreatingSpeciality

		// 06-07-2018 MKC
		// Rational Jazz Story #742981 - Add Primary Care Provider NPI to data extraction
		set pTarget.PrimaryCareProviderNPI = pSource.PrimaryCareProviderNPI
		
		set pTarget.Network = pSource.Network
		set pTarget.CLINCode = pSource.CLINCode
		set pTarget.CLINDesc = pSource.CLINDesc
		set pTarget.PaymentAuthorityCode = pSource.PaymentAuthorityCode
		set pTarget.PaymentAuthorityDesc = pSource.PaymentAuthorityDesc
		set pTarget.AppointmentDate = pSource.AppointmentDate
		set pTarget.TreatmentStartDate = pSource.TreatmentStartDate

		//Payor Status
		//MKC 06-06-2018
		//Jazz Task #752492 - Adding Payor Status to HSRM Data Extraction
		set pTarget.PayorStatus = pSource.PayorStatus
		
		//OHI Info
		//MKC 06-06-2018
		//Jazz Task# 752175 - Adding OHI data elements to HSRM data extraction
		set pTarget.OHI = ##class(%ListOfObjects).%New()
		for i=1:1:pSource.OHI.Count() {
			set tTargetOHI = ##class(CCRA.OHI).%New()
			set tSourceOHI = pSource.OHI.GetAt(i)
			set tTargetOHI.Payor = tSourceOHI.Payor
			set tTargetOHI.ValidFromDate = tSourceOHI.ValidFromDate
			set tTargetOHI.ValidToDate = tSourceOHI.ValidToDate
			set tTargetOHI.CardholderName = tSourceOHI.CardholderName
			set tTargetOHI.PayorCardNumber = tSourceOHI.PayorCardNumber
			set tTargetOHI.CardholderAddress.Address1 = tSourceOHI.CardholderAddress.Address1
			set tTargetOHI.CardholderAddress.City = tSourceOHI.CardholderAddress.City
			set tTargetOHI.CardholderAddress.State = tSourceOHI.CardholderAddress.State
			set tTargetOHI.CardholderAddress.ZipCode = tSourceOHI.CardholderAddress.ZipCode
			
			$$$ThrowOnError(pTarget.OHI.Insert(tTargetOHI))
			
		}
		
		// eCAMS fields
		//JLM 07-11-18
		//Jazz Task #772894 - eCAMS add additional required fields to CCRA data transformation
		set pTarget.CategoryOfCare = pSource.CategoryOfCare 
		set pTarget.StationID = pSource.StationID
		set pTarget.EstimatedCostOfCare = pSource.EstimatedCostOfCare
		set pTarget.SeocID = pSource.SeocID
		set pTarget.Comments= pSource.Comments 
		set pTarget.ReferringProviderNPI  = pSource.ReferringProviderNPI
		set pTarget.TreatingFacilityTaxID = pSource.TreatingFacilityTaxID
		set pTarget.TreatingProviderTaxID = pSource.TreatingProviderTaxID
		set pTarget.NoteOnAppointmentScreen = pSource.NoteOnAppointmentScreen 
		set pTarget.CommentOnReferralScreen = pSource.CommentOnReferralScreen

		//Transaction History
		for i=1:1:pSource.TransactionHistory.Count() {
			
			set tSourceTransaction = pSource.TransactionHistory.GetAt(i)
			if '(tSourceTransaction=$$$NULLOREF) {
				set tTransHistory = ##class(CCRA.TransactionHistory).%New()
				set tTransHistory.TransactionDate = tSourceTransaction.TransactionDate
				set tTransHistory.EffectiveDate = tSourceTransaction.EffectiveDate
				set tTransHistory.Reason = tSourceTransaction.Reason
				set tTransHistory.Status = tSourceTransaction.Status
				$$$ThrowOnError(pTarget.TransactionHistory.Insert(tTransHistory))
			}
		}
		
		//Eligibility (ES) Info
		set tCoPaymentStatus = ##class(CCRA.ES.CoPaymentStatus).%New()
		set tCoPaymentStatus.PriorityEnrollmentGroup = pSource.CoPaymentStatus.PriorityEnrollmentGroup
		set tCoPaymentStatus.CurrentMeansTest = pSource.CoPaymentStatus.CurrentMeansTest
		set tCoPaymentStatus.AgentOrangeExposureIndicator = pSource.CoPaymentStatus.AgentOrangeExposureIndicator
		set tCoPaymentStatus.CombatVeteranEligibilityIndicator = pSource.CoPaymentStatus.CombatVeteranEligibilityIndicator
		set tCoPaymentStatus.CombatVeteranEndDate = pSource.CoPaymentStatus.CombatVeteranEndDate
		set tCoPaymentStatus.Project112SHADIndicator = pSource.CoPaymentStatus.Project112SHADIndicator
		set tCoPaymentStatus.SouthwestAsiaIndicator = pSource.CoPaymentStatus.SouthwestAsiaIndicator
		set tCoPaymentStatus.IonizingRadiationIndicator = pSource.CoPaymentStatus.IonizingRadiationIndicator
		set tCoPaymentStatus.CampLejeuneIndicator = pSource.CoPaymentStatus.CampLejeuneIndicator
		
		//For each service connected info - add to CoPayment Status Info
		for i=1:1:pSource.CoPaymentStatus.ServiceConnectedInfo.Count() {
			set tServiceConnectedInfo = ##class(CCRA.ES.ServiceConnectedInfo).%New()
			set tServiceConnectedInfo.ServiceConnected = pSource.CoPaymentStatus.ServiceConnectedInfo.GetAt(i).ServiceConnected
			set tServiceConnectedInfo.ServiceConnectedPercentage = pSource.CoPaymentStatus.ServiceConnectedInfo.GetAt(i).ServiceConnectedPercentage
			set tServiceConnectedInfo.ServiceConnectedCondition = pSource.CoPaymentStatus.ServiceConnectedInfo.GetAt(i).ServiceConnectedCondition
			$$$ThrowOnError(tCoPaymentStatus.ServiceConnectedInfo.Insert(tServiceConnectedInfo))
		}
				
		set pTarget.CoPaymentStatus = tCoPaymentStatus
			
		//Military Sexual Trauma
		set tMilitarySexualTrauma = ##class(CCRA.ES.MilitarySexualTraumaInfo).%New()
		set tMilitarySexualTrauma.DeterminationFacility = pSource.MilitarySexualTraumaInfo.DeterminationFacility
		set tMilitarySexualTrauma.Status = pSource.MilitarySexualTraumaInfo.Status
		set tMilitarySexualTrauma.StatusChangeDate = pSource.MilitarySexualTraumaInfo.StatusChangeDate
		set pTarget.MilitarySexualTraumaInfo = tMilitarySexualTrauma
		
		//Nose Throat Radium Info
		set tNoseThroatRadiumInfo = ##class(CCRA.ES.NoseThroatRadiumInfo).%New()
		set tNoseThroatRadiumInfo.DeterminationFacility = pSource.NoseThroatRadiumInfo.DeterminationFacility
		set tNoseThroatRadiumInfo.DiagnosedWithCancer = pSource.NoseThroatRadiumInfo.DiagnosedWithCancer
		set tNoseThroatRadiumInfo.DiagnosisVerifiedDate = pSource.NoseThroatRadiumInfo.DiagnosisVerifiedDate
		set tNoseThroatRadiumInfo.FromAviatorServiceBefore1955 = pSource.NoseThroatRadiumInfo.FromAviatorServiceBefore1955
		set tNoseThroatRadiumInfo.FromSubmarinerServiceBefore1965 = pSource.NoseThroatRadiumInfo.FromSubmarinerServiceBefore1965
		set tNoseThroatRadiumInfo.ReceivingTreatment = pSource.NoseThroatRadiumInfo.ReceivingTreatment
		set tNoseThroatRadiumInfo.VerificationDate = pSource.NoseThroatRadiumInfo.VerificationDate
		set tNoseThroatRadiumInfo.VerificationFacility = pSource.NoseThroatRadiumInfo.VerificationFacility
		set tNoseThroatRadiumInfo.VerificationMethod = pSource.NoseThroatRadiumInfo.VerificationMethod
		
		set pTarget.NoseThroatRadiumInfo = tNoseThroatRadiumInfo
		
		for i=1:1:pSource.PreAuthorizations.Count() {
			
			set tSourcePreAuthorization = pSource.PreAuthorizations.GetAt(i)
			set tPreAuthorization = ##class(CCRA.PreAuthorization).%New()
			set tPreAuthorization.Comments = tSourcePreAuthorization.Comments
			set tPreAuthorization.DateFrom = tSourcePreAuthorization.DateFrom
			set tPreAuthorization.DateTo = tSourcePreAuthorization.DateTo
			set tPreAuthorization.Description = tSourcePreAuthorization.Description
			set tPreAuthorization.Duration = tSourcePreAuthorization.Duration
			set tPreAuthorization.PreAuthorizationNumber = tSourcePreAuthorization.PreAuthorizationNumber
			set tPreAuthorization.PreAuthorizationReason = tSourcePreAuthorization.PreAuthorizationReason
			set tPreAuthorization.PreAuthorizationStatus = tSourcePreAuthorization.PreAuthorizationStatus
			
			for j=1:1:tSourcePreAuthorization.Details.Count() {
			
				set tSourcePreAuthorizationDetail = tSourcePreAuthorization.Details.GetAt(j)
				set tPreAuthorizationDetail = ##class(CCRA.PreAuthorizationDetail).%New()
				set tPreAuthorizationDetail.LimitQty = tSourcePreAuthorizationDetail.LimitQty
				
				for k=1:1:tSourcePreAuthorizationDetail.OrderSets.Count() {
					
					set tSourceOrderSet = tSourcePreAuthorizationDetail.OrderSets.GetAt(k)
					set tPreAuthorizationOrderSet = ##class(CCRA.PreAuthorizationOrderSet).%New()
					set tPreAuthorizationOrderSet.Code = tSourceOrderSet.Code
					set tPreAuthorizationOrderSet.Desc = tSourceOrderSet.Desc
					
					for l=1:1:tSourceOrderSet.OrderSetDates.Count() {
						
						set tSourceOrderSetDate = tSourceOrderSet.OrderSetDates.GetAt(l)
						set tOrderSetDate = ##class(CCRA.PreAuthorizationOrderSetDate).%New()
						set tOrderSetDate.DateFrom = tSourceOrderSetDate.DateFrom
						set tOrderSetDate.DateTo = tSourceOrderSetDate.DateTo
						
						for m=1:1:tSourceOrderSetDate.OrderSetDateItems.Count() {
							
							set tSourceOrderSetDateItem = tSourceOrderSetDate.OrderSetDateItems.GetAt(m)
							set tOrderSetDateItem = ##class(CCRA.PreAuthorizationOrderSetDateItem).%New()
							set tOrderSetDateItem.ProcedureCode = tSourceOrderSetDateItem.ProcedureCode
							
							set tOrderSetDateItem.OrderSetDate = tOrderSetDate
						}
						
						set tOrderSetDate.OrderSet = tPreAuthorizationOrderSet
						
					}
					
					set tPreAuthorizationOrderSet.PreAuthorizationDetail = tPreAuthorizationDetail
					
				}
				
				set tPreAuthorizationDetail.PreAuthorization = tPreAuthorization
			}
			
			set tPreAuthorization.Referral = pTarget
			
		} 		
		
		
	} catch e {
		
		set tSC = e.AsStatus()
		
	}
	
	quit tSC
}

}
