Class CCRS.ReferralList.Process Extends Ens.BusinessProcess [ ClassType = persistent ]
{

Property CCRSReferralListTarget As Ens.DataType.ConfigName [ InitialExpression = "CCRS.JSONFile.SFTP.Operation" ];

Parameter SETTINGS = "CCRSReferralListTarget";

Method OnRequest(pRequest As CCRA.Message.ProcessReferralListRequest, Output pResponse As Ens.Response) As %Status
{
	#dim tSC As %Status = $$$OK
	#dim e As %Exception.AbstractException
	#dim tResultSet As %ResultSet
	#dim tJSONStream As %Stream.Object
	#dim tRequest As Ens.StreamContainer = ##class(Ens.StreamContainer).%New()
	#dim tCCRSReferral As CCRS.Referral
	#dim tReferralList As CCRS.ReferralList
	
	try {
		
		//Instantiate Referral List to be sent to CCRS
		set tReferralList = ##class(CCRS.ReferralList).%New()
		
		//For each Referral in pRequest
		// 	1. Execute Transform to CCRS Referral
		//	2. Add to CCRS Referral List
		for i=1:1:pRequest.Referrals.Count() {
			
			set tSourceReferral = pRequest.Referrals.GetAt(i)
			$$$ThrowOnError(##class(CCRS.ReferralList.Transform.CCRAReferralToCCRSReferral).Transform(tSourceReferral,.tCCRSReferral))
			if '(tCCRSReferral=$$$NULLOREF) d tReferralList.Referrals.Insert(tCCRSReferral)

		}
		
		if tReferralList.Referrals.Count() > 0 {
			
			//Determine Processing dates
			for i=1:1:tReferralList.Referrals.Count() {
				set tTS(tReferralList.Referrals.GetAt(i).ReferralLastUpdateDate)=""
			}
			
			//TODO: Do we need to save the Referral List (maybe for processing errors?)
			//$$$ThrowOnError(tReferralList.%Save())
		
			//Export Referral List to JSON stream
			//MKC - 02/28/2018 - Removed "q" flag to force numeric values to be output with quotes per CCRS Request.
			$$$ThrowOnError(##class(%ZEN.Auxiliary.jsonProvider).%WriteJSONStreamFromObject(.tJSONStream,tReferralList,,,,"tuw"))
					
			//Pass JSON stream to the HTTP Operation for deliver to HealthShare
			set tRequest = ##class(Ens.StreamContainer).%New()
			set tRequest.Stream = ##class(%Stream.FileCharacterGzip).%New()
			//Create Filename
			$$$ThrowOnError(..GetTS("start",.tTS,.tStartTS))
			$$$ThrowOnError(..GetTS("end",.tTS,.tEndTS))
			//MKC - 05/01/2018 - CM (Defect 725829)
			//Remove TimestampFormat as tStartTS and tEndTS are already in yyyy-mm-dd format
			//set tRequest.OriginalFilename = "CCRA_Referrals_" _ ..TimestampFormat(tStartTS) _ "-" _ ..TimestampFormat(tEndTS) _ "_Rev01.json.gz"
			set tRequest.OriginalFilename = "CCRA_Referrals_" _ tStartTS _ "-" _ tEndTS _ "_Rev01.json.gz"
			$$$ThrowOnError(tRequest.Stream.CopyFrom(tJSONStream))
			$$$ThrowOnError(..SendRequestSync(..CCRSReferralListTarget,tRequest,.tResponse))
			
		} else {
			//Nothing to send
			$$$LOGINFO("No Referrals to be sent.")
		}
		
	} catch e {
		
		set tSC = e.AsStatus()
		
	}
	
	quit tSC
}

Method GetTS(pType As %String, ByRef pTimeStamps As %TimeStamp, Output pTS As %TimeStamp) As %Status
{
	#dim tSC AS %Status = $$$OK
	#dim e AS %Exception.AbstractException
	
	try {
		
		if pType = "start" {
			set pTS = $order(pTimeStamps(""))
		} elseif pType = "end" {
			set key=$ORDER(pTimeStamps(""))
			while (key'="") {
				set pTS=key
			    set key = $ORDER(pTimeStamps(key))
   			}
		}
		
		
	} catch e {
		
		set tSC = e.AsStatus()
		
	}
	
	quit tSC
}

Method TimestampFormat(pTS As %String) As %String
{
	set tTS = $zdt(pTS,8)
	quit $p(tTS," ",1)_$p($p(tTS," ",2),":",1)_$p($p(tTS," ",2),":",2)_$p($p(tTS," ",2),":",3)
}

Storage Default
{
<Data name="ProcessDefaultData">
<Subscript>"Process"</Subscript>
<Value name="1">
<Value>CCRSReferralListTarget</Value>
</Value>
</Data>
<DefaultData>ProcessDefaultData</DefaultData>
<Type>%Library.CacheStorage</Type>
}

}
