Class DAS.Operation.FHIRDocumentationNotification Extends (EnsLib.REST.GenericOperation, HS.Util.Trace.Helper)
{

Parameter INVOCATION = "Queue";

// Override the OnInit method in order to use the DASHTTPRequest and update headers as needed

Method OnInit() As %Status
{
	Set ..%HttpRequest=##class(DAS.Message.DASHTTPRequest).%New()
	Quit $$$OK
}

Method OnMessage(pRequest As DAS.Message.FHIRDocumentNotificationRequest, Output pResponse As EnsLib.HTTP.GenericMessage) As %Status
{
 
	set tSC = $$$OK 

	try{

  		//Verify required data present in inbound message, error and quit if not
  		Set pCCN = $CASE(pRequest.CCN,
              "CCN1":"CCN_Region1","CCN2":"CCN_Region2","CCN3":"CCN_Region3",
              "CCN4":"CCN_Region4","TriWest":"CCN_TRW_National",
              :"Invalid Network")
  		$$$HSTRACE("Network: " _ pCCN)
  		Set pFHIRDocumentID = pRequest.FHIRDocumentId   
  		Set pConversationID = ..%RequestHeader.SessionId 
  		Set pTransactionID = ..%RequestHeader.MessageId()
  		
  		If (pRequest.Stream ="") {
  			Set tSC=$$$ERROR($$$EnsErrGeneral," Message stream empty")
  			Quit 
  		}
    	ElseIf (pCCN="")  {
  			Set tSC=$$$ERROR($$$EnsErrGeneral," CCN missing")
  			Quit 
  		}
    	ElseIf (pFHIRDocumentID="") {
  			Set tSC=$$$ERROR($$$EnsErrGeneral," FHIR Document ID required")
  			Quit 
  		}
  		
  		//Set custom DAS headers and message body
  	//	Do ..%HttpRequest.SetHeader("Content-Type","application/json+fhir") //not needed.  pulled from entitybody
  		Do ..%HttpRequest.SetHeader("X-RoutingSenderID","VA_CCRA")
  		Do ..%HttpRequest.SetHeader("X-RoutingReceiverIDs", pCCN) 
  		Do ..%HttpRequest.SetHeader("X-TransactionID",pTransactionID)
  		Do ..%HttpRequest.SetHeader("X-ConversationID", pConversationID) //GUID when doc is generated, ex: "b7d3e295-6e0c-47a8-a8fc-fe0c910dcf15"
  		Set ..%HttpRequest.EntityBody = pRequest.Stream
  		
  		$$$HSTRACE("Header List: " _ ..%HttpRequest.ReturnHeaders())
  		
  		//Create response and POST data to DAS
  		Set tHttpResponse = ##class(%Net.HttpResponse).%New()
  		Set tSC = ..Adapter.SendFDNS   aArray(.tHttpResponse, "POST", ..%HttpRequest, .pFormVarNames , .pData)
    		
  		
  		//Log response/error received from DAS
  		If $IsObject(tHttpResponse.Data) {
  			Set sResponseBody = tHttpResponse.Data.Read() 			
  			$$$LOGINFO(sResponseBody)
  		}
  		Else { 	$$$LOGERROR("No response received from  DAS") 		}
      
  	}
  catch ex {
  		Set tSC = ex.AsStatus()
  }
  
  return tSC
}

XData MessageMap
{
<MapItems>
		<MapItem MessageType="DAS.Message.FHIRDocumentNotificationRequest"> 
			<Method>OnMessage</Method>
		</MapItem>
	</MapItems>
}

}
