Class DAS.ReferralList.Process Extends Ens.BusinessProcess [ ClassType = persistent ]
{

Property DASOperation As Ens.DataType.ConfigName [ InitialExpression = "DAS.Operation.FHIRDocumentationNotification" ];

Property CCNList As %String(MAXLEN = "") [ InitialExpression = "CCN_Region1,CCN_Region2,CCN_Region3,CCN_Region4" ];

Property DemoMode As %Boolean [ InitialExpression = 1 ];

Property JSON As %Boolean [ InitialExpression = 1 ];

// saves referral as JSON when true, else saves as XML

Parameter SETTINGS = "DASOperation, DemoMode, JSON";

// sends a file to DAS.Operation.FilePassthrough when true

Method OnRequest(pRequest As CCRA.Message.ProcessReferralListRequest, Output pResponse As Ens.Response) As %Status
{
	#dim tSC As %Status = $$$OK
	#dim e As %Exception.AbstractException
	#dim tNetworkReferralList As DAS.CCNReferralList
	#dim tNetworkReferralLists As %ArrayOfObjects
	#dim tCCRSReferral As CCRA.WS.Client.s0.Referral
	#dim tDASReferral As DAS.CCNContractReferral

	try {

		//Create a CCN Referral List for each possible CCN Network.
		//Add the CCN Referral List to the array of Network Referral Lists (tNewworkReferralLists)
		set tNetworkCnt = $LENGTH(..CCNList,",")
		set tNetworkReferralLists = ##class(%ArrayOfObjects).%New()
		
		for i=1:1:tNetworkCnt {
			set tKey = $p(..CCNList,",",i)
			set tNetworkReferralList=##class(DAS.CCNReferralList).%New()
			
			$$$ThrowOnError(tNetworkReferralLists.SetAt(tNetworkReferralList,tKey))
		}
		
		//Walk thru the CCRS Referral List 
		//For each CCRS Referral
		//	1. Transform to DAS Referral
		//  2. Assign to a CCN Network List
		for i=1:1:pRequest.Referrals.Count() {
			
			set tCCRSReferral = pRequest.Referrals.GetAt(i)
			$$$ThrowOnError(..TransformCCRSReferralToDASReferral(tCCRSReferral,.tDASReferral))
			if '(tDASReferral=$$$NULLOREF) {
				set tNetwork=$zstrip(tDASReferral.CCNNetwork,"<>W")
				//Map to DAS Network
				//set tDASNetwork=$S(tNetwork="CC Network 1":"CCN_Region1",tNetwork="CC Network 2":"CCN_Region2",tNetwork="CC Network 3":"CCN_Region3",tNetwork="CC Network 4":"CCN_Region4")
				//set tDASNetwork=$S(tNetwork="Department of Defense":"CCN_Region1",tNetwork="CC Network 2":"CCN_Region2",tNetwork="CC Network 3":"CCN_Region3",tNetwork="CC Network 4":"CCN_Region4")
				set tDASNetwork=$REPLACE(tNetwork,"CC Network ","CCN_Region")
				if '(tNetworkReferralLists.GetAt(tDASNetwork)=$$$NULLOREF) {
					$$$ThrowOnError(tNetworkReferralLists.GetAt(tDASNetwork).Referrals.Insert(tDASReferral))
				}
			}
			
		}
		
		//Should now have an array containing a list of CCNReferralLists representing each of the Networks
		//Walk-thru Network Referrals List array - output referrals to xml - save xml as DAS FHIR Document
		//and send synchronously to the DAS REST Operation.
		set tKey = ""
		do {
			//$$$LOGINFO("tKey " _ tKey)
			set tNetworkReferralList = tNetworkReferralLists.GetNext(.tKey) q:(tNetworkReferralList=$$$NULLOREF)
			if (tNetworkReferralList.Referrals.Count() > 0) {
				
				//Save Referral List
				$$$ThrowOnError(..SaveReferralList(.tNetworkReferralList,.tFHIRDocument))
				
				//Create DAS Request Message
				set tDASRequest = ##class(DAS.Message.FHIRDocumentNotificationRequest).%New()
				set tDASRequest.FHIRDocumentId = tFHIRDocument.DocumentId
				set tDASRequest.CCN = tKey
			
				
				//Create DAS Message Body
				set tMessageBody = ##class(%Stream.GlobalCharacter).%New()
				$$$ThrowOnError(..CreateFHIRMessageBody(tDASRequest.FHIRDocumentId,.tMessageBody))
				
				//Add message body to DAS Request Message
				set tDASRequest.Stream = ##class(%Stream.GlobalCharacter).%New()
				$$$ThrowOnError(tDASRequest.Stream.CopyFrom(tMessageBody))
				
				//send to DAS REST Operation
				//$$$ThrowOnError(..SendRequestAsync(..DASOperation,tDASRequest,1,"DAS REST Post"))
				$$$ThrowOnError(..SendRequestAsync(..DASOperation,tDASRequest,0,"DAS REST Post"))
				
			}
			
		} while 1
		
		
	} catch e {
		
		set tSC = e.AsStatus()
		
	}

	quit tSC
}

Method SaveReferralList(ByRef pReferralList As DAS.CCNReferralList, Output pFHIRDocument As DAS.ReferralList.FHIRDocument) As %Status
{
	#dim tSC As %Status = $$$OK
	#dim e As %Exception.AbstractException
	#dim tReferralList As %Stream.GlobalCharacter = ##class(%Stream.GlobalCharacter).%New()
	
	try {
		
		set pFHIRDocument = ##class(DAS.ReferralList.FHIRDocument).%New()
		set pFHIRDocument.DocumentId = $System.Util.CreateGUID()
		set pFHIRDocument.CreationDate = $ZDT($NOW(), 3 , 1) 
		
		//**** XML Logic
		//if '(..JSON){
		//	set pFHIRDocument.ReferralListXML = ##class(%Stream.GlobalCharacter).%New()
		//	$$$ThrowOnError(pReferralList.XMLExportToStream(.tReferralList))
		//	$$$ThrowOnError(pFHIRDocument.ReferralListXML.CopyFrom(tReferralList))	
		//}
		//**** XML Logic
		
		//**** JSON Logic
		if (..JSON){
			set pFHIRDocument.ReferralListJSON = ##class(%Stream.GlobalCharacter).%New()
			set tSC = ##class(%ZEN.Auxiliary.jsonProvider).%WriteJSONStreamFromObject(.tReferralList,pReferralList,,,,"etuw")
			$$$ThrowOnError(pFHIRDocument.ReferralListJSON.CopyFrom(tReferralList))
		}
		else { //XML
			set pFHIRDocument.ReferralListXML = ##class(%Stream.GlobalCharacter).%New()
			$$$ThrowOnError(pReferralList.XMLExportToStream(.tReferralList))
			$$$ThrowOnError(pFHIRDocument.ReferralListXML.CopyFrom(tReferralList))	
		}

		set test = ..JSON
		$$$ThrowOnError(pFHIRDocument.%Save())
		
		//Save out to file for demo purposes
		if (..DemoMode){
			set sContainer = ##class(Ens.StreamContainer).%New()
			do sContainer.StreamSet(tReferralList)
			set tSC = ..SendRequestAsync("DAS.Operation.FilePassthrough", sContainer)		
		}
		
	} catch e {
		
		set tSC = e.AsStatus()
	
	}
	
	quit tSC
}

Method TransformCCRSReferralToDASReferral(pSource As CCRA.Referral, ByRef pTarget As DAS.CCNContractReferral) As %Status
{
	#dim tSC As %Status = $$$OK
	#dim e As %Exception.AbstractException
	
	try {
		
		set:(pSource=$$$NULLOREF) tSC = $$$ERROR($$$GeneralError,"DAS Referral Transformation Failed. Invalid Source CCRS Referral.")
		$$$ThrowOnError(tSC)
		
		set pTarget = ##class(DAS.CCNContractReferral).%New()
		set pTarget.ReferralId = pSource.ReferralNumber
		set pTarget.ReferralConsultId = pSource.ReferralConsultId
		//set pTarget.PrimaryPayer = pSource.Payor // 9/24/18 MM Task # 827025
		//set pTarget.SecondaryPayer = pSource.SecondaryPayor // 9/24/18 MM Task # 827025
		set pTarget.PayorStatus = pSource.PayorStatus   // 6/6/18 JLM  Task # 752549, add Payor Status	
		set pTarget.PrimaryCareProviderNPI =  pSource.PrimaryCareProviderNPI   // 6/7/18 JLM Task # 753165, Add PCP NPI	
		set pTarget.PrimaryCareProviderName =  pSource.PrimaryCareProviderName   // 10/04/18 MM Task # , Add PCP Name
		set pTarget.ReferralFromDate = pSource.ReferralDate
		set pTarget.ReferralToDate = pSource.ReferralExpirationDate
		set pTarget.Veteran = ##class(DAS.Veteran).%New()
		set pTarget.Veteran.LName = pSource.Patient.LastName
		set pTarget.Veteran.FName = pSource.Patient.FirstName
		set pTarget.Veteran.MName = "" ;TODO: add Middle Name to Patient.
		set pTarget.Veteran.ICN = pSource.Patient.ICN // 9/24/18 MM Task # 827025
		//set pTarget.Veteran.SSN = pSource.Patient.SSN // 9/24/18 MM Task # 827025
		set pTarget.Veteran.HomeAddress = ##class(DAS.Address).%New()
		set pTarget.Veteran.HomeAddress.Address1 = pSource.Patient.HomeAddress.Address1 // 9/24/18 MM Task # 827025
		set pTarget.Veteran.HomeAddress.Address2 = pSource.Patient.HomeAddress.Address2 // 9/24/18 MM Task # 827025
		set pTarget.Veteran.HomeAddress.City = pSource.Patient.HomeAddress.City // 9/24/18 MM Task # 827025
		set pTarget.Veteran.HomeAddress.State = pSource.Patient.HomeAddress.State // 9/24/18 MM Task # 827025
		set pTarget.Veteran.HomeAddress.ZipCode = pSource.Patient.HomeAddress.ZipCode // 9/24/18 MM Task # 827025
		set pTarget.Veteran.HomeAddress.Country = pSource.Patient.HomeAddress.Country // 9/24/18 MM Task # 827025
		//set pTarget.Veteran.Telephone = pSource.Patient.Telephone // 9/24/18 MM Task # 827025 remove 10/16/2018 MM
		set pTarget.Veteran.TelephoneHome = pSource.Patient.TelephoneHome // 10/04/18 MM Task # 
		set pTarget.Veteran.TelephoneBusiness = pSource.Patient.TelephoneBusiness // 10/04/18 MM Task # 
		set pTarget.Veteran.TelephoneMobile = pSource.Patient.TelephoneMobile // 10/04/18 MM Task # 
		set pTarget.Veteran.Email = pSource.Patient.Email // 10/04/18 MM Task # 
		set pTarget.Veteran.PreferredContactMethod = pSource.Patient.PreferredContactMethod // 10/04/18 MM Task # 
					
		set pTarget.SeocID = pSource.SeocID // 9/24/18 MM Task # 827025
		set pTarget.CCNNetworkCode = pSource.NetworkCode // 10/16/18 MM
		set pTarget.CCNNetwork = pSource.Network
		set pTarget.CLINCode = pSource.CLINCode // 10/16/18 MM
		set pTarget.CLINDesc = pSource.CLINDesc // 10/16/18 MM
		//set pTarget.ReferralDate = pSource.ReferralDate // 9/24/18 MM Task # 827025
		set pTarget.CCNProvider = pSource.ReferringProvider
		set pTarget.CCNFacility = pSource.ReferringFacility
		//set pTarget.AppointmentDate = pSource.AppointmentDate // 9/24/18 MM Task # 827025
		set pTarget.CareIndicatedDate = pSource.TreatmentStartDate
		set pTarget.PreferredProvider = pSource.PreferredProvider
		set pTarget.Comments = pSource.Comments // 9/24/18 MM Task # 827025
	//	set pTarget.OtherHealthInsuranceInfo = pSource.OtherInsuranceInfo  //No longer needed with the addition of OHI data below
		set pTarget.OrderingOfficerName = pSource.OrderingOfficerName
		set pTarget.StationNumber = pSource.StationID // 9/24/18 MM Task # 827025
		set pTarget.StationName = pSource.ReferringFacility // 9/24/18 MM Task # 827025
		set pTarget.CCNProviderNPI = pSource.TreatingProviderNPI // 9/24/18 MM Task # 827025
		
		// Set ReferralDate and AppointmentDate from Transaction History // 9/24/18 MM Task # 827025
		for i=1:1:pSource.TransactionHistory.Count() {
			set tSourceTransaction = pSource.TransactionHistory.GetAt(i)
			if '(tSourceTransaction=$$$NULLOREF) {
				if tSourceTransaction.Status = "Sent" {
					set pTarget.ReferralDate = tSourceTransaction.TransactionDate
				}
				if i=1 {
					//set pTarget.AppointmentDate = $SELECT(tSourceTransaction.TransactionDate'="":$zdt(tSourceTransaction.TransactionDate,3),1:"")
					set pTarget.AppointmentDate = tSourceTransaction.TransactionDate
				}
			}
		}
		//OHI Data, 6/6/18 JLM, Task #752238
		set pTarget.OHI = ##class(%ListOfObjects).%New() 
		//set pTarget.OHI = pSource.OHI
		//$$$LOGINFO("OHI count: " _ pSource.OHI.Count())
		for i=1:1:pSource.OHI.Count() {
			set tTargetOHI = ##class(DAS.OHI).%New()
			set tSourceOHI = pSource.OHI.GetAt(i)
			set tTargetOHI.Payor = tSourceOHI.Payor
			set tTargetOHI.ValidFromDate = tSourceOHI.ValidFromDate
			set tTargetOHI.ValidToDate = tSourceOHI.ValidToDate
			set tTargetOHI.CardholderName = tSourceOHI.CardholderName
			set tTargetOHI.PayorCardNumber = tSourceOHI.PayorCardNumber
			//set tTargetOHI.CardholderAddress.Address1 = tSourceOHI.CardholderAddress.Address1 // 9/24/18 MM Task # 827025
			//set tTargetOHI.CardholderAddress.City = tSourceOHI.CardholderAddress.City // 9/24/18 MM Task # 827025
			//set tTargetOHI.CardholderAddress.State = tSourceOHI.CardholderAddress.State // 9/24/18 MM Task # 827025
			//set tTargetOHI.CardholderAddress.ZipCode = tSourceOHI.CardholderAddress.ZipCode // 9/24/18 MM Task # 827025
			
			$$$ThrowOnError(pTarget.OHI.Insert(tTargetOHI))
			
		}
		
		//Pre-Authorizations		
		/*for i=1:1:pSource.PreAuthorizations.Count() {
			
			set tSourcePreAuthorization = pSource.PreAuthorizations.GetAt(i)
			set tPreAuthorization = ##class(DAS.PreAuthorization).%New()
			set tPreAuthorization.Comments = tSourcePreAuthorization.Comments
			set tPreAuthorization.DateFrom = tSourcePreAuthorization.DateFrom
			set tPreAuthorization.DateTo = tSourcePreAuthorization.DateTo
			set tPreAuthorization.Description = tSourcePreAuthorization.Description
			set tPreAuthorization.PreAuthorizationNumber = tSourcePreAuthorization.PreAuthorizationNumber
			set tPreAuthorization.PreAuthorizationReason = tSourcePreAuthorization.PreAuthorizationReason
			set tPreAuthorization.PreAuthorizationStatus = tSourcePreAuthorization.PreAuthorizationStatus
			
			for j=1:1:tSourcePreAuthorization.Details.Count() {
			
				set tSourcePreAuthorizationDetail = tSourcePreAuthorization.Details.GetAt(j)
				set tPreAuthorizationDetail = ##class(DAS.PreAuthorizationDetail).%New()
				set tPreAuthorizationDetail.LimitQty = tSourcePreAuthorizationDetail.LimitQty
				
				for k=1:1:tSourcePreAuthorizationDetail.OrderSets.Count() {
					
					set tSourceOrderSet = tSourcePreAuthorizationDetail.OrderSets.GetAt(k)
					set tPreAuthorizationOrderSet = ##class(DAS.PreAuthorizationOrderSet).%New()
					set tPreAuthorizationOrderSet.Code = tSourceOrderSet.Code
					set tPreAuthorizationOrderSet.Desc = tSourceOrderSet.Desc
					
					for l=1:1:tSourceOrderSet.OrderSetDates.Count() {
						
						set tSourceOrderSetDate = tSourceOrderSet.OrderSetDates.GetAt(l)
						set tOrderSetDate = ##class(DAS.PreAuthorizationOrderSetDate).%New()
						set tOrderSetDate.DateFrom = tSourceOrderSetDate.DateFrom
						set tOrderSetDate.DateTo = tSourceOrderSetDate.DateTo
						
						for m=1:1:tSourceOrderSetDate.OrderSetDateItems.Count() {
							
							set tSourceOrderSetDateItem = tSourceOrderSetDate.OrderSetDateItems.GetAt(m)
							set tOrderSetDateItem = ##class(DAS.PreAuthorizationOrderSetDateItem).%New()
							set tOrderSetDateItem.ProcedureCode = tSourceOrderSetDateItem.ProcedureCode
							
							set tOrderSetDateItem.OrderSetDate = tOrderSetDate
						}
						
						set tOrderSetDate.OrderSet = tPreAuthorizationOrderSet
						
					}
					
					set tPreAuthorizationOrderSet.PreAuthorizationDetail = tPreAuthorizationDetail
					
				}
				
				set tPreAuthorizationDetail.PreAuthorization = tPreAuthorization
			}
			
			set tPreAuthorization.Referral = pTarget
			
		} */		
	} catch e {
		
		set tSC = e.AsStatus()
			
	}
	
	quit tSC
}

Method OnResponse(pRequest As CCRA.Message.ProcessReferralListRequest, ByRef pResponse As Ens.Response, pCallrequest As DAS.Message.FHIRDocumentNotificationRequest, pCallresponse As Ens.Response, pCompletionKey As %String) As %Status
{

	#dim tSC As %Status = $$$OK
	#dim e As %Exception.AbstractException
	
	try {
	
		set tCCN = pCallrequest.CCN
		//TODO:Handle response Errors
	
	} catch e {
	
		set tSC = e.AsStatus()
		
	}
	
	quit tSC
}

ClassMethod CreateFHIRMessageBody(pDocumentId As %String, Output pMessageBody As %Stream.GlobalCharacter) As %Status
{

	/*Sample Message Body
	{
	"id": "DocumentReference/31c54d42-174c-4f27-a7e7-9a45271b5613",
	"resourceType": "DocumentReference",
	"text": {
	"fhir_comments": [{
	"div": "This is a text file with seeding request information, caret delimited."
	}]
	},
	"status": "generated",
	"indexed": "2017-08-03T15:06:30:30",
	"type": {
	"coding": [{
	"system": "http://loinc.org<http://loinc.org/> ",
	"code": "LP173405-4"
	}]
	},
	"content": [{
	"attachment": {
	"contentType": "text/plain",
	"data": "",
	"url": "/something/Binary/CCNNAME_Insurance_08072017.txt",
	"language": "en-US",
	"size": 101055,
	"hash": "/vPV/gsSj4MujDJm6TjMOQKyqSc=",
	"title": "The Health-Insurance Related Seeding Request File."
	}
	}]
	}
	*/
	#dim tSC As %Status = $$$OK
	#dim e As %Exception.AbstractException
	
	try {
		set pMessageBody = ##class(%Stream.GlobalCharacter).%New()
		d pMessageBody.Write("{""id"":""DocumentReference/"_pDocumentId_""",")
		d pMessageBody.Write("""resourceType"": ""DocumentReference"",")
		d pMessageBody.Write("""text"": {")
		d pMessageBody.Write("""fhir_comments"": [{")
		d pMessageBody.Write("""div"": ""This is a JSON file with CCRA Referral Data.""")
		d pMessageBody.Write("}]")
		d pMessageBody.Write("},")
		d pMessageBody.Write("""status"": ""current"",")
		set tTS = $zts
		set tTSFormatted = $p($zdt(tTS,3,1)," ",1)_"T"_$p($zdt(tTS,3,1)," ",2)
		d pMessageBody.Write("""indexed"": """_tTSFormatted_""",")
		d pMessageBody.Write("""content"": [{")
		d pMessageBody.Write("""attachment"": {")
		d pMessageBody.Write("""contentType"": ""application/json+fhir"",")
		d pMessageBody.Write("""url"": ""/csp/healthshare/ccraint/rest/DAS/Document/"_pDocumentId_"""")
		d pMessageBody.Write("}")
		d pMessageBody.Write("}]")
		d pMessageBody.Write("}")
		
	} catch e {
	
		set tSC = e.AsStatus()
		
	}
	
	quit tSC
}

ClassMethod TestOnRequest()
{
	
	set tProcess = ##class(Ens.Host).GetShadowInstance("DAS.ReferralList.Process")
	set tMsg = ##class(CCRA.Message.ProcessReferralListRequest).%OpenId(92731)
	
	d tProcess.OnInit()
	d tProcess.OnRequest(tMsg,.tResponse)
}

Storage Default
{
<Data name="ProcessDefaultData">
<Subscript>"Process"</Subscript>
<Value name="1">
<Value>DASOperation</Value>
</Value>
<Value name="2">
<Value>CCNList</Value>
</Value>
<Value name="3">
<Value>DemoMode</Value>
</Value>
<Value name="4">
<Value>JSON</Value>
</Value>
</Data>
<DefaultData>ProcessDefaultData</DefaultData>
<Type>%Library.CacheStorage</Type>
}

}
