Class DAS.ReferralList.RTProcess Extends Ens.BusinessProcess [ ClassType = persistent ]
{

Property DASOperation As Ens.DataType.ConfigName [ InitialExpression = "DAS.Operation.FHIRDocumentationNotification" ];

Property CCNList As %String(MAXLEN = "") [ InitialExpression = "CCN_Region1,CCN_Region2,CCN_Region3,CCN_Region4" ];

Property DemoMode As %Boolean [ InitialExpression = 0 ];

Property JSON As %Boolean [ InitialExpression = 1 ];

// saves referral as JSON when true, else saves as XML

Parameter SETTINGS = "DASOperation, DemoMode, JSON";

// sends a file to DAS.Operation.FilePassthrough when true

Method OnRequest(pRequest As DAS.Message.ReferralRequest, Output pResponse As Ens.Response) As %Status
{
	#dim tSC As %Status = $$$OK
	#dim e As %Exception.AbstractException
	#dim tNetworkReferralList As DAS.CCNReferralList
	#dim tNetworkReferralLists As %ArrayOfObjects
	#dim tCCRSReferral As CCRA.WS.Client.s0.Referral
	#dim tDASReferral As DAS.CCNContractReferral
	
	try {
			
		//Save Referral List
		$$$ThrowOnError(..SaveReferralList(pRequest,.tFHIRDocument))
				
		//Create DAS Request Message
		set tDASRequest = ##class(DAS.Message.FHIRDocumentNotificationRequest).%New()
		set tDASRequest.FHIRDocumentId = tFHIRDocument.DocumentId
		set tDASRequest.CCN = pRequest.NetworkCode		
				
		//Create DAS Message Body
		set tMessageBody = ##class(%Stream.GlobalCharacter).%New()
		$$$ThrowOnError(..CreateFHIRMessageBody(tDASRequest.FHIRDocumentId,.tMessageBody))
				
		//Add message body to DAS Request Message
		set tDASRequest.Stream = ##class(%Stream.GlobalCharacter).%New()
		$$$ThrowOnError(tDASRequest.Stream.CopyFrom(tMessageBody))
				
		//send to DAS REST Operation
		$$$ThrowOnError(..SendRequestAsync(..DASOperation,tDASRequest,0,"DAS REST Post"))
					
	} catch e {
		set tSC = e.AsStatus()	
	}
	quit tSC
}

Method SaveReferralList(ByRef pReferral As DAS.Message.ReferralRequest, Output pFHIRDocument As DAS.ReferralList.FHIRDocument) As %Status
{
	#dim tSC As %Status = $$$OK
	#dim e As %Exception.AbstractException
	#dim tReferralList As %Stream.GlobalCharacter = ##class(%Stream.GlobalCharacter).%New()
	
	try {
		
		set pFHIRDocument = ##class(DAS.ReferralList.FHIRDocument).%New()
		set pFHIRDocument.DocumentId = $System.Util.CreateGUID()
		set pFHIRDocument.CreationDate = $ZDT($NOW(), 3 , 1) 
		set pFHIRDocument.ReferralListJSON = ##class(%Stream.GlobalCharacter).%New()
		$$$ThrowOnError(pFHIRDocument.ReferralListJSON.CopyFrom(pReferral.Stream))  //pReferral.Stream is JSON stream from RealTime.Process.ExtractTransformReferral


		$$$ThrowOnError(pFHIRDocument.%Save())
		
		//Save out to file for demo purposes
		if (..DemoMode){
			set sContainer = ##class(Ens.StreamContainer).%New()
			do sContainer.StreamSet(pReferral.Stream)
			set tSC = ..SendRequestAsync("DAS.Operation.FilePassthrough", sContainer)		
		}
		
	} catch e {
		
		set tSC = e.AsStatus()
	
	}
	
	quit tSC
}

Method OnResponse(pRequest As CCRA.Message.ProcessReferralListRequest, ByRef pResponse As Ens.Response, pCallrequest As DAS.Message.FHIRDocumentNotificationRequest, pCallresponse As Ens.Response, pCompletionKey As %String) As %Status
{

	#dim tSC As %Status = $$$OK
	#dim e As %Exception.AbstractException
	
	try {
	
		set tCCN = pCallrequest.CCN
		//TODO:Handle response Errors
	
	} catch e {
	
		set tSC = e.AsStatus()
		
	}
	
	quit tSC
}

ClassMethod CreateFHIRMessageBody(pDocumentId As %String, Output pMessageBody As %Stream.GlobalCharacter) As %Status
{

	/*Sample Message Body
	{
	"id": "DocumentReference/31c54d42-174c-4f27-a7e7-9a45271b5613",
	"resourceType": "DocumentReference",
	"text": {
	"fhir_comments": [{
	"div": "This is a text file with seeding request information, caret delimited."
	}]
	},
	"status": "generated",
	"indexed": "2017-08-03T15:06:30:30",
	"type": {
	"coding": [{
	"system": "http://loinc.org<http://loinc.org/> ",
	"code": "LP173405-4"
	}]
	},
	"content": [{
	"attachment": {
	"contentType": "text/plain",
	"data": "",
	"url": "/something/Binary/CCNNAME_Insurance_08072017.txt",
	"language": "en-US",
	"size": 101055,
	"hash": "/vPV/gsSj4MujDJm6TjMOQKyqSc=",
	"title": "The Health-Insurance Related Seeding Request File."
	}
	}]
	}
	*/
	#dim tSC As %Status = $$$OK
	#dim e As %Exception.AbstractException
	
	try {
		set pMessageBody = ##class(%Stream.GlobalCharacter).%New()
		d pMessageBody.Write("{""id"":""DocumentReference/"_pDocumentId_""",")
		d pMessageBody.Write("""resourceType"": ""DocumentReference"",")
		d pMessageBody.Write("""text"": {")
		d pMessageBody.Write("""fhir_comments"": [{")
		d pMessageBody.Write("""div"": ""This is an XML file with CCRA Referral Data.""")
		d pMessageBody.Write("}]")
		d pMessageBody.Write("},")
		d pMessageBody.Write("""status"": ""current"",")
		set tTS = $zts
		set tTSFormatted = $p($zdt(tTS,3,1)," ",1)_"T"_$p($zdt(tTS,3,1)," ",2)
		d pMessageBody.Write("""indexed"": """_tTSFormatted_""",")
		d pMessageBody.Write("""content"": [{")
		d pMessageBody.Write("""attachment"": {")
		d pMessageBody.Write("""contentType"": ""application/json+fhir"",")
		d pMessageBody.Write("""url"": ""/csp/healthshare/ccraint/rest/DAS/Document/"_pDocumentId_"""")
		d pMessageBody.Write("}")
		d pMessageBody.Write("}]")
		d pMessageBody.Write("}")
		
	} catch e {
	
		set tSC = e.AsStatus()
		
	}
	
	quit tSC
}

ClassMethod TestOnRequest()
{
	
	set tProcess = ##class(Ens.Host).GetShadowInstance("DAS.ReferralList.Process")
	set tMsg = ##class(CCRA.Message.ProcessReferralListRequest).%OpenId(44542)
	
	d tProcess.OnInit()
	d tProcess.OnRequest(tMsg,.tResponse)
}

Storage Default
{
<Data name="RTProcessDefaultData">
<Subscript>"RTProcess"</Subscript>
<Value name="1">
<Value>DASOperation</Value>
</Value>
<Value name="2">
<Value>CCNList</Value>
</Value>
<Value name="3">
<Value>DemoMode</Value>
</Value>
<Value name="4">
<Value>JSON</Value>
</Value>
</Data>
<DefaultData>RTProcessDefaultData</DefaultData>
<Type>%Library.CacheStorage</Type>
}

}
