Class ES.Process.ESRequestTransform Extends (Ens.BusinessProcess, HS.Util.Trace.Helper) [ ClassType = persistent ]
{

Property ESRequestOperation As Ens.DataType.ConfigName [ InitialExpression = "ToES GetSummaryRequest" ];

Property HSRMRequestOperation As Ens.DataType.ConfigName [ InitialExpression = "ToHSRM SaveQuestionnaire" ];

Property SendToES As %Boolean [ InitialExpression = 1 ];

Property SendToHsrm As %Boolean [ InitialExpression = 1 ];

Parameter SETTINGS = "ESRequestOperation, HSRMRequestOperation, SendToES, SendToHsrm";

Method OnRequest(pRequest As ES.Message.ESDataRequest) As %Status
{
  
  #dim tSC As %Status = $$$OK
  #dim err As %Exception.AbstractException
  #dim hsrmRequest as ES.HSRM.Request.SaveRequest
  
  try {
    
    s esRequest = ##class(ES.Request.getEESummaryRequest).%New()
    s esRequest.key = pRequest.ICN, esRequest.requestName = pRequest.RequestName
    
    i ..SendToES {
    
      // send ES request
	  s esResponse = ##class(ES.Response.getEESummaryResponse).%New()
	  $$$ThrowOnError(..SendRequestSync(..ESRequestOperation,esRequest,.esResponse))
	  
	  i esResponse.noDataMessage = "Testing, skipped ES request" {return tSC}
	  
	  i esResponse.noDataMessage '= "SOAP Error" {
	  
	    s hsrmRequest = ##class(ES.HSRM.Request.SaveRequest).%New()
	    
	    // transform es response to questionnaire save request
	    $$$ThrowOnError(##class(ES.Transform.FromESToHSRM).Transform(esResponse,.hsrmRequest))	      
	    
	    s hsrmRequest.rs.QUESPAWaitingListDR = pRequest.ReferralId
	    s hsrmRequest.rs.EpisodeID = pRequest.EpisodeId
	    s hsrmRequest.rs.PatientID = pRequest.PatientId
	    
	    ;$$$ThrowOnError(##class(ES.DTL.FromESToHSRM).Transform(esResponse,.hsrmRequest))
	    ;$$$ThrowOnError(##class(ES.DTL.FromInterfaceToHSRM).Transform(pRequest,.hsrmRequest))
	    i ..SendToHsrm {
	    
	      // send questionnaire save request to HSRM operation
	      s hsrmResponse = ##class(ES.HSRM.Response.SaveResponse).%New()
	      $$$ThrowOnError(..SendRequestSync(..HSRMRequestOperation,hsrmRequest,hsrmResponse))
	      
	    }
	  }
    }
    
  } catch err {
    
    $$$LOGERROR("ES/HSRM request failed - error: "_err.DisplayString())
    s tSC = err.AsStatus()
    
  }
  
  q tSC
}

Storage Default
{
<Data name="ESRequestTransformDefaultData">
<Subscript>"ESRequestTransform"</Subscript>
<Value name="1">
<Value>ESRequestOperation</Value>
</Value>
<Value name="2">
<Value>HSRMRequestOperation</Value>
</Value>
<Value name="3">
<Value>ToFileOperation</Value>
</Value>
<Value name="4">
<Value>TraceOperations</Value>
</Value>
<Value name="5">
<Value>SendToHsrm</Value>
</Value>
<Value name="6">
<Value>SendToES</Value>
</Value>
</Data>
<DefaultData>ESRequestTransformDefaultData</DefaultData>
<Type>%Library.CacheStorage</Type>
}

}
