/// transform response from ES system to questionnaire format for HSRM
Class ES.Transform.FromESToHSRM Extends Ens.DataTransform
{

ClassMethod Transform(pSource As ES.Response.getEESummaryResponse, pTarget As ES.HSRM.Request.SaveRequest) As %Status
{
  #dim tSC As %Status = $$$OK
  #dim err As %Exception.AbstractException
  
  // tab1
  #dim sRatedDisabily as ES.sch.ratedDisabilityInfo
  #dim tRatedDisabily as ES.HSRM.s0.TC.api.questionnaire.QUSXXESQQTAB1.Edit
  
  // tab2
  #dim sInsurance as ES.sch.insuranceInfo
  #dim tInsurance as ES.HSRM.s0.TC.api.questionnaire.QUSXXESQQTAB2.Edit
    
  try {
  
    s pTarget = ##class(ES.HSRM.Request.SaveRequest).%New()
    
    s pTarget.rs.QQ1 = pSource.summary.enrollmentDeterminationInfo.priorityGroup
    s pTarget.rs.QQ3 = ..ToCheckbox(pSource.summary.enrollmentDeterminationInfo.specialFactors.agentOrangeInd=1)
    s pTarget.rs.QQ4 = ..ToDisplayDate(pSource.summary.militaryServiceInfo.combatVeteranEligibilityEndDate,5)
    s pTarget.rs.QQ5 = ..ToDisplayDate(pSource.summary.militaryServiceInfo.combatVeteranEligibilityEndDate,5)
    s pTarget.rs.QQ6 = ..ToCheckbox(pSource.summary.militaryServiceInfo.shadIndicator=1)
    s pTarget.rs.QQ7 = ..ToCheckbox(pSource.summary.enrollmentDeterminationInfo.specialFactors.envContaminantsInd=1)
    s pTarget.rs.QQ8 = ..ToCheckbox(pSource.summary.enrollmentDeterminationInfo.specialFactors.radiationExposureInd=1)
    s pTarget.rs.QQ9 = ..ToCheckbox(pSource.summary.enrollmentDeterminationInfo.specialFactors.campLejeuneInd)
    
    s pTarget.rs.QQ13 = pSource.summary.enrollmentDeterminationInfo.militarySexualTraumaInfo.determinationFacility
    s pTarget.rs.QQ14 = pSource.summary.enrollmentDeterminationInfo.militarySexualTraumaInfo.status
    s pTarget.rs.QQ15 = ..ToDisplayDate(pSource.summary.enrollmentDeterminationInfo.militarySexualTraumaInfo.statusChangeDate,5)
    s pTarget.rs.QQ16 = pSource.summary.enrollmentDeterminationInfo.noseThroatRadiumInfo.determinationFacility
    s pTarget.rs.QQ17 = ..ToCheckbox(pSource.summary.enrollmentDeterminationInfo.noseThroatRadiumInfo.diagnosedWithCancer)
    s pTarget.rs.QQ18 = ..ToDisplayDate(pSource.summary.enrollmentDeterminationInfo.noseThroatRadiumInfo.diagnosisVerifiedDate,5)
    s pTarget.rs.QQ19 = ..ToCheckbox(pSource.summary.enrollmentDeterminationInfo.noseThroatRadiumInfo.fromAviatorServiceBefore1955=1)
    s pTarget.rs.QQ20 = ..ToCheckbox(pSource.summary.enrollmentDeterminationInfo.noseThroatRadiumInfo.fromSubmarinerServiceBefore1965=1)
    s pTarget.rs.QQ21 = pSource.summary.enrollmentDeterminationInfo.noseThroatRadiumInfo.receivingTreatment
    s pTarget.rs.QQ22 = ..ToDisplayDate(pSource.summary.enrollmentDeterminationInfo.noseThroatRadiumInfo.verificationDate,5)
    s pTarget.rs.QQ23 = pSource.summary.enrollmentDeterminationInfo.noseThroatRadiumInfo.verificationFacility
    s pTarget.rs.QQ24 = pSource.summary.enrollmentDeterminationInfo.noseThroatRadiumInfo.verificationMethod
    
    // tab1
    s key = ""
	f {
	
	  s sRatedDisabily = pSource.summary.enrollmentDeterminationInfo.serviceConnectionAward.ratedDisabilities.GetNext(.key)
	  q:sRatedDisabily=$$$NULLOREF  ;end of list
	  
	  s tRatedDisabily = ##class(ES.HSRM.s0.TC.api.questionnaire.QUSXXESQQTAB1.Edit).%New()
	  s tRatedDisabily.QTAB1Q10 = sRatedDisabily.disability
	  s tRatedDisabily.QTAB1Q11 = sRatedDisabily.percentage
	  s tRatedDisabily.QTAB1Q12 = sRatedDisabily.serviceConnected
	  
	  $$$ThrowOnError(pTarget.rs.QTAB1.Insert(tRatedDisabily))
	  
	}
	
	// tab2
    s key = ""
	f {
	
	  s sInsurance = pSource.summary.insuranceList.GetNext(.key)
	  q:sInsurance=$$$NULLOREF  ;end of list
	  
	  s tInsurance = ##class(ES.HSRM.s0.TC.api.questionnaire.QUSXXESQQTAB2.Edit).%New()
	  s tInsurance.QTAB2Q25 = sInsurance.companyName
	  s tInsurance.QTAB2Q27 =sInsurance.planType
	  s tInsurance.QTAB2Q28 = sInsurance.policyNumber
	  s tInsurance.QTAB2Q29 = ##class(ES.Util.FunctionSet).DisplayDateFromOther(sInsurance.policyEffectiveDate,5,8)
	  s tInsurance.QTAB2Q30 = ##class(ES.Util.FunctionSet).DisplayDateFromOther(sInsurance.policyExpirationDate,5,8)
	  s tInsurance.QTAB2Q31 = sInsurance.policyHolderName
	  
	  $$$ThrowOnError(pTarget.rs.QTAB2.Insert(tInsurance))
	  	  
	}
    
  } catch err {
  
    s tSC = err.AsStatus()
    
  }
  
  q tSC
}

ClassMethod ToCheckbox(val) As %String
{

  q ##class(ES.Util.FunctionSet).CheckboxFromBoolean(val)
}

ClassMethod ToDisplayDate(val, outFormat) As %String
{

  q ##class(ES.Util.FunctionSet).DisplayDateFromODBC(val,outFormat)
}

}
