Class ES.Util.FunctionSet Extends Ens.Rule.FunctionSet
{

/// Calculate age by passing date of birth in internal format  ( ex.: $ZDH("04/26/2005",1) )
ClassMethod currentAge(date As %Date = "") As %Integer [ CodeMode = expression ]
{
$select(date="":"",1:($ZD($H,8)-$ZD(date,8)\10000))
}

/// If <var>value</var> is equal to <var>1</var>, returns <var>on</var>. Otherwise, returns an empty string.
ClassMethod CheckboxFromBoolean(value As %String = "") As %String [ CodeMode = expression, Final ]
{
$select(value = 1:"on",1:"")
}

/// Convert the given <var>value</var> from ODBC format to specified format <br/>
/// Example ODBC date: 1991-02-20T00:00:00Z <br/>
/// 
/// The <var>outFormat</var> options are the following: <ol>
///   <li>mm/dd/yy (default)</li>
///   <li>yyyy-mm-dd hh:mm:ss</li>
///    <li>mm-dd-yyyy hh:mm:ss</li>
///   <li>horolog/internal format</li>
///   <li>mm/dd/yyyy</li>
/// 		</ol>
/// The <var>includeTime</var> options are the following: <ul>
///   <li>0 - Do not include time (default)</li>
///   <li>1 - Include time</li></ul>
/// If <var>includeTime</var> is selected, the time will be included for those formats with time.
ClassMethod DisplayDateFromODBC(value As %Integer = "", outFormat As %Integer = 1, includeTime As %Boolean = 0) As %String [ Final ]
{
	
  i value && $zdh(value,3,,,,,,,"") {
    
    s ODBCDate = value
      
    s yyyy=$e(ODBCDate,1,4),mm=$e(ODBCDate,6,7),dd=$e(ODBCDate,9,10),hh=$e(ODBCDate,12,13),
      mn=$e(ODBCDate,15,16),ss=$e(ODBCDate,18,19)
    
    i outFormat=1 {  //mm/dd/yy
      s displayDate=mm_"/"_dd_"/"_$e(yyyy,3,4)
    }
    elseif outFormat=2 {  //yyyy-mm-dd hh:mm:ss (time is optional)
      s displayDate=yyyy_"-"_mm_"-"_dd
    }
    elseif outFormat=3 {  //mm-dd-yyyy hh:mm:ss (time is optional)
      s displayDate=mm_"-"_dd_"-"_yyyy
    }
    elseif outFormat = 4 {  // internal format
      s displayDate = $zdh(value,3,,,,,,,"")
      ;s displayDate=59464 ;erroring with this date on hsrm side
    }
    elseif outFormat = 5 {  //mm/dd/yyyy
      s displayDate=mm_"/"_dd_"/"_yyyy
    }

    s:includeTime displayDate = displayDate_" "_hh_":"_mn_":"_ss
  }
  q $g(displayDate)
}

/// Convert the given <var>value</var> from specified inFormat to specified outFormat <br/>
/// 
/// The <var>outFormat</var> options are the following: <ol>
///   <li>mm/dd/yy (default)</li>
///   <li>yyyy-mm-dd hh:mm:ss</li>
///    <li>mm-dd-yyyy hh:mm:ss</li>
///   <li>horolog/internal format</li>
///   <li>mm/dd/yyyy</li>
/// 		</ol>
/// 
/// The <var>inFormat</var> options are the following: <ul>
///   <li>8 - [yy]yymmdd (default)</li>
/// 	</ul>
ClassMethod DisplayDateFromOther(value As %Integer = "", outFormat As %Integer = 1, inFormat As %String = 8) As %String [ Final ]
{
	
	s inFormat = 8
	
	i value && $zdh(value,8,,,,,,,"") {
		s inDate = value
		
		i inFormat = 8 {
			
			s yyyy=$e(inDate,1,*-4), mm=$e(inDate,*-3,*-2), dd=$e(inDate,*-1,*)
			
		}
		
	  i outFormat = 1 {  //mm/dd/yy
	    s displayDate=mm_"/"_dd_"/"_$e(yyyy,3,4)
	  }
	  elseif outFormat = 2 {  //yyyy-mm-dd hh:mm:ss (time is optional)
	    s displayDate=yyyy_"-"_mm_"-"_dd
	  }
	  elseif outFormat = 3 {  //mm-dd-yyyy hh:mm:ss (time is optional)
	    s displayDate=mm_"-"_dd_"-"_yyyy
	  }
	  elseif outFormat = 4 {  // internal format
	    s displayDate = $zdh(inDate,3,,,,,,,"")
	    ;s displayDate=59464 ;erroring with this date on hsrm side
	  }
	  elseif outFormat = 5 {  //mm/dd/yyyy
	    s displayDate=mm_"/"_dd_"/"_yyyy
	  }
	}
	
	q $g(displayDate)
}

}
