/// makes a call to PPMS REST service, returns provider data for providers modified on specified date
Class PPMS.Client.REST.GetModifiedProviders.Operation Extends (EnsLib.REST.Operation, HS.Util.Trace.Helper)
{

/// add all properties from PPMS to provider object
Property ParseAllProperties As %Boolean;

Parameter SETTINGS = "ParseAllProperties";

Parameter INVOCATION = "Queue";

Method getPPMSProviders(pRequest As PPMS.Client.Message.GetProvidersRequest, Output pResponse As PPMS.Client.Message.GetProvidersResponse) As %Status
{
  
  #dim tSC As %Status = $$$OK
  #dim err AS %Exception.AbstractException
  #dim tProvider As PPMS.Client.Provider
  
  try {
  
    s pResponse = ##class(PPMS.Client.Message.GetProvidersResponse).%New()
    
    // build URL
    // Format:  https://np.dws.ppms.DOMAIN.EX/v1.0/GetProviderByModifiedOnDate?modifiedOnDate=2018-02-08
    // Append the last update date to the URL configured for adapter - Date must be in ODBC Format (YYYY-MM-DD)
    
    s tURL=..Adapter.URL_"?modifiedOnDate="_$zd(pRequest.ModifiedOnDate,3)
    
    // execute call
    s tSC = ..Adapter.GetURL(tURL,.tHttpResponse)
    
    i ($$$ISERR(tSC)) && ($IsObject(tHttpResponse)) && ($IsObject(tHttpResponse.Data)) && (tHttpResponse.Data.Size) {
        
      s tSC=$$$ERROR($$$EnsErrGeneral,$$$StatusDisplayString(tSC)_":"_tHttpResponse.Data.Read())
      
    }
    
    i tSC["No Providers found modified on this date" {
    
      s tSC = $$$OK
      
    } else {
    
      $$$ThrowOnError(tSC)
      
      i $IsObject(tHttpResponse) {
      
        $$$HSTRACE("tHttpResponse","tHttpResponse",tHttpResponse.Data)
        
        // Convert JSON into a Proxy Cache Array
        s tJSONArray={}.%FromJSON(tHttpResponse.Data)
          
        // Get Iterator for each provider in the JSON Stream (each "value")
        s tIter = tJSONArray.%Get("value").%GetIterator()
        s key = ""
        
        // Iterate thru the "value" collection to create a Provider object for each Provider.
        while tIter.%GetNext(.key) {
        
          s tProvider = ##class(PPMS.Client.Provider).%New()
          s tProvider.NPI = tJSONArray.%Get("value").%Get(key).%Get("ProviderIdentifier")
          s tProvider.Name = tJSONArray.%Get("value").%Get(key).%Get("Name")
          s tProvider.OnLeie = tJSONArray.%Get("value").%Get(key).%Get("OnLeie")
          s tProvider.ModifiedOnDate = tJSONArray.%Get("value").%Get(key).%Get("ModifiedOnDate")
          s address = tJSONArray.%Get("value").%Get(key).%Get("Address"), address = $zstrip(address,"<>W")
          i $l(address) {
          
            s address = $e(address,1,49)
            s tProvider.AddressStreet = $zstrip($p(address,$char(10),1),"<>W")
            s cityStateZip = $p(address,$char(10),2), city = $p(cityStateZip,",",1), city = $zstrip(city,"<>W")
            s tProvider.AddressCity = city
            s stateZip = $p(cityStateZip,",",2)
            s state = $p(stateZip," ",2), state = $zstrip(state,"<>W")
            s tProvider.AddressState = state
            s zip =  $p(stateZip," ",3), zip = $zstrip(zip,"<>W")
            s tProvider.AddressZipCode = zip
            
          }
          
          // affiliations
          s tProvider.HealthNetAffiliation = tJSONArray.%Get("value").%Get(key).%Get("HealthNetAffiliation")
          s tProvider.ProviderAgreementAffiliation = tJSONArray.%Get("value").%Get(key).%Get("ProviderAgreementAffiliation")
          s tProvider.Pc3Affiliation = tJSONArray.%Get("value").%Get(key).%Get("Pc3Affiliation")
          s tProvider.DodAffiliation = tJSONArray.%Get("value").%Get(key).%Get("DoDAffiliation")
          s tProvider.VaAcademicAffiliate = tJSONArray.%Get("value").%Get(key).%Get("VaAcademicAffiliation")
          s tProvider.TriWestAffiliation = tJSONArray.%Get("value").%Get(key).%Get("TriWestAffiliation")
          s tProvider.IhsAffiliation = tJSONArray.%Get("value").%Get(key).%Get("IhsAffiliation")
          s tProvider.ChoiceAffiliation = tJSONArray.%Get("value").%Get(key).%Get("ChoiceAffiliation")
          
          ;the following properties aren't currently needed
          i ..ParseAllProperties {
              
            s tProvider.ProviderType = tJSONArray.%Get("value").%Get(key).%Get("ProviderType")
            s tProvider.QualityRankingTotalScore = tJSONArray.%Get("value").%Get(key).%Get("QualityRankingTotalScore")
            // Property not in Provider class
            //s tProvider.QualityRankingLastUpdated = tJSONArray.%Get("value").%Get(key).%Get("QualityRankingLastUpdate")
            s tProvider.MainPhone = tJSONArray.%Get("value").%Get(key).%Get("MainPhone")
            s tProvider.Email = tJSONArray.%Get("value").%Get(key).%Get("Email")                
            s tProvider.ProviderStatusReason = tJSONArray.%Get("value").%Get(key).%Get("ProviderStatusReason")
            s tProvider.PrimaryCarePhysician = tJSONArray.%Get("value").%Get(key).%Get("PrimaryCarePhysician")
            s tProvider.HighPerformance = tJSONArray.%Get("value").%Get(key).%Get("HighPerformance")
            s tProvider.Fqhc = tJSONArray.%Get("value").%Get(key).%Get("Fqhc")
            s tProvider.AccountableCareOrganization = tJSONArray.%Get("value").%Get(key).%Get("AccountableCareOrganization")
            s tProvider.IsAcceptingNewPatients = tJSONArray.%Get("value").%Get(key).%Get("IsAcceptingNewPatients")
            s tProvider.ProviderEthnicity = tJSONArray.%Get("value").%Get(key).%Get("ProviderEthnicity")
            s tProvider.ProviderGender = tJSONArray.%Get("value").%Get(key).%Get("ProviderGender")
            s tProvider.Religion = tJSONArray.%Get("value").%Get(key).%Get("Religion")
            s tProvider.OrganizationId = tJSONArray.%Get("value").%Get(key).%Get("OrganizationId")
            s tProvider.ServiceProviderType = tJSONArray.%Get("value").%Get(key).%Get("ServiceProviderType")
            s tProvider.SpecialInstruction = tJSONArray.%Get("value").%Get(key).%Get("SpecialInstruction")
            s tProvider.OwnedCareSiteName = tJSONArray.%Get("value").%Get(key).%Get("OwnedCareSiteName")
            s tProvider.OrganizationFax = tJSONArray.%Get("value").%Get(key).%Get("OrganizationFax")
            s tProvider.OrganizationStatus = tJSONArray.%Get("value").%Get(key).%Get("OrganizationStatus")
            s tProvider.IsExternal = tJSONArray.%Get("value").%Get(key).%Get("IsExternal")
            s tProvider.InternalType = tJSONArray.%Get("value").%Get(key).%Get("InternalType")
            s tProvider.LicensingJuristicion = tJSONArray.%Get("value").%Get(key).%Get("LicensingJuristicion")
            s tProvider.CanCreateHealthCareOrders = tJSONArray.%Get("value").%Get(key).%Get("CanCreateHealthcareOrders")
            s tProvider.InternalAppointmentStatus = tJSONArray.%Get("value").%Get(key).%Get("InternalAppointmentStatus")
            s tProvider.ExternalHealthProviderType = tJSONArray.%Get("value").%Get(key).%Get("ExternalHealthProviderType")
            s tProvider.ExternalInstitutionDeaNumber = tJSONArray.%Get("value").%Get(key).%Get("ExternalInstitutionDeaNumber")
            s tProvider.ExternalLeieCheckDate = tJSONArray.%Get("value").%Get(key).%Get("ExternalLeieCheckDate")
            s tProvider.ValidationSource = tJSONArray.%Get("value").%Get(key).%Get("ValidationSource")
            s tProvider.ContactMethod = tJSONArray.%Get("value").%Get(key).%Get("ContactMethod")
            s tProvider.BulkEmails = tJSONArray.%Get("value").%Get(key).%Get("BulkEmails")
            s tProvider.BulkMails = tJSONArray.%Get("value").%Get(key).%Get("BulkMails")
            s tProvider.Emails = tJSONArray.%Get("value").%Get(key).%Get("Emails")
            s tProvider.Mails = tJSONArray.%Get("value").%Get(key).%Get("Mails")
            s tProvider.PhoneCalls = tJSONArray.%Get("value").%Get(key).%Get("PhoneCalls")
            s tProvider.Faxes = tJSONArray.%Get("value").%Get(key).%Get("Faxes")
            s tProvider.NpiKey = tJSONArray.%Get("value").%Get(key).%Get("NpiKey")
            
          }
          
          //Add Provider to Response collection
          d pResponse.Providers.SetAt(tProvider,key)
          
        }
      }
        
      $$$ThrowOnError(pResponse.%Save())
    }
  } catch err {
    s tSC = err.AsStatus()
  }
  
  quit tSC
}

XData MessageMap
{
<MapItems>
  <MapItem MessageType="PPMS.Client.Message.GetProvidersRequest">
    <Method>getPPMSProviders</Method>
  </MapItem>
</MapItems>
}

}
T