Class PPMS.Client.REST.GetProviderSpecialties.Operation Extends (EnsLib.REST.Operation, HS.Util.Trace.Helper)
{

/// add all properties from PPMS to specialty obj
Property ParseAllProperties As %Boolean;

Parameter SETTINGS = "ParseAllProperties";

Parameter INVOCATION = "Queue";

Method getProviderSpecialities(pRequest As PPMS.Client.Message.GetSpecialtiesRequest, Output pResponse As PPMS.Client.Message.GetSpecialtiesResponse) As %Status
{
  
  #dim tSC As %Status = $$$OK
  #dim err AS %Exception.AbstractException
  #dim tSpecialty As PPMS.Client.Specialty
  
  try {
  
    s pResponse = ##class(PPMS.Client.Message.GetSpecialtiesResponse).%New()
    
    // build URL
    // Format:  https://np.dws.ppms.DOMAIN.EX/v1.0/Providers(1770995755)/ProviderSpecialties
    
    s tURL = ..Adapter.URL
    s tURL = $Replace(tURL,"(inputNPI)","("_pRequest.NPI_")")
    
    // execute call
    s tSC = ..Adapter.GetURL(tURL,.tHttpResponse)
    
    i $$$ISERR(tSC) && $IsObject(tHttpResponse) && $IsObject(tHttpResponse.Data) && tHttpResponse.Data.Size {
      
      s tSC=$$$ERROR($$$EnsErrGeneral,$$$StatusDisplayString(tSC)_":"_tHttpResponse.Data.Read())
      
    }
    
    i tSC["Found" {
    
      s tSC = $$$OK
      
    } else {
    
      $$$ThrowOnError(tSC)
      
      i $IsObject(tHttpResponse) {
      
        $$$HSTRACE("tHttpResponse","tHttpResponse",tHttpResponse.Data)
        
        // Convert JSON into a Proxy Cache Array
        s tJSONArray={}.%FromJSON(tHttpResponse.Data)
        
        // Get Iterator for each provider in the JSON Stream (each "value")
        s tIter = tJSONArray.%Get("value").%GetIterator()
        s tKey = ""
        
        // Iterate thru the "value" collection to create a specialty object for each provider.
        while tIter.%GetNext(.key) {
          
          s tSpecialty = ##class(PPMS.Client.Specialty).%New()
          s tSpecialty.CodedSpecialty = tJSONArray.%Get("value").%Get(key).%Get("CodedSpecialty")
          s tSpecialty.IsPrimaryType = tJSONArray.%Get("value").%Get(key).%Get("IsPrimaryType")
          s tSpecialty.Name = tJSONArray.%Get("value").%Get(key).%Get("Name")
          
          ;the following properties aren't currently needed
          i ..ParseAllProperties {
            
            s tSpecialty.OwnerName = tJSONArray.%Get("value").%Get(key).%Get("OwnerName")
            s tSpecialty.ProviderName = tJSONArray.%Get("value").%Get(key).%Get("ProviderName")
            s tSpecialty.SpecialtyName = tJSONArray.%Get("value").%Get(key).%Get("SpecialtyName")
            
          }
          
          // add specialty to response collection
          s tSC = pResponse.Specialties.Insert(tSpecialty)
          
        }
          
      }
        
      ;$$$ThrowOnError(pResponse.%Save())
      
    }
    
  } catch err {
      
    s tSC = err.AsStatus()
    
  }
  
  quit tSC
}

XData MessageMap
{
<MapItems>
  <MapItem MessageType="PPMS.Client.Message.GetSpecialtiesRequest">
    <Method>getProviderSpecialities</Method>
  </MapItem>
</MapItems>
}

}
T