Class PPMS.Maintenance.Process Extends (Ens.BusinessProcess, HS.Util.Trace.Helper) [ ClassType = persistent ]
{

Property GetModifiedProvidersOperation As Ens.DataType.ConfigName [ InitialExpression = "ToPPMS GetModifiedProviders" ];

Property GetProviderSpecialitiesOperation As Ens.DataType.ConfigName [ InitialExpression = "ToPPMS GetProviderSpecialties" ];

Property HSRMOperation As Ens.DataType.ConfigName [ InitialExpression = "ToHSRM SaveProviderRequest" ];

Property GetSpecialties As %Boolean [ InitialExpression = 1 ];

Property SendToHsrm As %Boolean [ InitialExpression = 1 ];

Parameter SETTINGS = "GetModifiedProvidersOperation, GetProviderSpecialitiesOperation, HSRMOperation, GetSpecialties, SendToHsrm";

Method OnRequest(pRequest As PPMS.Client.Message.GetProvidersRequest) As %Status
{
	
  #dim tSC As %Status = $$$OK
  #dim err As %Exception.AbstractException
  #dim tProvider as PPMS.Client.Message.GetSpecialtiesResponse
  #dim tTransformedProviders as PPMS.SDA3.CareProvider
  #dim specialtiesResponse as PPMS.Client.Message.GetSpecialtiesResponse
  #dim providersResponse as PPMS.Client.Message.GetProvidersResponse
  
  try {
    
    // get PPMS Updates via RESTful Service
    s providersResponse = ##class(PPMS.Client.Message.GetProvidersResponse).%New()
    $$$ThrowOnError(..SendRequestSync(..GetModifiedProvidersOperation,pRequest,.providersResponse))
    
    i 'providersResponse.Providers.Count() {
    
      $$$LOGINFO("No providers found for ModifiedOnDate: "_$zd(pRequest.ModifiedOnDate,3))
      
    } else {
	  $$$LOGINFO("Provider count: "_providersResponse.Providers.Count())
      i ..GetSpecialties {
      
        s pKey = ""
        f {
        
          s tProvider = providersResponse.Providers.GetNext(.pKey)
          
          q:tProvider=$$$NULLOREF  ;end of list
          continue:tProvider.NPI="" ;no NPI, skip provider
          
          s specialtiesRequest = ##class(PPMS.Client.Message.GetSpecialtiesRequest).%New()
          s specialtiesRequest.NPI = tProvider.NPI
          
          s specialtiesResponse = ##class(PPMS.Client.Message.GetSpecialtiesResponse).%New()
          $$$ThrowOnError(..SendRequestSync(..GetProviderSpecialitiesOperation,specialtiesRequest,.specialtiesResponse))
          
          s sKey = ""
          f {
          
            s tSpecialty = specialtiesResponse.Specialties.GetNext(.sKey)
            q:tSpecialty=$$$NULLOREF  ;end of list
            
            s specialtyCode = tSpecialty.CodedSpecialty
            s specialtyName = tSpecialty.Name
            
            i $l(specialtyCode) || $l(specialtyName) {
            
              s specialty = ##class(HS.SDA3.CodeTableDetail).%New()
              s specialty.Code = specialtyCode
              s specialty.Description = specialtyName
              s tSC = tProvider.Speciality.Insert(specialty)
              
            }
          }
          
        }
      }
      
      // transform client provider to SDA provider
      s tTransformedProviders = ##class(PPMS.SDA3.CareProvider).%New()
      $$$ThrowOnError(##class(PPMS.Transform.FromClientProviderToSDA3CareProvider).Transform(providersResponse,.tTransformedProviders))
      
      s tProvidersRequest = ##class(PPMS.Message.ProvidersRequest).%New()
      s tProvidersRequest.Providers = tTransformedProviders
      s tResponse = ##class(Ens.Response).%New()
      
      i ..SendToHsrm {
	      
	      // send to HSRM operation
	      $$$ThrowOnError(..SendRequestSync(..HSRMOperation,tProvidersRequest,.tResponse))
	      
      }
      
      //Log Completion to Event Log
      $$$LOGINFO("PPMS update successful for ModifiedOnDate: "_$zd(pRequest.ModifiedOnDate,3))
      
    }
    
  } catch err {
    
    $$$LOGALERT("PPMS Update failed for ModifiedOnDate: "_$zd(pRequest.ModifiedOnDate,3))
    s tSC = err.AsStatus()
    
  }
  
  q tSC
}

Storage Default
{
<Data name="ProcessDefaultData">
<Subscript>"Process"</Subscript>
<Value name="1">
<Value>PPMSOperation</Value>
</Value>
<Value name="2">
<Value>HSRMOperation</Value>
</Value>
<Value name="3">
<Value>TraceOperations</Value>
</Value>
<Value name="4">
<Value>SendToHsrm</Value>
</Value>
<Value name="5">
<Value>PPMSGetModifiedProvidersOperation</Value>
</Value>
<Value name="6">
<Value>PPMSProviderSpecialitiessOperation</Value>
</Value>
<Value name="7">
<Value>GetModifiedProvidersOperation</Value>
</Value>
<Value name="8">
<Value>GetProviderSpecialitiesOperation</Value>
</Value>
<Value name="9">
<Value>GetSpecialties</Value>
</Value>
</Data>
<DefaultData>ProcessDefaultData</DefaultData>
<Type>%Library.CacheStorage</Type>
}

}
