/// Transform response from the ClientProvider to SDA3CareProvider
Class PPMS.Transform.FromClientProviderToSDA3CareProvider Extends Ens.DataTransform
{

ClassMethod Transform(pSource As PPMS.Client.Message.GetProvidersResponse, pTarget As PPMS.SDA3.CareProvider) As %Status
{
  #dim tSC As %Status = $$$OK
  #dim err As %Exception.AbstractException
  #dim sourceProvider as PPMS.Client.Provider
  #dim tProvider as HS.SDA3.CodeTableDetail.CareProvider
  #dim sourceSpecialty as HS.SDA3.CodeTableDetail
  #dim tSpecialty as HS.SDA3.CodeTableDetail
  
  try {
  
    s pTarget = ##class(PPMS.SDA3.CareProvider).%New()
    
    s pKey = ""
    f {
    
      s sourceProvider = pSource.Providers.GetNext(.pKey)
      q:sourceProvider=$$$NULLOREF  ;end of list
      
      s tProvider = ##class(HS.SDA3.CodeTableDetail.CareProvider).%New()
      s tProvider.CareProviderType.Description = "Provider"
      s tProvider.Code = sourceProvider.NPI
      
      // parse name
      s name = sourceProvider.Name, name = $zstrip(name,"*C")
      i name["," {  ; names contains a comma
      
        s familyName = $p(name,","), familyName = $zstrip(familyName,"<>W")
        s tProvider.Name.FamilyName = familyName
        s givenName = $p(name,",",2), givenName = $zstrip(givenName,"<>W")
        s tProvider.Name.GivenName = givenName
        s tProvider.Description = $g(givenName)_" "_$g(familyName)
       
      } else { ; no comma in name, use name as received for description
        s tProvider.Description = name
      }
      
      s tProvider.Address.Street = sourceProvider.AddressStreet
      s tProvider.Address.City.Description = sourceProvider.AddressCity
      s tProvider.Address.State.Code = sourceProvider.AddressState
      
      ;to-do, will this be changed to a list?
      // add affiliation
      s tProvider.CareProviderGroup.Description = ..GetAffiliation(sourceProvider)
      
      // if OnLEIE, set ToTime to modified date/time, else set to future date
      i sourceProvider.OnLeie {
        s tProvider.ToTime = sourceProvider.ModifiedOnDate
      } else {
        s tProvider.ToTime = $zdt("94234,00000",3,1)
      }
      
      s sKey = ""
      // insert all specialties
      f {
      
        s sourceSpecialty = sourceProvider.Speciality.GetNext(.sKey)
        q:((sourceSpecialty=$$$NULLOREF) || (sourceSpecialty.Code = ""))
        
        s tSpecialty = ##class(HS.SDA3.CodeTableDetail).%New()
        
        s tSpecialty.Code = ##class(Ens.Util.FunctionSet).Lookup("CCRA.Lookup.PPMS.SpecialtyCrosswalk", sourceSpecialty.Code)
        continue:(tSpecialty.Code = "") ;invalid/unmapped code
        ;s tSpecialty.Code = sourceSpecialty.Code
        ;s tSpecialty.Description = sourceSpecialty.Description
        
        $$$ThrowOnError(tProvider.Speciality.Insert(tSpecialty))
        
      }
      
      $$$ThrowOnError(pTarget.CareProviders.Insert(tProvider))
            
    }
    
  } catch err {
  
    s tSC = err.AsStatus()
        
  }
    
  q tSC
}

ClassMethod GetAffiliation(sourceProvider As PPMS.Client.Provider) As %String
{

  i sourceProvider.HealthNetAffiliation {
        s affiliation = "HealthNetAffiliation"
    }
    i sourceProvider.ProviderAgreementAffiliation {
        s affiliation = "ProviderAgreementAffiliation"
    }
    i sourceProvider.Pc3Affiliation {
        s affiliation = "Pc3Affiliation"
    }
    i sourceProvider.DodAffiliation {
        s affiliation = "DodAffiliation"
    }
    i sourceProvider.TriWestAffiliation {
        s affiliation = "TriWestAffiliation"
    }
    i sourceProvider.IhsAffiliation {
        s affiliation = "IhsAffiliation"
    }
    i sourceProvider.ChoiceAffiliation {
        s affiliation = "ChoiceAffiliation"
    }
    
    q $g(affiliation)
}

}
