Class RealTime.Process.ExtractTransformReferral Extends (Ens.BusinessProcess, HS.Util.Trace.Helper) [ ClassType = persistent ]
{

Property HSRMTarget As Ens.DataType.ConfigName [ InitialExpression = "HSRM.ReferralDataExtraction.WS.Client" ];

Property eCAMSTarget As Ens.DataType.ConfigName [ InitialExpression = "eCAMS.Operation.ReferralDocumentToeCAMS" ];

Property DASTarget As Ens.DataType.ConfigName [ InitialExpression = "DAS.ReferralList.RTProcess" ];

Property SEOCRESTClient As Ens.DataType.ConfigName [ InitialExpression = "SEOCDB.REST.Client" ];

Parameter SETTINGS = "HSRMTarget,eCAMSTarget,DASTarget";

Method OnRequest(pInputReferralID As Ens.StringContainer, Output pResponse As Ens.Response) As %Status
{
 
	#dim tSC As %Status = $$$OK
	#dim e As %Exception.AbstractException
	#dim tRetrieveReferralResponse AS CCRA.Message.RetrieveReferralResponse
	#dim tSEOCRequest AS SEOC.Maintenance.DB.Message.RetrieveSEOCRequest
	#dim tRESTClientResponse As EnsLib.HTTP.GenericMessage
	#dim teCAMSRequest AS eCAMS.Message.eCAMSReferralRequest
	#dim tSEOCRESTClientResponse as EnsLib.HTTP.GenericMessage
	#dim ReferralIdMode as Integer = 1 //Assuming Referral ID is sent for now, which uses mode 1
	#dim tReferralStatus as %String = "" 	//status of referral, used to determine where to send messages
	#dim tNetworkCode as %String = "" 	//Network code referral is assigned to, used to determine where to send messages
	#dim tNetwork as %String = ""		//translated Network value (i.e. DAS or FSC) after NetworkCode is looked up in CCRA.Lookup.ReferralNetworkRouting	
	#dim tNetWorkReferralStatus as %String = ""	 //combo of network and status to be looked up in CCRA.Lookup.ReferralStatus to determine if status/network is a valid combo
	#dim tValidStatus as %String = "" // output of CCRA.Lookup.ReferralStatus for message routing
	
	try {		
		//Get referral data from HSRM web service
		set tRetrieveReferralRequest = ##class(CCRA.Message.RetrieveReferralRequest).%New()
		set tRetrieveReferralRequest.Id = pInputReferralID.StringValue
		set tRetrieveReferralRequest.Mode = ReferralIdMode 				
		do ..SendRequestSync(..HSRMTarget,tRetrieveReferralRequest,.tRetrieveReferralResponse, 30)

		//Get SEOC data or act accordingly if not available 
		if $IsObject(tRetrieveReferralResponse.RetrieveReferralResult) {	
		
			//Build JSON message body to include referral and SEOC sections if available
			set tNetworkCode = tRetrieveReferralResponse.RetrieveReferralResult.NetworkCode	
			set tReferralStatus = tRetrieveReferralResponse.RetrieveReferralResult.Status	
			set tRetrieveReferralResponse.RetrieveReferralResult.TreatmentStartDate = $ZDATE(tRetrieveReferralResponse.RetrieveReferralResult.TreatmentStartDate, "3") //reformat date to be YYYY-MM-DD
			set tMessageBody = ##class(%ZEN.proxyObject).%New()
			set tMessageBody.Referral = tRetrieveReferralResponse.RetrieveReferralResult
		
			//8-21-18  JLM Adding logic to pull local copy of SEOC rather than using API call 
			try{ 
				set tsc = ..RetreiveSEOCFromSQL(tRetrieveReferralResponse.RetrieveReferralResult.SeocID, .tSEOCData)
				set tMessageBody.SEOC = tSEOCData
			}
			catch(e){
				set tSC = e.AsStatus()	
			}
			
			//Add JSON message body to eCAMS Request Message
			set tSC = ##class(%ZEN.Auxiliary.jsonProvider).%WriteJSONStreamFromObject(.tJSONStream,tMessageBody,,,,"tu")

			//Send referral message to appropriate operation based on network code
			&sql(SELECT DataValue INTO :tNetwork  FROM Ens_Util.LookupTable WHERE TableName='CCRA.Lookup.ReferralNetworkRouting' AND KeyName=:tNetworkCode)	
			
			 //SEND ALL REFERRALS TO ECAMS not in a status of initial, suspended, or approved
			set tNetWorkReferralStatus = "FSC|" _ tReferralStatus
			&sql(SELECT ISNULL(DataValue, 'Y') INTO :tValidStatus  FROM Ens_Util.LookupTable WHERE TableName='CCRA.Lookup.ReferralStatus' AND KeyName=:tNetWorkReferralStatus)				
			if (tValidStatus ="Y"){ 
				set teCAMSRequest = ##class(eCAMS.Message.eCAMSReferralRequest).%New()			
				set teCAMSRequest.DocumentId = pInputReferralID.StringValue
				set teCAMSRequest.Stream = ##class(%Stream.GlobalCharacter).%New()					
				$$$ThrowOnError(teCAMSRequest.Stream.CopyFrom(tJSONStream))
				$$$ThrowOnError(..SendRequestAsync(..eCAMSTarget,teCAMSRequest,.tResponse))
			}
			
			//handle the rest of the vendors based on network and status
			set tNetWorkReferralStatus = tNetwork _ "|" _ tReferralStatus
			set tValidStatus = "N"
			&sql(SELECT DataValue INTO :tValidStatus  FROM Ens_Util.LookupTable WHERE TableName='CCRA.Lookup.ReferralStatus' AND KeyName=:tNetWorkReferralStatus)	
			if (tValidStatus ="Y"){
				if (tNetwork="DAS") { 
					set tDASRequest = ##class(DAS.Message.ReferralRequest).%New()			
					set tDASRequest.NetworkCode = tNetworkCode
					set tDASRequest.Stream = ##class(%Stream.GlobalCharacter).%New()					
					$$$ThrowOnError(tDASRequest.Stream.CopyFrom(tJSONStream))
					$$$ThrowOnError(..SendRequestAsync(..DASTarget,tDASRequest,.tResponse))		
				}
				//add more networks as needed
			}
		

		}	
		else { 	
			$$$LOGERROR("Error received trying to retrieve referral.")
		}  
		
	}
	catch e {
		set tSC = e.AsStatus()	
	}
	
	quit tSC
}

Method RetreiveSEOCFromSQL(pSEOCExternalID As %String, Output tSEOC As SEOC.Maintenance.DB.OBJ.SEOC) As %Status
{
	/*
		8-22-18   JLM   Story #808168
		receives external SEOC ID
		looks up SEOC data in HSIE database, SEOC.Maintenance.DB.OBJ.SEOC
		this table is refreshed nightly
	*/
	
	#dim tSC As %Status = $$$OK
	
	try{
		&sql(SELECT ID1 INTO :tID FROM SEOC_Maintenance_DB_OBJ.SEOC WHERE seocId = :pSEOCExternalID)
		if '(tID="") {
				set tSEOC = ##class(SEOC.Maintenance.DB.OBJ.SEOC).%OpenId(tID)
		}
	}
	catch e{
		set tSC = e.AsStatus()	
		$$$LOGERROR("Error while retrieving SEOC: "_tRetrieveReferralResponse.RetrieveReferralResult.SeocID _ $char(10) _ tSC)
	}
	
	quit tSC
}

Method OnResponse(request As %Library.Persistent, ByRef response As %Library.Persistent, callrequest As %Library.Persistent, callresponse As %Library.Persistent, pCompletionKey As %String) As %Status
{
	
	Quit $$$OK
}

Storage Default
{
<Data name="ExtractTransformReferralDefaultData">
<Subscript>"ExtractTransformReferral"</Subscript>
<Value name="1">
<Value>HSRMTarget</Value>
</Value>
<Value name="2">
<Value>TraceOperations</Value>
</Value>
<Value name="3">
<Value>DASTarget</Value>
</Value>
<Value name="4">
<Value>SEOCRESTClient</Value>
</Value>
<Value name="5">
<Value>eCAMSTarget</Value>
</Value>
<Value name="6">
<Value>eCAMSStatus</Value>
</Value>
</Data>
<DefaultData>ExtractTransformReferralDefaultData</DefaultData>
<Type>%Library.CacheStorage</Type>
}

}
