Class SEOC.Maintenance.DB.API.Process Extends (Ens.BusinessProcess, HS.Util.Trace.Helper) [ ClassType = persistent ]
{

Property SEOCRESTClient As Ens.DataType.ConfigName [ InitialExpression = "SEOCDB.REST.Client" ];

Property SEOCWSClient As Ens.DataType.ConfigName [ InitialExpression = "HSRM.SEOC.WS.Client" ];

Parameter SETTINGS = "SEOCRESTClient, SEOCWSClient";

Method OnRequest(pRequest As SEOC.Maintenance.DB.Message.RetrieveSEOCRequest, Output pResponse As Ens.Response) As %Status
{
	#dim tSC As %Status = $$$OK
	#dim e As %Exception.AbstractException
	#dim tRESTClientResponse As EnsLib.HTTP.GenericMessage
	#dim tJSONObj As %DynamicAbstractObject
	#dim tSEOC As %DynamicObject
	
	try {
		
		//Forward request to REST Client
		$$$ThrowOnError(..SendRequestSync(..SEOCRESTClient,pRequest,.tRESTClientResponse,60))
		
		//If SEOC data received, create Dynamic Object
		set tJSONObj = ##class(%DynamicObject).%FromJSON(tRESTClientResponse.Stream)
		
		//For each SEOC returned: 
		//	1. extract Dynamic Object
		//  2. parse into SEOC
		//  3. transform in HSRM SEOC
		//	4. add/update HISE database table with SEOC
		//  5. forward to HSRM

		for i=0:1:tJSONObj.%Get("Seocs").%Size()-1 {
			
			//Extract and parse JSON into SEOC object
			set tSEOC = ##class(SEOC.Maintenance.DB.OBJ.SEOC).CreateFromDynamicObj(tJSONObj.%Get("Seocs").%Get(i).%Get("Seoc"))	
			$$$HSTRACE("After Extracting SEOC","tSEOC",tSEOC)
			
			if '(tSEOC="") {
				
				//transform to HSRM SEOC
				$$$ThrowOnError(##class(SEOC.Maintenance.DB.Transform.SEOCDBtoHSRMSEOC).Transform(tSEOC,.tHSRMSEOC))
				
				//8-14-18 JLM  Add SEOC DB API internal ID and SEOC ID to lookup table, story #800107
				//8-22-18  Replaced with SQL table, story #808168
				//set uSEOCID = $ZCONVERT(tSEOC.seocId, "U")
				//do ##class(Ens.Util.LookupTable).%UpdateValue("SEOC_ID", uSEOCID , tSEOC.id, 1)
				
				//8-21-18 JLM  Save SEOCs to HSIE database nightly
				set internalID = tSEOC.id
				&sql(SELECT ID1 INTO :iID
					FROM SEOC_Maintenance_DB_OBJ.SEOC 
					WHERE id = :internalID)		
				
				if (SQLCODE=100){  
					set oSEOC = ##class(SEOC.Maintenance.DB.OBJ.SEOC).%New()
					set oSEOC = tSEOC 
				//	set sPAL = tSEOC.PAL  // 9-7-18 JLM  SEOC returns 0/1, schema states false/true
				//	if (sPAL = "0")
				//		set sPAL = "FALSE"
				//	else
				//		set sPAL = "TRUE"
				//	$$$HSTRACE("Saving SEOC","tSEOC.PAL",sPAL)
				//	set tSEOC.PAL = sPAL
					set tSEOC.seocId = $ZCONVERT(tSEOC.seocId, "U")
					try{
						do tSEOC.%Save()   //raise alert if save fails, but continue one
					}
					catch ex{
						$$$LOGINFO("Failed to save SEOC to the database: " _ tSEOC.seocId _ "\n Error: " _ ex.AsStatus())
					}
				}
				
				$$$HSTRACE("After Transformation","tHSRMSEOC",tHSRMSEOC)
				
				//Invoke HSRM SEOC API
				set tSEOCUploadRequest = ##class(SEOC.Maintenance.Message.MainRequest).%New()
				set tSEOCUploadRequest.SEOCData = tHSRMSEOC
				set tSEOCUploadSC = ..SendRequestSync(..SEOCWSClient,tSEOCUploadRequest,.tSEOCUploadResponse,15)
				
				//Log Error if captured from the SendRequestSync call
				if $$$ISERR(tSEOCUploadSC) {
					$$$LOGSTATUS(tSEOCUploadSC)
				} 			
			}
			
			
		}		
	} catch e {
		set tSC = e.AsStatus()
	}
	
	quit tSC
}

Storage Default
{
<Data name="ProcessDefaultData">
<Subscript>"Process"</Subscript>
<Value name="1">
<Value>SEOCRESTClientTarget</Value>
</Value>
<Value name="2">
<Value>JSONParserTarget</Value>
</Value>
<Value name="3">
<Value>SEOCTransformTarget</Value>
</Value>
<Value name="4">
<Value>TraceOperations</Value>
</Value>
<Value name="5">
<Value>SEOCWSClient</Value>
</Value>
<Value name="6">
<Value>SEOCRESTClient</Value>
</Value>
</Data>
<DefaultData>ProcessDefaultData</DefaultData>
<Type>%Library.CacheStorage</Type>
}

}
