Class SEOC.Maintenance.DB.Operation.RetrieveSEOC Extends (EnsLib.REST.GenericOperation, HS.Util.Trace.Helper)
{

Parameter INVOCATION = "Queue";

Property ServiceName As %String [ InitialExpression = "SEOC.DB.API" ];

Parameter SETTINGS = "ServiceName:Basic";

Method OnInit() As %Status
{
	set ..%HttpRequest = ##class(%Net.HttpRequest).%New()
	Quit $$$OK
}

Method OnMessage(pRequest As SEOC.Maintenance.DB.Message.RetrieveSEOCRequest, Output pResponse As EnsLib.HTTP.GenericMessage) As %Status
{
 
	#dim tSC As %String = $$$OK 
	#dim tHttpResponse As %Net.HttpResponse = ##class(%Net.HttpResponse).%New()
	#dim tURL as %String
	
	try{

  		//Set custom SEOC headers
  		do ..%HttpRequest.SetHeader("Accept", "*/*")
  		
		//Get WS Location from HS Service Registry
		Set tEndPoint = ##class(HS.Registry.Service.HTTP).EndPointForNameType(..ServiceName,"HTTP")
		If $isObject(tEndPoint) {
			Set ..Adapter.URL = tEndPoint.URL 
			Set ..Adapter.HTTPServer = tEndPoint.Host
			Set ..Adapter.HTTPPort = tEndPoint.Port
			Set ..Adapter.SSLConfig = tEndPoint.SSLConfig
		} else {
			$$$ThrowOnError($$$ERROR($$$GeneralError,"ServiceName not found"))
		}		
  		  		
  		//Set Adapter URL path depending on value of SEOC ID in Request Message
  		//SEOC Id = "" - Retrieve all Active SEOCs (URL = .../v1/seoc/active)
  		//SEOC Id = <SEOC ID> - Retrieve just the one SEOC (URL = .../v1/name/{name})
  		If '(pRequest.SEOCId=""){
  			//set tURL = $p(..Adapter.URL,"active",1) _"name/" _ pRequest.SEOCId
  			set tURL = $p(..Adapter.URL,"active",1) _ pRequest.SEOCId
  		}
  		Else{
  		 	set tURL = ..Adapter.URL
  		 }			
  		$$$HSTRACE("SEOC tURL","tURL",tURL)
  		
  		//Execute the "GET"
  		Set tSC = ..Adapter.SendFDNS   aArray(.tHttpResponse, "GET", ..%HttpRequest, ,,tURL )
  		
    	//$$$HSTRACE("After REST Get","tHttpResponse",tHttpResponse)	
  		
  		//Log response/error received from DAS
  		If $IsObject(tHttpResponse.Data) {
	  		set tStream = ##class(%Stream.GlobalCharacter).%New()
	  		$$$ThrowOnError(tStream.CopyFrom(tHttpResponse.Data))
	  		set pResponse = ##class(EnsLib.HTTP.GenericMessage).%New(tStream)			
  		}
  		Else { 	$$$LOGERROR("No response received from SEOC DB") 		}
      
  	}
  catch ex {
  		
  		Set tSC = ex.AsStatus()
  }
  
  quit tSC
}

XData MessageMap
{
<MapItems>
		<MapItem MessageType="SEOC.Maintenance.Message.RetrieveSEOCRequest"> 
			<Method>OnMessage</Method>
		</MapItem>
	</MapItems>
}

}
