Class SEOC.Maintenance.Process Extends Ens.BusinessProcess [ ClassType = persistent ]
{

Property FieldDelimiter As %String(MAXLEN = 2) [ InitialExpression = "," ];

Property HeaderRowCount As %Integer [ InitialExpression = 2 ];

Property SEOCNumberCol As %Integer [ InitialExpression = 2 ];

Property ServiceLineCol As %Integer [ InitialExpression = 3 ];

Property CategoryCareCol As %Integer [ InitialExpression = 4 ];

Property SEOCNameCol As %Integer [ InitialExpression = 5 ];

Property CMSTaxonomyCodeCol As %Integer [ InitialExpression = 6 ];

Property ClinicalServiceCol As %Integer [ InitialExpression = 6 ];

Property VersionCol As %Integer [ InitialExpression = 7 ];

Property EffectiveStartDateCol As %Integer [ InitialExpression = 8 ];

Property EffectiveEndDateCol As %Integer [ InitialExpression = 9 ];

Property PALSEOCCol As %Integer [ InitialExpression = 10 ];

Property PreCertSEOCCol As %Integer [ InitialExpression = 11 ];

Property ServiceNumberCol As %Integer [ InitialExpression = 12 ];

Property ServiceRequestedCol As %Integer [ InitialExpression = 13 ];

Property DurationCol As %Integer [ InitialExpression = 14 ];

Property AuthVisitsCol As %Integer [ InitialExpression = 15 ];

Property CPTCodeCol As %Integer [ InitialExpression = 16 ];

Property DisclaimerCol As %Integer [ InitialExpression = 17 ];

Property SEOCWSClient As Ens.DataType.ConfigName [ InitialExpression = "HSRM.SEOC.WS.Client" ];

Parameter SETTINGS = "FieldDelimiter,HeaderRowCount,SEOCNumberCol,ServiceLineCol,CategoryCareCol,SEOCNameCol,ClinicalServiceCol,VersionCol,EffectiveStartDateCol,EffectiveEndDateCol,PALSEOCCol,PreCertSEOCCol,ServiceNumberCol,ServiceRequestedCol,DurationCol,AuthVisitsCol,CPTCodeCol,DisclaimerCol,SEOCWSClient";

Method OnRequest(pRequest As Ens.StreamContainer, Output pResponse As Ens.Response) As %Status
{
	#dim tSC As %Status = $$$OK
	#dim tProcessSEOCStatus As %Status = $$$OK
	#dim e As %Exception.AbstractException
	#dim tSEOCStream As %Stream.FileBinary
	#dim tLineSep As %String = $c(13)_$c(10)
	#dim tColSep As %String = ","
	#dim tChunkSize As %Iterator = 32000
		
	try {
		
		/*
		Local Debugging
		set tPath = "C:\Users\DNS   colbym\Documents\SEOC\"
		set tFilename = "HSRM SEOC Taxonomy_Build 1 Approved Structure_Updated 2.26 EH_CSV.csv"
		set tFile = ##class(%Stream.FileBinary).%New()
		$$$ThrowOnError(tFile.LinkToFile(tPath_tFilename))
		w tFile.Size
		
		
		set tStream = ##class(%Stream.GlobalBinary).%New()
		set tStream.LineTerminator = $char(13,10)
		$$$ThrowOnError(tStream.CopyFrom(tFile))
		*/
		
		if (pRequest.Stream.Size<1) {
			$$$ThrowOnError($$$ERROR($$$GeneralError,"SEOC File: "_pRequest.OriginalFilename_" is empty. Nothing to Process"))
		}
		set tStream = ##Class(%Stream.GlobalBinary).%New()
		$$$ThrowOnError(tStream.CopyFrom(pRequest.Stream))	
		//Rewind Stream - just to be certain we are at the beginning
		$$$ThrowOnError(tStream.Rewind())
		set tStream.LineTerminator = $c(13,10)
		
		// The SEOC source file is exported from Excel in a standard CSV format.
		// The default CSV export will have $c(13,10) as the line terminator (used for each row)
		// The default CSV export will wrap cell (field) data with double-quotes 
		//		if the cell contains
		// 			embedded LF characters - $c(10) 
		//		or 
		//			if the cell contains embedded commas.
		// The SEOC source file contains all the SEOCs
		// Each SEOC is identified by the SEOC Id
		// A SEOC contains many rows (1 for each Service Procedure)
		// A SEOC contains 16 Columns (a column's data maybe wrapped in quotes and within the quotes contain ",")
		
		// Navigate past the Header Rows
		set tStream.LineTerminator = $c(13,10)
		for i=1:1:..HeaderRowCount {
			d tStream.ReadLine()
		}
		
		//Begin SEOC Row Processing
		set tSEOCRow = tStream.ReadLine()
		set tSEOCId = $p(tSEOCRow,",",..SEOCNumberCol)
		set tCurrentSEOCId = tSEOCId
		set tSEOCRowNumber = 1

		do {
				
			// For Each SEOC
			// The SEOC row will be parsed into a the Field List global 
			// The Field List Global has the following structure:
			//		^FieldList(<SEOCId>,<Row Number>,<FieldNumber>)=FieldValue
			
			if '(tSEOCId = tCurrentSEOCId) {
				
				//Process Current SEOC
				k tSEOCObj
				set tSEOCProcessedSC = ..ProcessSEOC(.tFieldList)
				if $$$ISERR(tSEOCProcessedSC) {
					set tWarningMsg = "SEOC Processing failed for SEOC: " _ tCurrentSEOCId _ "   Error: "_ $$$StatusDisplayString(tSEOCProcessedSC,$C(13,10,43,13,10))
					$$$LOGWARNING(tWarningMsg)
				} else {
					$$$LOGINFO("SEOC: " _ tCurrentSEOCId _ " processed successfully.")
				}
				q:tStream.AtEnd
				
				//Begin new SEOC
				if '(tSEOCId = "") {
					set tCurrentSEOCId = tSEOCId
					set tSEOCRowNumber = 1
					k tFieldList
					$$$ThrowOnError(..ParseSEOCRow(tSEOCId,tSEOCRowNumber,tSEOCRow,.tFieldList))
					set tSEOCRowNumber = tSEOCRowNumber+1
				} else {
					quit
				}
				
			} else {
				
				//q:((tSEOCId="")||(tSEOCRow=""))
				$$$ThrowOnError(..ParseSEOCRow(tSEOCId,tSEOCRowNumber,tSEOCRow,.tFieldList))
				set tSEOCRowNumber = tSEOCRowNumber+1
			} 

			set tSEOCRow = tStream.ReadLine()
			set tSEOCId = $p(tSEOCRow,",",..SEOCNumberCol) q:(tSEOCId="")
			
						
		} while 1
		 		
	} catch e {
	
		set tSC = e.AsStatus()
			
	}
	
	quit tSC
}

Method ParseSEOCRow(pSEOCId As %String, pSEOCRowNumber As %Integer, pSEOCRow As %String, ByRef pFieldList) As %Status
{
	#dim tSC As %Status = $$$OK
	#dim e As %Exception.AbstractException
	
	try {
		
		set tCommaCnt = $LENGTH(pSEOCRow,",")
		set tFieldCnt = 0
		for i=1:1:tCommaCnt {
			
			set tPiece = $p(pSEOCRow,",",i)
			if $FIND(tPiece,"""") {
				//The piece contains a double-quote - does it contain the end-quote?
				if $LENGTH(tPiece,"""")>2 {
					//piece contains both beginning and ending quote
					set tFieldValue = tPiece
				} else {
					//piece only contains the beginning quote
					//increment with $p until the end quote is located.
					//concatenate all the pieces
					set tFieldValue = tPiece_","
					do {
						set i=i+1
						set tPiece = $p(pSEOCRow,",",i)
						if $FIND(tPiece,"""") {
							set tFieldValue = tFieldValue_tPiece
							quit
						} else {
							set tFieldValue = tFieldValue_tPiece_","
						}
						
					} while 1
				}

			} else {
				
				set tFieldValue = tPiece
			}
			
			set tFieldCnt = tFieldCnt+1
			set pFieldList(pSEOCId,pSEOCRowNumber,tFieldCnt)=tFieldValue
		
		}
		
	} catch e {
		
		set tSC = e.AsStatus()
		
	}
	
	quit tSC
}

Method ProcessSEOC(ByRef pFieldList) As %Status
{
	#dim tSC As %Status = $$$OK
	#dim e As %Exception.AbstractException
	
	try {
	
		set tSEOCIdKey = $ORDER(pFieldList(""))
		set objSEOC = ##class(SEOC.Maintenance.s0.SEOCData).%New()
		if '(objSEOC=$$$NULLOREF) {
			set objSEOC.SEOCIDNumber = tSEOCIdKey
			set objSEOC.ServiceLine = pFieldList(tSEOCIdKey,1,..ServiceLineCol)
			set objSEOC.CategoryOfCare = pFieldList(tSEOCIdKey,1,..CategoryCareCol)
			set objSEOC.SEOCName = pFieldList(tSEOCIdKey,1,..SEOCNameCol)
			//MKC -2018/06/19 - Defect #759122
			//Clinical Service may be alphanumeric The Clinical Service will be received
			//in the format "<code> - <description".
			//Use $p to extract the code.
			set objSEOC.ClinicalService = $p(pFieldList(tSEOCIdKey,1,..ClinicalServiceCol),"-",1)
			set objSEOC.Version = pFieldList(tSEOCIdKey,1,..VersionCol)
			set objSEOC.EffectiveStartDate = pFieldList(tSEOCIdKey,1,..EffectiveStartDateCol)
			set objSEOC.EffectiveEndDate = pFieldList(tSEOCIdKey,1,..EffectiveEndDateCol)
			set objSEOC.PALSEOC = pFieldList(tSEOCIdKey,1,..PALSEOCCol)
			set objSEOC.PreCertSEOC = pFieldList(tSEOCIdKey,1,..PreCertSEOCCol)
			set objSEOC.Disclaimer =  $ZSTRIP(pFieldList(tSEOCIdKey,1,..DisclaimerCol),"<>","""")
			//MKC -2018/06/19 - Defect #759122
			//CMS Taxonomy code may be alphanumeric The CMS Taxonomy Code will be received
			//in the format "<code> - <description".
			//Remove $FNUMBER to extract the code.
			//set tCMSCode = $FNUMBER($p(pFieldList(tSEOCIdKey,1,..CMSTaxonomyCodeCol),"-",1))
			set tCMSCode = $p(pFieldList(tSEOCIdKey,1,..CMSTaxonomyCodeCol),"-",1)
			//MKC - 2018/02/27 per Email (Anthony P. - CMS Taxonomy Codes are required - do not process SEOCs with a missing CMS Code)
			if (tCMSCode = "") {
				set tErrorMsg = "CMS Taxonomy Code is missing."
				$$$ThrowOnError($$$ERROR($$$GeneralError,tErrorMsg))
			} else {
				set objSEOC.CMSTaxonomyCode = tCMSCode
			}
			set objSEOC.Duration = pFieldList(tSEOCIdKey,1,..DurationCol)
		}
		
		set tRowKey=""
		do {
			
			set tRowKey=$ORDER(pFieldList(tSEOCIdKey,tRowKey)) 	q:(tRowKey="")

			set objSEOCService = ##class(SEOC.Maintenance.s0.SEOCService).%New()
			if '(objSEOCService=$$$NULLOREF) {
				set objSEOCService.NoOfService = pFieldList(tSEOCIdKey,tRowKey,..ServiceNumberCol)
				set objSEOCService.ServiceCareRequested = pFieldList(tSEOCIdKey,tRowKey,..ServiceRequestedCol)
				//MKC - 02/28/2018 per Email (Anthony P. - Authorized No Visits that non-numerical or 0 must be 
				//set to NULL ("")
				if (+pFieldList(tSEOCIdKey,tRowKey,..AuthVisitsCol) = 0) {
					set objSEOCService.AuthorizedNoOfVisits = ""
				} else {
					set objSEOCService.AuthorizedNoOfVisits = pFieldList(tSEOCIdKey,tRowKey,..AuthVisitsCol)
				}
				//MKC - 02/28/2018 per Email (Anthony P. - CPT Codes are required - do not process SEOCs with missing CPT Codes)
				if pFieldList(tSEOCIdKey,tRowKey,..CPTCodeCol)="" {
					set tErrorMsg = "CPT Codes missing for Service/Care Requested: "_objSEOCService.ServiceCareRequested
					$$$ThrowOnError($$$ERROR($$$GeneralError,tErrorMsg))
				} else {
				
					//MKC - 03/15/2018 - 
					//CPT codes = "TBD" do not process.
					//CPT codes = "NA" process with ""
					//CPT codes = comma-delimited string, process
					if pFieldList(tSEOCIdKey,tRowKey,..CPTCodeCol)="TBD" {
						set tErrorMsg = "CPT Codes to-be-determined for Service/Care Requested: "_objSEOCService.ServiceCareRequested
						$$$ThrowOnError($$$ERROR($$$GeneralError,tErrorMsg))
					} elseif pFieldList(tSEOCIdKey,tRowKey,..CPTCodeCol)="NA" {
						set objSEOCService.CPTCodes = pFieldList(tSEOCIdKey,tRowKey,..CPTCodeCol)
					} else {
						set objSEOCService.CPTCodes = pFieldList(tSEOCIdKey,tRowKey,..CPTCodeCol)
					}
				}
				$$$ThrowOnError(objSEOC.Services.Insert(objSEOCService))
			}
			
		} while 1
		
		/*User for local debugging
		w !,"SEOC: ",objSEOC.SEOCIDNumber,!	
		zw objSEOC
		*/
		
		set tSEOCUploadRequest = ##class(SEOC.Maintenance.Message.MainRequest).%New()
		set tSEOCUploadRequest.SEOCData = objSEOC
		$$$ThrowOnError(..SendRequestSync(..SEOCWSClient,tSEOCUploadRequest,.tSEOCUploadResponse,15))

	} catch e {
		
		set tSC = e.AsStatus()
		
	}
	
	quit tSC
}

Storage Default
{
<Data name="ProcessDefaultData">
<Subscript>"Process"</Subscript>
<Value name="1">
<Value>FieldDelimiter</Value>
</Value>
<Value name="2">
<Value>PackCodeColumn</Value>
</Value>
<Value name="3">
<Value>PackDescriptionColumn</Value>
</Value>
<Value name="4">
<Value>PackDateFromColumn</Value>
</Value>
<Value name="5">
<Value>PackDateToColumn</Value>
</Value>
<Value name="6">
<Value>ServiceLineColumn</Value>
</Value>
<Value name="7">
<Value>ServiceLineDescColumn</Value>
</Value>
<Value name="8">
<Value>DurationLocaton</Value>
</Value>
<Value name="9">
<Value>CPTAuthNumberVisitsColumn</Value>
</Value>
<Value name="10">
<Value>CPTCodesColumn</Value>
</Value>
<Value name="11">
<Value>CMSCodeForSpecialtyColumn</Value>
</Value>
<Value name="12">
<Value>PackMultipleEpisode</Value>
</Value>
<Value name="13">
<Value>PackSelectionMethod</Value>
</Value>
<Value name="14">
<Value>CPTCodeErrors</Value>
</Value>
<Value name="15">
<Value>VerifiedCPTCodes</Value>
</Value>
<Value name="16">
<Value>SEOCId</Value>
</Value>
<Value name="17">
<Value>HeaderRowCount</Value>
</Value>
<Value name="18">
<Value>SEOCNumberCol</Value>
</Value>
<Value name="19">
<Value>ServiceLineCol</Value>
</Value>
<Value name="20">
<Value>CategoryCareCol</Value>
</Value>
<Value name="21">
<Value>SEOCNameCol</Value>
</Value>
<Value name="22">
<Value>CMSTaxonomyCodeCol</Value>
</Value>
<Value name="23">
<Value>ClinicalServiceCol</Value>
</Value>
<Value name="24">
<Value>VersionCol</Value>
</Value>
<Value name="25">
<Value>EffectiveStartDateCol</Value>
</Value>
<Value name="26">
<Value>EffectiveEndDateCol</Value>
</Value>
<Value name="27">
<Value>PALSEOCCol</Value>
</Value>
<Value name="28">
<Value>PreCertSEOCCol</Value>
</Value>
<Value name="29">
<Value>DisclaimerCol</Value>
</Value>
<Value name="30">
<Value>DurationCol</Value>
</Value>
<Value name="31">
<Value>ServiceNumberCol</Value>
</Value>
<Value name="32">
<Value>ServiceRequestedCol</Value>
</Value>
<Value name="33">
<Value>CPTCodeCol</Value>
</Value>
<Value name="34">
<Value>AuthVisitsCol</Value>
</Value>
<Value name="35">
<Value>SEOCWSClient</Value>
</Value>
</Data>
<DefaultData>ProcessDefaultData</DefaultData>
<Type>%Library.CacheStorage</Type>
}

}
