Class eCAMS.Operation.ReferralDocumentToeCAMS Extends (EnsLib.REST.GenericOperation, HS.Util.Trace.Helper)
{

Parameter INVOCATION = "Queue";

Parameter ADAPTER = "EnsLib.HTTP.OutboundAdapter";

Property ServiceName As %String [ InitialExpression = "eCAMS.SendReferral" ];

Parameter SETTINGS = "ServiceName:Basic";

Property Adapter As EnsLib.HTTP.OutboundAdapter;

Method OnInit() As %Status
{
	Set ..%HttpRequest=##class(eCAMS.Message.eCAMSHTTPRequest).%New()
	$$$HSTRACE(" OnInit " )
	Quit $$$OK
}

// 8-1-18 JLM

// Sends referral to FSC/eCAMS through a RESTful API

// Send any errors to Ens.Alert for processing

Method OnMessage(pRequest As eCAMS.Message.eCAMSReferralRequest, Output pResponse As EnsLib.HTTP.GenericMessage) As %Status
{

	#dim tSC As %Status = $$$OK
	#dim tHttpResponse As %Net.HttpResponse
	#dim tEndPoint as HS.Registry.Service.HTTP
	 
	try{	
			//Get WS Location from HS Service Registry
			Set tEndPoint = ##class(HS.Registry.Service.HTTP).EndPointForNameType(..ServiceName,"HTTP")
			If $isObject(tEndPoint) {
				Set ..Adapter.URL = tEndPoint.URL 
				Set ..Adapter.HTTPServer = tEndPoint.Host
				Set ..Adapter.HTTPPort = tEndPoint.Port
			} else {
				$$$ThrowOnError($$$ERROR($$$GeneralError,"ServiceName not found"))
			}		
			
			//send referral to FSC/eCAMS
			Do ..%HttpRequest.SetHeader("Authorization","Basic dmFmc2NhdXRoZWNhbXNfZGV2OnRJNU41alVZTlVhU1NuN0tqeHVs")
			Set ..%HttpRequest.EntityBody =  pRequest.Stream		
	  		$$$HSTRACE("Header List: " _ ..%HttpRequest.ReturnHeaders())
	  		Set tSC = ..Adapter.SendFDNS   aArray(.tHttpResponse, "POST", ..%HttpRequest, .pFormVarNames , .pData)

	      	//Process response/error received from FSC/eCAMS
	      	If $IsObject(tHttpResponse) {
		  		If $IsObject(tHttpResponse.Data) {
		  			Set sResponseBody = tHttpResponse.Data.Read() 		
		  			Set sStatus = tHttpResponse.StatusCode
		  			$$$HSTRACE("HTTP Status: " _ sStatus _ $char(10) _ "Response body: " _ sResponseBody)  			
		  			if (sStatus = "200"){
		  				/* Check that there wasn't a data validation error. 
		  					200 only means that the message was received/parsed, not that the data contained in the body was valid.
		  				*/
		  				Set requestObject = {}.%FromJSON(sResponseBody)
			  			set sResponseStatus = requestObject.ResponseStatus
			  			if (sResponseStatus = "Error"){
			  				set sResponseCode = requestObject.ResponseStatusCode
			  				Set sErrorMessage = requestObject.ErrorMessages.message.message
			  				set exception = ##class(%Exception.General).%New("eCAMS Data Validation Error ",sResponseCode,, sErrorMessage )
			  				throw exception
			  			}				
		  			}
		  			elseif (sStatus = "404" ){
		  				$$$HSTRACE("404 Error")
		  				$$$LOGERROR("Endpoint Not Found:  " _ $char(10) _ sStatus _ $char(10) _ sResponseBody)
		  				set exception = ##class(%Exception.General).%New("eCAMS Connection Error ",sStatus,, sResponseBody )
		  				throw exception
		  			}		  			
		  			elseif (sStatus = "500" ){
		  				$$$LOGERROR("Connection Error:  " _ $char(10) _ sStatus _ $char(10) _ sResponseBody)
		  				set exception = ##class(%Exception.General).%New("eCAMS Connection Error ",sStatus,, sResponseBody )
		  				throw exception
		  			}
		  			else{
		  				$$$LOGERROR("Generic Error Received:  " _ $char(10) _ sStatus _ $char(10) _ sResponseBody)
		  				set exception = ##class(%Exception.General).%New("eCAMS Exception",sStatus,, sResponseBody )
		  				throw exception
		  			}
		  		}
		  		else{
		  			set sError = "No response body received from eCAMS"
		  			$$$LOGERROR(sError)
		  			set exception = ##class(%Exception.General).%New("eCAMS Exception","600",, sError )
		  			throw exception
		  		}
		  	}	      
	  }
	  catch ex {
	  		Set tSC = ex.AsStatus()
	  		$$$LOGERROR("Error sending message to eCAMS: " _ tSC)
	  }
	  	
	  return tSC
}

XData MessageMap
{
<MapItems>
		<MapItem MessageType="eCAMS.Message.eCAMSReferralRequest"> 
			<Method>OnMessage</Method>
		</MapItem>
	</MapItems>
}

}
