Class eCAMS.Process.ExtractTransformReferral Extends (Ens.BusinessProcess, HS.Util.Trace.Helper) [ ClassType = persistent ]
{

// Build 4.0 

// Checked in 9/26/18

// Process called by eCAMS.Service.GetHSRMReferral to extract and transform referrals real time for eCAMS

Property HSRMTarget As Ens.DataType.ConfigName [ InitialExpression = "HSRM.ReferralDataExtraction.WS.Client" ];

Property eCAMSTarget As Ens.DataType.ConfigName [ InitialExpression = "eCAMS.Operation.ReferralDocumentToeCAMS" ];

Property SEOCRESTClient As Ens.DataType.ConfigName [ InitialExpression = "SEOCDB.REST.Client" ];

Parameter SETTINGS = "HSRMTarget,eCAMSTarget";

// Method OnRequest(pInputReferralID As Ens.StringContainer, Output pResponse As Ens.Response) As %Status

Method OnRequest(pInputReferralID As Ens.StringContainer, Output pResponse As Ens.Response) As %Status
{
 
	#dim tSC As %Status = $$$OK
	#dim e As %Exception.AbstractException
	#dim tRetrieveReferralResponse AS CCRA.Message.RetrieveReferralResponse
	#dim tSEOCRequest AS SEOC.Maintenance.DB.Message.RetrieveSEOCRequest
	#dim tRESTClientResponse As EnsLib.HTTP.GenericMessage
	#dim teCAMSRequest AS eCAMS.Message.eCAMSReferralRequest
	#dim tSEOCRESTClientResponse as EnsLib.HTTP.GenericMessage
	#dim ReferralIdMode as Integer = 1 //Assuming Referral ID is sent for now, which uses mode 1
	
	try {		
		//Get referral data from HSRM web service
		set tRetrieveReferralRequest = ##class(CCRA.Message.RetrieveReferralRequest).%New()
		set tRetrieveReferralRequest.Id = pInputReferralID.StringValue
		set tRetrieveReferralRequest.Mode = ReferralIdMode 						
		do ..SendRequestSync(..HSRMTarget,tRetrieveReferralRequest,.tRetrieveReferralResponse, 30)

		//Get SEOC data or act accordingly if not available 
		if $IsObject(tRetrieveReferralResponse.RetrieveReferralResult) {	
		
			//Build JSON message body to include referral and SEOC sections if available
			Set tMessageBody = ##class(%ZEN.proxyObject).%New()
			set tRetrieveReferralResponse.RetrieveReferralResult.TreatmentStartDate = $ZDATE(tRetrieveReferralResponse.RetrieveReferralResult.TreatmentStartDate, "3") //reformat date to be YYYY-MM-DD
			Set tMessageBody.Referral = tRetrieveReferralResponse.RetrieveReferralResult
		
			//8-21-18  JLM Adding logic to pull local copy of SEOC rather than using API call 
			try{ 
				set tsc = ..RetreiveSEOCFromSQL(tRetrieveReferralResponse.RetrieveReferralResult.SeocID, .tSEOCData)
				set tMessageBody.SEOC = tSEOCData
			}
			catch(e){
				set tSC = e.AsStatus()	
			}
			
			//Add JSON message body to eCAMS Request Message
			set tSC = ##class(%ZEN.Auxiliary.jsonProvider).%WriteJSONStreamFromObject(.tJSONStream,tMessageBody,,,,"tu")
			set teCAMSRequest = ##class(eCAMS.Message.eCAMSReferralRequest).%New()			
			set teCAMSRequest.DocumentId = pInputReferralID.StringValue
			set teCAMSRequest.Stream = ##class(%Stream.GlobalCharacter).%New()					
			$$$ThrowOnError(teCAMSRequest.Stream.CopyFrom(tJSONStream))
			
			/*read from eCAMS sample file rather than generating out of HSRM for testing purposes
			set teCAMSFileStream = ##class(%Stream.FileCharacter).%New()
			set tSC = teCAMSFileStream.LinkToFile("Att2AFC.txt")
			$$$HSTRACE("FileStream: " _ teCAMSFileStream.Read())
			$$$ThrowOnError(teCAMSRequest.Stream.CopyFrom(teCAMSFileStream))
			*/
				
			//Send referral message to operation
			$$$ThrowOnError(..SendRequestAsync(..eCAMSTarget,teCAMSRequest,.tResponse))
		}
		else { 	
			$$$LOGERROR("Error received when sending message to " _ ..eCAMSTarget)
		}  
		
	}
	catch e {
		set tSC = e.AsStatus()	
	}
	
	quit tSC
}

Method GetSEOCFromAPI(pSEOCExternalID As %String, Output iSeocID, Output tSEOCRESTClientResponse As EnsLib.HTTP.GenericMessage) As %Status
{
	/* 
		8-14-18   JLM   Story #800107
		receives external SEOC ID
		looks up internal ID based on the look up table, SEOC_ID
		xcalls SEOC API with internal ID and returns SEOC data in a JSON stream
	*/
	#dim tSC As %Status = $$$OK
	
	try{
		set tSEOCRequest = ##class(SEOC.Maintenance.DB.Message.RetrieveSEOCRequest).%New()
		set iSeocID = ##class(Ens.Util.FunctionSet).Lookup("SEOC_ID",pSEOCExternalID,"0",3)
		set tSEOCRequest.SEOCId = iSeocID
		if '(iSeocID = "0")
			do ..SendRequestSync(..SEOCRESTClient,tSEOCRequest,.tSEOCRESTClientResponse,30)
	}
	catch e{
		set tSC = e.AsStatus()	
		$$$LOGERROR("Error while retrieving SEOC: "_tRetrieveReferralResponse.RetrieveReferralResult.SeocID _ $char(10) _ tSC)
	}
	
	quit tSC
}

Method RetreiveSEOCFromSQL(pSEOCExternalID As %String, Output tSEOC As SEOC.Maintenance.DB.OBJ.SEOC) As %Status
{
	/*
		8-22-18   JLM   Story #808168
		receives external SEOC ID
		looks up SEOC data in HSIE database, SEOC.Maintenance.DB.OBJ.SEOC
		this table is refreshed nightly
	*/
	
	#dim tSC As %Status = $$$OK
	
	try{
		&sql(SELECT ID1 INTO :tID FROM SEOC_Maintenance_DB_OBJ.SEOC WHERE seocId = :pSEOCExternalID)
		if '(tID="") {
				set tSEOC = ##class(SEOC.Maintenance.DB.OBJ.SEOC).%OpenId(tID)
		}
	}
	catch e{
		set tSC = e.AsStatus()	
		$$$LOGERROR("Error while retrieving SEOC: "_tRetrieveReferralResponse.RetrieveReferralResult.SeocID _ $char(10) _ tSC)
	}
	
	quit tSC
}

Method OnResponse(request As %Library.Persistent, ByRef response As %Library.Persistent, callrequest As %Library.Persistent, callresponse As %Library.Persistent, pCompletionKey As %String) As %Status
{
	
	Quit $$$OK
}

Storage Default
{
<Data name="ExtractTransformReferralDefaultData">
<Subscript>"ExtractTransformReferral"</Subscript>
<Value name="1">
<Value>HSRMTarget</Value>
</Value>
<Value name="2">
<Value>eCAMSTarget</Value>
</Value>
<Value name="3">
<Value>SEOCRESTClient</Value>
</Value>
<Value name="4">
<Value>TraceOperations</Value>
</Value>
</Data>
<DefaultData>ExtractTransformReferralDefaultData</DefaultData>
<Type>%Library.CacheStorage</Type>
}

}
