Class eCAMS.Service.REST Extends %CSP.REST
{

// Dispatch Class to handle communication from HSRM when new referrals are ready 

// Receives call from eCAMS.Service.GetHSRMReferral with referral id

Parameter ADAPTER = "EnsLib.HTTP.InboundAdapter";

/// This prefix will be removed from the beginning of the actual URL before matching it against the UrlMap.
/// If this prefix begins with '^' then the CSP Application will also be removed from the beginning of the URL if this service is being invoked via the CSP gateway.
/// If this prefix begins with '|' then the Ensemble Config Name for this service will be removed from the beginning of the URL, as well as the CSP Application if this service is being invoked via the CSP gateway.
Parameter EnsServicePrefix = "|";

Parameter PREFIXCASESENSITIVE As %Boolean = 0;

Parameter HandleCorsRequest = 1;

XData UrlMap
{
<Routes>
		<Route Url="/Document/:DocumentId" Method="GET" Call="RetreiveDocument" />  
		<Route Url="/Document/:DocumentId" Method="POST" Call="RetreiveDocument" /> 	
	</Routes>
}

ClassMethod RetreiveDocument(pDocumentId As %String, pHTTPResponseOutput As %Stream.GlobalCharacter) As %Status
{

	#dim bDebug As %Boolean = 1
    #dim tSC As %Status = $$$OK
    #dim e As %Exception.AbstractException   

    if (bDebug) { 
    	kill ^JessaDebug
    	Set ^JessaDebug("RetreiveDocument")="Fired"
    }
    
    try {  //Call service with referral ID to pass off to process and extract data
    
    	if '(pDocumentId="") { //verify document id was provided, else quit and error
    		Set tSC = ##class(Ens.Director).CreateBusinessService("eCAMS.Service.GetHSRMReferral", .tService)
    		If ($$$ISERR(tSC)) 	{
    			Quit
    		}		
    		Set sContainer = ##class(Ens.StringContainer).%New(pDocumentId)
    		Set tSC = tService.ProcessInput(sContainer, .pOutput)
    		If ($$$ISERR(tSC))  {
    			Quit 
    		}
    		Set %response.Status = ..#HTTP200OK  	
    	}
    }
    catch (e) {
        Set tSC=e.AsStatus()
        Set %response.Status = ..#HTTP400BADREQUEST   
    } 
    
    quit tSC
}

}
