Class CCRA.Transform.HSRMReferralToCCRAReferral Extends Ens.DataTransform
{

ClassMethod Transform(pSource As User.PAWaitingList, pTarget As CCRA.Referral) As %Status
{
	
	#dim tSC As %Status = $$$OK
	#dim e As %Exception.AbstractException
	#dim tTransHistory As CCRA.TransactionHistory
	#dim tSourceTransaction As User.PAWaitingListTransaction
	#dim tESQ As questionnaire.QUSXXES
	#dim tESQChild As questionnaire.QUSXXESQQTAB1
	#dim tSEOCId as %String = ""
	
	try { 
		
		set pTarget = ##class(CCRA.Referral).%New()
		
		//Patient Demographics
		set tPatient = ##class(CCRA.Patient).%New()
		/* JLM 9-24-18  PAPERID is SSN, not ICN
			set tPatient.ICN = pSource.WLPAPMIDR.PAPMIRowId.PAPERID
		*/ 
		set tPatient.ICN = pSource.WLPAPMIDR.PAPMIDVAnumber
		set tPatient.LastName = pSource.WLPAPMIDR.PAPMIRowId.PAPERName
		set tPatient.FirstName = pSource.WLPAPMIDR.PAPMIRowId.PAPERName2
		set tPatient.MName = pSource.WLPAPMIDR.PAPMIRowId.PAPERName3  //10/5/18  Changed from PAPERName2, which is first name
		set tPatient.Gender = pSource.WLPAPMIDR.PAPMIRowId.PAPERSexDR.CTSEXGender 
		/* JLM 9-17-18 - gender was not being sent. Original code: 
			set tPatient.Gender =  pSource.WLPAPMIDR.PAPMIRowId.PAPERBirthGenderDR.CTSEXGender
		*/
		set tPatient.DOB = $SELECT(pSource.WLPAPMIDR.PAPMIRowId.PAPERDob'="":$zdt(pSource.WLPAPMIDR.PAPMIRowId.PAPERDob,3),1:"")
		set tPatient.HomeAddress.Address1 = pSource.WLPAPMIDR.PAPMIRowId.PAPERStNameLine1
		set tPatient.HomeAddress.Address2 = pSource.WLPAPMIDR.PAPMIRowId.PAPERForeignAddress
		set tPatient.HomeAddress.City = pSource.WLPAPMIDR.PAPMIRowId.PAPERCityCodeDR.CTCITDesc
		set tPatient.HomeAddress.State = pSource.WLPAPMIDR.PAPMIRowId.PAPERStateCodeDR.CTSTTDesc
		set tPatient.HomeAddress.ZipCode = pSource.WLPAPMIDR.PAPMIRowId.PAPERZipDR.CTZIPCode
		set tPatient.HomeAddress.Country = pSource.WLPAPMIDR.PAPMIRowId.PAPERCountryDR.CTCOUDesc
		set tPatient.SSN = pSource.WLPAPMIDR.PAPMIRowId.PAPERID
		// MM 10/04/2018 Add the following to Patient record
		set tPatient.TelephoneHome = pSource.WLPAPMIDR.PAPMIRowId.PAPERTelH
		set tPatient.TelephoneBusiness = pSource.WLPAPMIDR.PAPMIRowId.PAPERTelO
		set tPatient.TelephoneMobile = pSource.WLPAPMIDR.PAPMIRowId.PAPERMobPhone
		set tPatient.Email = pSource.WLPAPMIDR.PAPMIRowId.PAPEREmail
		set tPatient.PreferredContactMethod = pSource.WLPAPMIDR.PAPMIRowId.PAPERPreferredContactMethod
		set pTarget.Patient = tPatient
		
		//Referral Info
		set pTarget.Diagnosis = pSource.WLICDDR.MRCIDDesc
		// 06-05-2018 MKC
		// Rational Jazz Task #750981 - added Diagnosis code
		set pTarget.DiagnosisCode = pSource.WLICDDR.MRCIDCode
		// 03-15-2018 MKC
		// Rational Jazz Task #695598
		// Changed source for ReferralNumber from WLNO to WLReferralUniqueID
		// 03-16-2018 MKC
		// Rational Jazz Story #699845
		// Need to include the Consult ID along with the Referral Number
		// ReferralNumber = HSRM Referral Number, i.e VA000000000192
		// ConsultId = CPRS Consult Id, 
		set pTarget.ReferralNumber = pSource.WLNO
		set pTarget.ReferralConsultId = pSource.WLReferralUniqueID
		set pTarget.ReferralDate = $SELECT(pSource.WLDateRefLetter'="":$zdt(pSource.WLDateRefLetter,3),1:"")
		set pTarget.ReferralExpirationDate = $SELECT(pSource.WLRefExpiryDate'="":$zdt(pSource.WLRefExpiryDate,3),1:"")
		set pTarget.ReferralPeriod = pSource.WLReferralPeriodDR.REFPERDesc
		set pTarget.ReferralLastUpdateDate = $SELECT(pSource.WLUpdateDate'="":$zdt(pSource.WLUpdateDate,3),1:"")
		set pTarget.ReferralCategory = pSource.WLEpisSubTypeDR.SUBTDesc
		set pTarget.MedicalNecessity = pSource.WLPresentComplaint
		set pTarget.SourceOfReferral = pSource.WLSourceOfAdditDR.SADDDesc
		set pTarget.Status = pSource.WLWaitListStatusDR.WLSDesc
		set pTarget.ReferralReturnReason = pSource.WLRefStatusReasonDR.REFSTREADesc
		set pTarget.ServiceRequested = pSource.WLStateProcDR.SPPPDesc
		set pTarget.ReferringFacility = pSource.WLRefDepHospDR.HOSPDesc
		set pTarget.ReferringProvider = pSource.WLRefDocDR.REFDDesc
		set pTarget.ReferringSpecialty = pSource.WLRefDepDR.CTLOCDesc
		set pTarget.TreatingFacility = pSource.WLHospitalDR.HOSPDesc
		set pTarget.TreatingProvider = pSource.WLDoctorDR.CTPCPDesc
		set pTarget.TreatingProviderNPI = pSource.WLDoctorDR.CTPCPCode
		set pTarget.TreatingSpeciality = pSource.WLCTLOCDR.CTLOCDesc
		
		// 07/27/2018  JLM
		// Rational Jazz Story #742981 - Add Primary Care Provider NPI to data extraction
		
		//look up patient id, required to find PCP
		set tPatientID = "" 
		set tReferralNo = pSource.WLNO
		&sql(SELECT WL_PAPMI_DR->PAPMI_PAPER_DR INTO :tPatientID
			FROM SQLUser.pa_waitinglist
			WHERE  wl_no = :tReferralNo)
		// MM 10/4/2018 add PCP name
		set tPCPNPI = ""
		set tPCPName = ""
		&sql(SELECT top 1 CP_CareProv_DR->CTPCP_Code,CP_CareProv_DR->CTPCP_Desc INTO :tPCPNPI, :tPCPName
		 	FROM SQLUser.PA_PersonCareProvider
			WHERE CP_ParRef = :tPatientID)
		set pTarget.PrimaryCareProviderNPI = tPCPNPI
		set pTarget.PrimaryCareProviderName = tPCPName
		set pTarget.Network = pSource.WLWaitListTypeDR.WLTDesc
		set pTarget.NetworkCode = pSource.WLWaitListTypeDR.WLTCode  //JLM 10-11-18 Adding Network Code
		set pTarget.CLINCode = pSource.WLCareTypeDR.CARETYPCode
		set pTarget.CLINDesc = pSource.WLCareTypeDR.CARETYPDesc
		set pTarget.PaymentAuthorityCode = pSource.WLProgramFundingDR.PFSCode
		set pTarget.PaymentAuthorityDesc = pSource.WLProgramFundingDR.PFSDesc
		set pTarget.AppointmentDate = pSource.WLApptDate
		set pTarget.TreatmentStartDate = pSource.WLTreatmentStartDate 
		
		//Payor Status
		//MKC 06-06-2018
		//Jazz Task #752492 - Adding Payor Status to HSRM Data Extraction
		set pTarget.PayorStatus = pSource.WLPACLegalStatusDR.LEGALSTDesc
		
		//OHI Info
		//JLM 07-26-18
		//Jazz Task# 752166 - Adding OHI data elements to HSRM data extraction
		set pTarget.OHI = ##class(%ListOfObjects).%New()
		set tReferralId = pSource.%Id()
		set tPayor = ""
		set tDateFrom = ""
		set tDateTo = ""
		set tCardHolderName = ""
		set tCardNo = ""
		set tAddress = ""
		set tCity = "" 
		set tState = ""
		set tZip = ""
			
	    &sql(DECLARE C2 CURSOR FOR
	        SELECT INS_InsType_DR->INST_Desc, INS_DateValidFrom, INS_DateValidTo, INS_CardholderName, 	INS_CardNo, INS_CardholderAddress, INS_CardholderCity_DR, 	INS_CardholderZip_DR
	        INTO :tPayor, :tDateFrom, :tDateTo, :tCardHolderName, :tCardNo, :tAddress, :tCity, :tZip
			FROM SQLUser.pa_adminsurance
			WHERE ins_parref->paadm_waitlist_dr = :tReferralId
	    )
	
	    &sql(OPEN C2)
	    &sql(FETCH C2)
	
	    While (SQLCODE = 0) {        
	        set tOHI = ##class(CCRA.OHI).%New()
			set tOHI.Payor = tPayor
			set tOHI.ValidFromDate = $ZDATE(tDateFrom, "3")
			set tOHI.ValidToDate = $ZDATE(tDateTo, "3") 
			set tOHI.CardholderName = tCardHolderName
			set tOHI.PayorCardNumber =  tCardNo
			set tOHI.CardholderAddress.Address1 =  tAddress
			set tOHI.CardholderAddress.Address2 =  ""
			set tOHI.CardholderAddress.City =  tCity
			set tOHI.CardholderAddress.State =  ""
			set tOHI.CardholderAddress.ZipCode = tZip
			
			$$$ThrowOnError(pTarget.OHI.Insert(tOHI))

	         &sql(FETCH C2)
	    }
	    &sql(CLOSE C2)
   

		// eCAMS fields
		//JLM 07-11-18
		//Jazz Task #772894 - eCAMS add additional required fields to CCRA data transformation		
		set pTarget.CategoryOfCare = pSource.WLStateProcDR.SPPPNPPPDR.NPPPDesc 
		set pTarget.StationID = pSource.WLRefDepHospDR.HOSPCode
		set pTarget.EstimatedCostOfCare = pSource.WLEstimatedCostOfCare 		
		set pTarget.Sta6 = $P(pSource.WLReferralUniqueID, "_") //JLM 08/13/18
		
		//Lookup SEOC ID
		set sReferralID = pSource.WLNO
		set sPACKDesc = pSource.WLPackageDR.PACKDesc
		set tSEOCId = ""
		&sql(SELECT ap.PACK_Code INTO :tSEOCId
			FROM SQLUser.PA_WaitingList As wl
				JOIN SQLUser.PA_PersonPackage AS pp On pp.PACK_RowId = wl.WL_Package_DR
				JOIN SQLUser.ARC_Package As ap ON pp.PACK_Desc = ap.PACK_Desc
				WHERE wl.WL_NO = :sReferralID)				
		
		//8-22-18 JLM  
		//SQL is returning a false value when ap.PACK_Code is empty
		if '(SQLCODE=100){  
			set pTarget.SeocID = tSEOCId 
		}
		else{
			set pTarget.SeocID = "SEOC unavailable in HSRM"
		}
			 
		set pTarget.Comments = ""    //loop through comments object
		for i=1:1:pSource.WLRemarks.Count() {
			if pTarget.Comments '= "" {
				set pTarget.Comments = pTarget.Comments _ $c(13,10) _ pSource.WLRemarks.GetAt(i)
			} else {
				set pTarget.Comments = pSource.WLRemarks.GetAt(i)
			}
		}
		
		/* JLM 09-25-18 WLRefDocDR column is empty.  Changing based on Data Entities and Field Definitions Document, Dated 6/5/18 
			pSource.WLRefDocDR.REFDCode
		*/
		set pTarget.ReferringProviderNPI  = pSource.WLRefDocIntDR.CTPCPCode 
		set pTarget.TreatingFacilityTaxID = pSource.WLHospitalDR.HOSPCode
		set pTarget.TreatingProviderTaxID = "" // Provider Tax ID not available  pSource.WLDoctorDR.CTPCPCode
		set pTarget.NoteOnAppointmentScreen = "" 
		set pTarget.CommentOnReferralScreen = "" 	
		
		
		/* 10/22/18 JLM New TriWest Fields	*/
		set pTarget.PriorityCode = pSource.WLWaitListPriorDR.WLPCode
		set pTarget.PriorityDescription = pSource.WLWaitListPriorDR.WLPDesc
		set pTarget.ClinicallyIndicatedDate = $SELECT(pSource.WLTreatmentStartDate'="":$zdt(pSource.WLTreatmentStartDate,3),1:"") 
		set pTarget.TreatingSpecialityCode = pSource.WLCTLOCDR.CTLOCCode
		if '(pSource.WLApptLocDR.CTLOCAddress= $$$NULLOREF)  {
			if (pSource.WLApptLocDR.CTLOCAddress.Count() >=1)
				set pTarget.TreatingFacilityAddress.Address1  =  pSource.WLApptLocDR.CTLOCAddress.GetAt(1)
		}
		set pTarget.TreatingFacilityAddress.City = pSource.WLApptLocDR.CTLOCCityDR.CTCITDesc
		set pTarget.TreatingFacilityAddress.State =pSource.WLApptLocDR.CTLOCRegionDR.CTRGDesc
		set pTarget.TreatingFacilityAddress.ZipCode = pSource.WLApptLocDR.CTLOCZipDR.CTZIPCode
		set pTarget.TreatingFacilityPhone = pSource.WLApptLocDR.CTLOCTelephone _ " " _ pSource.WLApptLocDR.CTLOCTelephoneExt
		set pTarget.TreatingFacilityFax = pSource.WLApptLocDR.CTLOCFax
		set pTarget.AffilitationDesc = pSource.WLCareProvGroupDR.CPGDesc 
		set pTarget.AffilitationCode = pSource.WLCareProvGroupDR.CPGCode 
		set pTarget.Appointments = ##class(%ListOfObjects).%New()
		set tApptDate = ""
		set tApptTime = ""
			
	    &sql(DECLARE C3 CURSOR FOR
	        SELECT APPT_AS_ParRef->AS_Date, APPT_AS_ParRef->AS_SessStartTime
	        INTO :tApptDate, :tApptTime
			FROM SQLUser.RB_Appointment
			WHERE APPT_WaitList_DR= :tReferralId
	    )   
	    &sql(OPEN C3)
	    &sql(FETCH C3)
	    While (SQLCODE = 0) {     
	        set tAppointment = ##class(CCRA.Appointment).%New()
			set tAppointment.AppointmentDate =  $SELECT(tApptDate'="":$zdt(tApptDate,3),1:"") 
			set tAppointment.AppointmentTime =  $ZTIME(tApptTime,1) 
			$$$ThrowOnError(pTarget.Appointments.Insert(tAppointment))
	        &sql(FETCH C3)
	    }
	    &sql(CLOSE C3)
	    /* END TriWest Fields	*/
	    
		//Transaction History
		for i=1:1:pSource.ChildPAWaitingListTransaction.Count() {
			
			set tSourceTransaction = pSource.ChildPAWaitingListTransaction.GetAt(i)
			if '(tSourceTransaction=$$$NULLOREF) {
				set tTransHistory = ##class(CCRA.TransactionHistory).%New()
				set tTransHistory.TransactionDate = $SELECT(tSourceTransaction.TRANSDate'="":$zdt(tSourceTransaction.TRANSDate,3),1:"")
				set tTransHistory.EffectiveDate = $SELECT(tSourceTransaction.TRANSDateEffective'="":$zdt(tSourceTransaction.TRANSDateEffective,3),1:"")
				set tTransHistory.Reason = tSourceTransaction.TRANSReasonForChangeDR.WLRCDesc
				set tTransHistory.Status = tSourceTransaction.TRANSStatusDR.WLSDesc
				$$$ThrowOnError(pTarget.TransactionHistory.Insert(tTransHistory))
			}
		}
		
		//Eligibility (ES) Info
		//Instantiate ES Questionnaire - it is linked to the Referral via the RowId of the Referral.
		
		set tESQId = ""
		
		&sql(SELECT ID INTO :tESQId FROM questionnaire.QUSXXES 
				WHERE QUESPAWaitingListDR = :tReferralId)
		
		if '(tESQId = "")  {
			
			set tESQ = ##class(questionnaire.QUSXXES).%OpenId(tESQId)
					
			set tCoPaymentStatus = ##class(CCRA.ES.CoPaymentStatus).%New()
			set tCoPaymentStatus.PriorityEnrollmentGroup = tESQ.QQ1
			set tCoPaymentStatus.CurrentMeansTest = tESQ.QQ2
			set tCoPaymentStatus.AgentOrangeExposureIndicator = tESQ.QQ3
			set tCoPaymentStatus.CombatVeteranEligibilityIndicator = $SELECT(tESQ.QQ4'="":$zdt(tESQ.QQ4,3),1:"") //tESQ.QQ4  JLM 11/20/18
			set tCoPaymentStatus.CombatVeteranEndDate = $SELECT(tESQ.QQ5'="":$zdt(tESQ.QQ5,3),1:"")
			set tCoPaymentStatus.Project112SHADIndicator = tESQ.QQ6
			set tCoPaymentStatus.SouthwestAsiaIndicator = tESQ.QQ7
			set tCoPaymentStatus.IonizingRadiationIndicator = tESQ.QQ8
			set tCoPaymentStatus.CampLejeuneIndicator = tESQ.QQ9
			
			//For each service connected info - add to CoPayment Status Info
			for i=1:1:tESQ.ChildQTAB1DR.Count() {
				set tServiceConnectedInfo = ##class(CCRA.ES.ServiceConnectedInfo).%New()
				set tServiceConnectedInfo.ServiceConnected = tESQ.ChildQTAB1DR.GetAt(i).QTAB1Q12
				set tServiceConnectedInfo.ServiceConnectedPercentage = tESQ.ChildQTAB1DR.GetAt(i).QTAB1Q11
				set tServiceConnectedInfo.ServiceConnectedCondition = tESQ.ChildQTAB1DR.GetAt(i).QTAB1Q10
				$$$ThrowOnError(tCoPaymentStatus.ServiceConnectedInfo.Insert(tServiceConnectedInfo))
			}
				
			set pTarget.CoPaymentStatus = tCoPaymentStatus
			
			//Military Sexual Trauma
			set tMilitarySexualTrauma = ##class(CCRA.ES.MilitarySexualTraumaInfo).%New()
			set tMilitarySexualTrauma.DeterminationFacility = tESQ.QQ13
			set tMilitarySexualTrauma.Status = tESQ.QQ14
			set tMilitarySexualTrauma.StatusChangeDate = $SELECT(tESQ.QQ15'="":$zdt(tESQ.QQ15,3),1:"")
			set pTarget.MilitarySexualTraumaInfo = tMilitarySexualTrauma
			
			//Nose Throat Radium Info
			set tNoseThroatRadiumInfo = ##class(CCRA.ES.NoseThroatRadiumInfo).%New()
			set tNoseThroatRadiumInfo.DeterminationFacility = tESQ.QQ16
			set tNoseThroatRadiumInfo.DiagnosedWithCancer = tESQ.QQ17
			set tNoseThroatRadiumInfo.DiagnosisVerifiedDate = $SELECT(tESQ.QQ18'="":$zdt(tESQ.QQ18,3),1:"")
			set tNoseThroatRadiumInfo.FromAviatorServiceBefore1955 = tESQ.QQ19
			set tNoseThroatRadiumInfo.FromSubmarinerServiceBefore1965 = tESQ.QQ20
			set tNoseThroatRadiumInfo.ReceivingTreatment = tESQ.QQ21
			set tNoseThroatRadiumInfo.VerificationDate = $SELECT(tESQ.QQ22'="":$zdt(tESQ.QQ22,3),1:"")
			set tNoseThroatRadiumInfo.VerificationFacility = tESQ.QQ23
			set tNoseThroatRadiumInfo.VerificationMethod = tESQ.QQ24
			
			set pTarget.NoseThroatRadiumInfo = tNoseThroatRadiumInfo
		
		}
		
		//Primary SEOC - maintained in the WLPackageDR property and is a reference to a PAPersonPackage
		if '(pSource.WLPackageDR = $$$NULLOREF) {
			
			$$$ThrowOnError(..BuildPreAuthorization(pSource,pSource.WLPackageDR,.tPreAuth))
			
			if '(tPreAuth=$$$NULLOREF) $$$ThrowOnError(pTarget.PreAuthorizations.Insert(tPreAuth))
		}
		
		//Multiple Services - Maintained in the ChildPAWaitingListSecOper property which is a reference
		//to a collection of PAWaitingListSecOper objects. The SECOPPackageDR property of the PAWaitingListSecOper
		//object is the reference to the PAPersonPackage.
		
		for i=1:1:pSource.ChildPAWaitingListSecOper.Count() {
			
			$$$ThrowOnError(..BuildPreAuthorization(pSource,pSource.ChildPAWaitingListSecOper.GetAt(i).SECOPPackageDR,.tPreAuth))
			if '(tPreAuth=$$$NULLOREF) $$$ThrowOnError(pTarget.PreAuthorizations.Insert(tPreAuth))
			
		}
		
		
		
	} catch e {
		
		set tSC = e.AsStatus()
		
	}
	
	quit tSC
}

ClassMethod BuildPreAuthorization(pSource As User.PAWaitingList, ByRef pPAPersonPackage As User.PAPersonPackage, pPreAuth As CCRA.PreAuthorization) As %Status
{
	#dim tSC As %Status = $$$OK
	#dim e As %Exception.AbstractException
	#dim tPAPersonPackageDet As User.PAPersonPackageDet
	#dim tOrderSet As User.ARCOrdSets
	#dim tOrderSetDate As User.ARCOrdSetDate
	#dim tOrderSetDateItem As User.ARCOrdSetDateItem
	#dim tPreAuthPackageDet As CCRA.PreAuthorizationDetail
	#dim tPreAuthOrderSet As CCRA.PreAuthorizationOrderSet
	#dim tPreAuthOrderSetDate As CCRA.PreAuthorizationOrderSetDate
	#dim tPreAuthOrderSetDateItem As CCRA.PreAuthorizationOrderSetDateItem
	
	try {
		
		//Pre-authorizations = PA_PersonPackage (HSRM) 
		set pPreAuth = ##class(CCRA.PreAuthorization).%New()
		set pPreAuth.PreAuthorizationNumber = pPAPersonPackage.PACKNumber
		set pPreAuth.Description = pPAPersonPackage.PACKDesc
		set pPreAuth.Duration = pPAPersonPackage.PACKNumberOfDays
		set pPreAuth.PreAuthorizationReason = pPAPersonPackage.PACKReasonDR.PACKREADesc
		set pPreAuth.PreAuthorizationStatus = pPAPersonPackage.PACKStatusDR.PACKSTDesc
		set pPreAuth.DateFrom = $zdt(pPAPersonPackage.PACKEffectiveDateFrom,3)
		set pPreAuth.DateTo = $SELECT(pPAPersonPackage.PACKEffectiveDateTo'="":$zdt(pPAPersonPackage.PACKEffectiveDateTo,3),1:"")
		//MKC - 6/28/2018
		//Added Comments to referral object.
		set pPreAuth.Comments=""
		for i=1:1:pPAPersonPackage.PACKComments.Count() {
			if pPreAuth.Comments '= "" {
				set pPreAuth.Comments = pPreAuth.Comments _ $c(13,10) _ pPAPersonPackage.PACKComments.GetAt(i)
			} else {
				set pPreAuth.Comments = pPAPersonPackage.PACKComments.GetAt(i)
			}
		}
		//Pre-authorization Details = PA_PersonPackageDet (HSRM)
		//MKC - 2/14/2018 - CPT Codes are maintained in the Order Sets of the WLPackageDetail. Within the Order Set
		//there is a child collection of OrderSetsDate. It is assumed that there will never be more than one
		//OrderSetDate and that it is this OrderSetDate that contains the list of approved CPT Codes
		for tDetailCnt=1:1:pPAPersonPackage.ChildPAPersonPackageDet.Count() {
			
			set tPAPersonPackageDet = pPAPersonPackage.ChildPAPersonPackageDet.GetAt(tDetailCnt)
			
			if '(tPAPersonPackageDet= $$$NULLOREF)  {
				
				set tPreAuthPackageDet = ##class(CCRA.PreAuthorizationDetail).%New()
				set tPreAuthPackageDet.LimitQty = tPAPersonPackageDet.DETLimitQty
				set tPreAuthPackageDet.PreAuthorization = pPreAuth
				
				//PreAuthorization Order Set = ARCOrdSets (HSRM)
				set tOrderSet = tPAPersonPackageDet.DETARCOSDR
				if '(tOrderSet=$$$NULLOREF) {
					
					set tPreAuthOrderSet = ##class(CCRA.PreAuthorizationOrderSet).%New()
					set tPreAuthOrderSet.Code = tOrderSet.ARCOSCode
					set tPreAuthOrderSet.Desc = tOrderSet.ARCOSDesc
					//set tPreAuthOrderSet.PreAuthorizationDetail = tPAPersonPackageDet
					$$$ThrowOnError(tPreAuthPackageDet.OrderSets.Insert(tPreAuthOrderSet))
					
					//PreAuthorization Order Set Date = ARCOrdSetDate (HSRM)
					set tOrderSetDateCnt = tOrderSet.ChildARCOrdSetDate.Count() //MKC - 2/14/2018 *** Not Used
					set tOrderSetDateCnt = 1 
					for i=1:1:tOrderSetDateCnt {
						set tOrderSetDate = tOrderSet.ChildARCOrdSetDate.GetNext("")
						if '(tOrderSetDate=$$$NULLOREF) {
							set tPreAuthOrderSetDate = ##class(CCRA.PreAuthorizationOrderSetDate).%New()
						 	set tPreAuthOrderSetDate.DateFrom = $SELECT(tOrderSetDate.DATEDateFrom'="":$zdt(tOrderSetDate.DATEDateFrom,3),1:"")
							set tPreAuthOrderSetDate.DateTo = $SELECT(tOrderSetDate.DATEDateTo'="":$zdt(tOrderSetDate.DATEDateTo,3),1:"")
							set tPreAuthOrderSetDate.OrderSet = tPreAuthOrderSet
							set tItemKey = ""
							for j=1:1:tOrderSetDate.ChildARCOrdSetDateItem.Count() {
								set tOrderSetDateItem = tOrderSetDate.ChildARCOrdSetDateItem.GetNext(.tItemKey)
								if '(tOrderSetDateItem=$$$NULLOREF) {
									set tPreAuthOrderSetDateItem = ##class(CCRA.PreAuthorizationOrderSetDateItem).%New()
									set tPreAuthOrderSetDateItem.ProcedureCode = tOrderSetDateItem.ITMARCIMDR.ARCIMCode
									set tPreAuthOrderSetDateItem.OrderSetDate = tPreAuthOrderSetDate
								}
							}
						}
					} 
				}
			}
		}
		
	} catch e {
		
		set tSC = e.AsStatus()
		
	}
	
	quit tSC
}

}
