﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Reflection;
using VA.QA.Common.UIFramework.Selenium;
using NUnit.Framework;

namespace Quintity.TestFramework.Core
{
    /// <summary>
    /// Assert and auto create a message logging the two values on failure
    /// </summary>
    public static class TestAsserts
    {
        public static void AreEqual(bool expected, bool actual)
        {
            Assert.IsTrue(expected == actual, string.Format("{0}, Expected: {1}   Actual: {2}", MethodBase.GetCurrentMethod().Name, expected, actual));
        }

        public static void AreEqual(string expected, string actual)
        {
            Assert.IsTrue(expected == actual, string.Format("{0}, Expected: {1}   Actual: {2}", MethodBase.GetCurrentMethod().Name, expected, actual));
        }

        public static void AreNotEqual(string expected, string actual)
        {
            Assert.IsTrue(expected != actual, string.Format("{0}, Expected: {1}   Actual: {2}", MethodBase.GetCurrentMethod().Name, expected, actual));
        }

        public static void AreEqual(int expected, int actual)
        {
            Assert.IsTrue(expected == actual, string.Format("{0}, Expected: {1}   Actual: {2}", MethodBase.GetCurrentMethod().Name, expected, actual));
        }

        public static void AreNotEqual(int expected, int actual)
        {
            Assert.IsTrue(expected != actual, string.Format("{0}, Expected: {1}   Actual: {2}", MethodBase.GetCurrentMethod().Name, expected, actual));
        }

        public static void AreNotEqual(bool expected, bool actual)
        {
            Assert.IsTrue(expected != actual, string.Format("{0}, Expected: {1}   Actual: {2}", MethodBase.GetCurrentMethod().Name, expected, actual));
        }

        public static void NotNull(object value)
        {
            Assert.IsNotNull(value, string.Format("{0}, Expected: {1}   Actual: {2}", MethodBase.GetCurrentMethod().Name, null, value));
        }

        public static void WaitForTrue(Func<bool> condition, int maxWaitTimeInSeconds)
        {
            var actual = false;
            var expected = true;
            var timeSpan = new TimeSpan(0, 0, maxWaitTimeInSeconds);
            var sw = new Stopwatch();
            sw.Start();
            while (true)
            {
                actual = condition.Invoke();
                if (sw.Elapsed.Seconds >= maxWaitTimeInSeconds)
                    break;
                System.Threading.Thread.Sleep(500);
            }
            sw.Stop();
            Assert.IsTrue(expected == actual, string.Format("{0}, Expected: {1}   Actual: {2}", MethodBase.GetCurrentMethod().Name, expected, actual));
        }

        public static void IsPageLoaded(IPage page)
        {
            Assert.IsTrue(page.IsPageLoaded(), string.Format("Page not loaded, Page name: {0}", page.ToString()));
        }

        public static void Contains<T>(T expected, IEnumerable<T> collection)
        {
            foreach (var item in collection)
                if (expected.Equals(item))
                    return;
            Assert.Fail(string.Format("{0}, Expected: {1}, Collection values: {2}", MethodBase.GetCurrentMethod().Name, expected, string.Join(", ", collection)));
        }
    }
}
