﻿using VA.QA.Common.UIFramework.Selenium.Config;
using OpenQA.Selenium;
using OpenQA.Selenium.Chrome;
using OpenQA.Selenium.Firefox;
using OpenQA.Selenium.IE;
using OpenQA.Selenium.Remote;
using System;
using System.Diagnostics;

namespace VA.QA.Common.UIFramework.Selenium
{
    public class EnvironmentManager
    {
        private static EnvironmentManager instance = new EnvironmentManager();
        private Driver browser;
        private IWebDriver driver;

        public static EnvironmentManager Instance
        {
            get
            {
                if (instance == null)
                    instance = new EnvironmentManager();
                return instance;
            }
        }
        public Driver Browser { get { return browser; } }


        private EnvironmentManager()
        {
        }

        ~EnvironmentManager()
        {
            if (driver != null)
                driver.Quit();
        }

        public IWebDriver CreateDriverInstance()
        {

            if (TestRunConfig.Driver == Driver.Remote)
                Trace.TraceInformation("Createing '{0}' driver, {1} browser", TestRunConfig.Driver, TestRunConfig.remoteBrowserName);
            else
                Trace.TraceInformation("Createing '{0}' driver", TestRunConfig.Driver);

            switch (TestRunConfig.Driver)
            {
                case Driver.Chrome:
                    var cOptions = new ChromeOptions();

                    cOptions.AddAdditionalCapability("useAutomationExtension", false);
                    //cOptions.AddArguments(" --start-maximized");
                    //cOptions.AddArguments("--disable-extensions");
                    //cOptions.AddArguments("--always-authorize-plugins");
                    //cOptions.AddArguments("--disable-popup-blocking");


                    driver = new ChromeDriver(cOptions);
                // driver = new ChromeDriver();
                    break;

                //case Selenium.Browser.Chrome:
                //    var cOptions = new ChromeOptions();
                //    cOptions.AddArguments("start-maximized");
                //    capabilities = DesiredCapabilities.Chrome();
                //    capabilities.SetCapability(ChromeOptions.Capability, cOptions);
                //    break;


                case Driver.Firefox:
                    var fOptions = new FirefoxOptions();
                    fOptions.BrowserExecutableLocation = @"C:\Program Files\Mozilla Firefox\firefox.exe";
                    driver = new FirefoxDriver(fOptions);
                    break;
                //case Driver.IE:
                //    driver = new InternetExplorerDriver();
                //    break;
                //case Driver.Remote:
                //    driver = ConfigureRemoteDriver();
                //    break;
                default:
                    throw new NotImplementedException();
            }
            driver.Manage().Timeouts().ImplicitWait = new TimeSpan(0, 0, SeleniumConfiguration.ImplicitWaitSeconds);


            return driver;
        }


        //public IWebDriver CreateDriverInstance()
        //{
        //    //var fOptions = new FirefoxOptions();
        //    //fOptions.BrowserExecutableLocation = @"C:\Program Files\Mozilla Firefox\firefox.exe";
        //    //driver = new FirefoxDriver(fOptions);      
        //    //driver.Manage().Timeouts().ImplicitWait = new TimeSpan(0, 0, SeleniumConfiguration.ImplicitWaitSeconds);
        //    //return driver;
        //    var fOptions = new ChromeOptions();
        //    fOptions.AddAdditionalCapability("useAutomationExtension", false);
        //    //fOptions.AddArgument("--disable-extensions");
        //    //fOptions.BrowserExecutableLocation = @"C:\Program Files\Mozilla Firefox\firefox.exe";
        //    driver = new ChromeDriver(fOptions);
        //    driver.Manage().Timeouts().ImplicitWait = new TimeSpan(0, 0, SeleniumConfiguration.ImplicitWaitSeconds);
        //    return driver;

        //}



        //private IWebDriver ConfigureRemoteDriver()
        //{
        //    DesiredCapabilities capabilities = null;
        //    switch (TestRunConfig.RemoteBrowserName)
        //    {
        //        case Selenium.Browser.Chrome:
        //            capabilities = DesiredCapabilities.Chrome();
        //            break;
        //        case Selenium.Browser.Firefox:
        //            capabilities = DesiredCapabilities.Firefox();
        //            break;
        //        case Selenium.Browser.HtmlUnit:
        //            if (TestRunConfig.IsJavaScriptEnabled)
        //                capabilities = DesiredCapabilities.HtmlUnitWithJavaScript();
        //            else
        //                capabilities = DesiredCapabilities.HtmlUnit();
        //            break;
        //        case Selenium.Browser.IE:
        //            capabilities = DesiredCapabilities.InternetExplorer();
        //            break;
        //        default:
        //            throw new NotImplementedException();
        //    }

        //   // capabilities.IsJavaScriptEnabled = TestRunConfig.IsJavaScriptEnabled;
        //    capabilities.SetCapability(CapabilityType.AcceptSslCertificates, TestRunConfig.AcceptSslCertificates);
        //    capabilities.SetCapability(CapabilityType.HasNativeEvents, true);
        //    capabilities.SetCapability(CapabilityType.HandlesAlerts, false);
        //    capabilities.SetCapability(CapabilityType.UnexpectedAlertBehavior, "accept");
        //    capabilities.SetCapability(CapabilityType.SupportsApplicationCache, false);
        //    capabilities.SetCapability("applicationCacheEnabled", false);

        //    //capabilities.SetCapability("CommandTimeout", config.SeleniumServer.CommandTimeout);
        //    //capabilities.SetCapability(ChromeOptions.Capability, options);
        //    //driver.Manage().Timeouts().SetPageLoadTimeout(new TimeSpan(0, 0, SeleniumConfiguration.PageLoadTimeoutSeconds));
        //    //driver.Manage().Timeouts().SetScriptTimeout(new TimeSpan(0, 0, SeleniumConfiguration.ScriptTimeoutSeconds));
        //    return new RemoteWebDriver(TestRunConfig.Address, capabilities, TestRunConfig.CommandTimeoutTimeSpan);
            
        //}

        public void CloseCurrentDriver()
        {
            
            if (driver != null)
            {
                if (driver.WaitUntilAlertExists(0))
                {
                    Trace.TraceInformation("Alert present, alert text: {0}", driver.Alert().Text);
                    driver.Alert().Dismiss();
                }
                driver.Close();
                driver.Quit();
            }
            driver = null;
            
        }
    }
}