﻿using System;

namespace OpenQA.Selenium
{
    internal class ExpectedCondition
    {
        public static Func<ISearchContext, IWebElement> ElementExists(By locator)
        {
            return (iSearchContext) =>
            {
                try
                {
                    return iSearchContext.FindElement(locator);
                }
                catch (NoSuchElementException)
                {
                    return null;
                }
            };
        }

        public static Func<ISearchContext, bool> ElementNotExists(By locator)
        {
            return (iSearchContext) =>
            {
                try
                {
                    return !iSearchContext.ElementExists(locator);
                }
                catch (StaleElementReferenceException)
                {
                    return false;
                }
            };
        }

        public static Func<ISearchContext, IWebElement> ElementVisible(By locator)
        {
            return (iSearchContext) =>
            {
                try
                {
                    return ElementIfVisable(iSearchContext.FindElement(locator));
                }
                catch (StaleElementReferenceException)
                {
                    return null;
                }
                catch (NoSuchElementException)
                {
                    return null;
                }
            };
        }

        public static Func<ISearchContext, bool> ElementTestEquals(By locator, string text)
        {
            return (iSearhContext) => { return iSearhContext.FindElement(locator).Text == text; };
        }

        public static Func<IWebDriver, bool> AlertTextEquals(string text)
        {
            return (IWebDriver) =>
            {
                try
                {
                    IAlert alert = IWebDriver.SwitchTo().Alert();
                    if (alert != null)
                        return (text == alert.Text);
                }
                catch (NoAlertPresentException)
                { }
                return false;
            };
        }

        public static Func<IWebDriver, bool> AlertTextContains(string text)
        {
            return (IWebDriver) =>
            {
                try
                {
                    IAlert alert = IWebDriver.SwitchTo().Alert();
                    if (alert != null)
                        return (alert.Text.Contains(text));
                }
                catch (NoAlertPresentException)
                { }
                return false;
            };
        }

        private static IWebElement ElementIfVisable(IWebElement iWebElement)
        {
            return iWebElement.Displayed ? iWebElement : null;
        }
    }
}