﻿using VA.QA.Common.UIFramework.Selenium;
using System;

namespace OpenQA.Selenium
{
    public static class SearchContextExtension
    {
        public static bool ElementExists(this ISearchContext iSearchContext, By locator)
        {
            return (iSearchContext.FindElements(locator).Count > 0);
        }

        public static bool WaitUntil<T>(this ISearchContext iSearchContext, Func<ISearchContext, T> condition, int maxWaitTimeInSeconds = GlobalConstants.DefaultMaxWaitInSeconds)
        {
            var searchContext = (ISearchContext)iSearchContext;
            var wait = new SeachContextWait(searchContext, TimeSpan.FromSeconds(maxWaitTimeInSeconds));
            try
            {
                wait.Until(condition);
            }
            catch (WebDriverTimeoutException)
            {
                return false;
            }
            return true;
        }

        public static bool WaitUntilExists(this ISearchContext iSearchContext, By locator, int maxWaitTimeInSeconds = GlobalConstants.DefaultMaxWaitInSeconds)
        {
            return iSearchContext.WaitUntil(ExpectedCondition.ElementExists(locator), maxWaitTimeInSeconds);
        }
        public static bool WaitUntilNotExists(this ISearchContext iSearchContext, By locator, int maxWaitTimeInSeconds = GlobalConstants.DefaultMaxWaitInSeconds)
        {
            return iSearchContext.WaitUntil(ExpectedCondition.ElementNotExists(locator), maxWaitTimeInSeconds);
        }

        public static bool WaitUntilVisible(this ISearchContext iSearchContext, By locator, int maxWaitTimeInSeconds = GlobalConstants.DefaultMaxWaitInSeconds)
        {
            return iSearchContext.WaitUntil(ExpectedCondition.ElementVisible(locator), maxWaitTimeInSeconds);
        }

        public static bool WaitUntilTextEquals(this ISearchContext iSearchContext, By locator, string text, int maxWaitTimeInSeconds = GlobalConstants.DefaultMaxWaitInSeconds)
        {
            return iSearchContext.WaitUntil(ExpectedCondition.ElementTestEquals(locator, text), maxWaitTimeInSeconds);
        }
    }
}
