﻿using VA.QA.Common.UIFramework.Selenium;
using OpenQA.Selenium.Support.UI;
using System;
using System.Drawing.Imaging;
using System.Threading;

namespace OpenQA.Selenium
{
    public static class WebDriverExtension
    {
        public static void TakeScreenShot(this IWebDriver iWebDirver, string fileName, OpenQA.Selenium.ScreenshotImageFormat imageFormat)
        {
            var driver = (ITakesScreenshot)iWebDirver;
            var screenShot = driver.GetScreenshot();
            screenShot.SaveAsFile(fileName, imageFormat);
        }

        public static IWebElement FindElementWithStaleCheck(this IWebDriver iWebDriver, By selector)
        {
            try
            {
                return iWebDriver.FindElement(selector);
            }
            catch (StaleElementReferenceException stale)
            {
                var loops = 5;
                var cur = 0;
                while (cur <= loops)
                {
                    var exists = iWebDriver.ElementExists(selector);
                    if(exists)
                    {
                        return iWebDriver.FindElement(selector);
                    }
                    Thread.Sleep(100);
                }
                throw stale;
            }
        }
        public static bool DriverWaitUntil<T>(this IWebDriver iWebDriver, Func<IWebDriver, T> condition, int maxWaitTimeInSeconds = GlobalConstants.DefaultMaxWaitInSeconds)
        {
            var driver = (IWebDriver)iWebDriver;
            var wait = new WebDriverWait(driver, TimeSpan.FromSeconds(maxWaitTimeInSeconds));
            try
            {
                wait.Until(condition);
            }
            catch (WebDriverTimeoutException)
            {
                return false;
            }
            return true;
        }

        public static bool WaitUntilAlertExists(this IWebDriver iWebDriver, int maxWaitTimeInSeconds = GlobalConstants.DefaultMaxWaitInSeconds)
        {
            return iWebDriver.DriverWaitUntil(ExpectedConditions.AlertIsPresent(), maxWaitTimeInSeconds);
        }

        public static bool WaitUntilAlertTextEquals(this IWebDriver iWebDriver, string text, int maxWaitTimeInSeconds = GlobalConstants.DefaultMaxWaitInSeconds)
        {
            return iWebDriver.DriverWaitUntil(ExpectedCondition.AlertTextEquals(text), maxWaitTimeInSeconds);
        }

        public static bool WaitUntilAlertTextContains(this IWebDriver iWebDriver, string text, int maxWaitTimeInSeconds = GlobalConstants.DefaultMaxWaitInSeconds)
        {
            return iWebDriver.DriverWaitUntil(ExpectedCondition.AlertTextContains(text), maxWaitTimeInSeconds);
        }

        public static bool WaitUntilPost(this IWebDriver iWebDriver, Action postAction, int maxWaitTimeInSeconds = GlobalConstants.DefaultMaxWaitInSeconds)
        {
            IJavaScriptExecutor executeScrip = iWebDriver as IJavaScriptExecutor;
            executeScrip.ExecuteScript("$('body').add('<div id=\"waitForMe\"></div>')");
            postAction.Invoke();
            return iWebDriver.WaitUntilNotExists(By.Id("waitForMe"), maxWaitTimeInSeconds);
        }

        public static bool WaitUntillTextContains(this IWebDriver iWebDriver, By locator, string text, int maxWaitTimeInSeconds = GlobalConstants.DefaultMaxWaitInSeconds)
        {
            return iWebDriver.DriverWaitUntil(ExpectedConditions.TextToBePresentInElement(iWebDriver.FindElement(locator), text), maxWaitTimeInSeconds);
        }

        public static IAlert Alert(this IWebDriver iWebDriver)
        {
            return iWebDriver.SwitchTo().Alert();
        }

        public static IJavaScriptExecutor Scripts(this IWebDriver iWebDriver)
        {
            return (IJavaScriptExecutor)iWebDriver;
        }
                
        public static IWebElement MoveToElement(this IWebDriver iWebDriver, By locator)
        {
            var element = iWebDriver.FindElement(locator);
            MoveToElement(iWebDriver, iWebDriver.FindElement(locator));
            return element;
        }

        public static void MoveToElement(this IWebDriver iWebDriver, IWebElement iWebElement)
        {
            ILocatable loc = (ILocatable)iWebElement;
            IMouse mouse = ((IHasInputDevices)iWebDriver).Mouse;
            mouse.MouseMove(loc.Coordinates);
            // this function works for non-window scroll bar // this dosent iWebDriver.Scripts().ExecuteScript(string.Format("javascript:window.scrollTo('{0}','{1}')", iWebElement.Location.X.ToString(), iWebElement.Location.Y.ToString()));
        }

        public static void MoveToElementJavaScript(this IWebDriver iWebDriver, By locator)
        {
            var element = iWebDriver.FindElement(locator);
            iWebDriver.Scripts().ExecuteScript(string.Format("javascript:window.scrollTo('{0}','{1}')", element.Location.X.ToString(), element.Location.Y.ToString()));
        }

        public static IWebDriver SwitchBrowserWindow<T>(this IWebDriver iWebDriver, Func<ISearchContext,T> condition)
        {
            var currentWindowHandle = iWebDriver.CurrentWindowHandle;
            IWebDriver newWindowDriver = null;
            var windowIterator = iWebDriver.WindowHandles;
            foreach(var window in windowIterator)
            {
                var handel = window;
                newWindowDriver = iWebDriver.SwitchTo().Window(window);
                if (newWindowDriver.WaitUntil(condition, 2))
                    return newWindowDriver;
            }
            return null;
        }
    }
}
